#ifndef __TY_DSP_ADAPTER_H__
#define __TY_DSP_ADAPTER_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "tuya_ipc_media.h"
#include "ty_sys_av.h"

/**
 * @brief
 *
 */
#define E_DSP_HC_TARGET_MAX 3
/**
 * @brief
 *
 */
#define E_DSP_MD_TARGET_MAX 3

/**
 * @brief 视频流类型
 *
 */
typedef enum
{
    E_DSP_MEDIA_VIDEO_REQUEST_IDR = 0,    ///< 在最近的编码，请求IDR帧编码
    E_DSP_MEDIA_VIDEO_FLUSH_STREAM,       ///< 刷掉编码器里残留的旧码流，并以IDR帧开始编码
    E_DSP_MEDIA_VIDEO_IDR_TYPE_MAX,
} E_DSP_MEDIA_VIDEO_SET_IDR_TYPE;

/**
 * @brief 音频设置
 *
 */
typedef enum
{
    E_DSP_SET_AUDIO_SPEAKER_START,     ///< spk开启
    E_DSP_SET_AUDIO_SPEAKER_STOP,      ///< spk关闭
    E_DSP_SET_AUDIO_SPEAKER_VOLUME,    ///< spk音量
    E_DSP_SET_AUDIO_MIC_VOLUME,        ///< mic音量
    E_DSP_SET_AUDIO_MAX,
} E_DSP_SET_AUDIO;

/**
 * @brief
 *
 */
typedef enum
{
    E_DSP_GET_VIDEO_IDR_UTC,    ///< 上一帧I帧获取的utc时间
    E_DSP_GET_VIDEO_MAX,        ///<
} E_DSP_GET_VIDEO;

/**
 * @brief
 *
 */
typedef enum
{
    E_DSP_AUDIO_SOURCE_TYPE_FILE = 0,        ///< 本地文件播放
    E_DSP_AUDIO_SOURCE_TYPE_P2P = 1,         ///< p2p流播放
    E_DSP_AUDIO_SOURCE_TYPE_WEBRTC = 2,      ///< webRTC流播放
    E_DSP_AUDIO_SOURCE_TYPE_STREAMER = 3,    ///< 推流网关，如Chromecast */
} E_DSP_AUDIO_SOURCE_TYPE;                   ///< 播放的音频来源 */

/**
 * @brief 本地播放的文件编码格式
 *
 */
typedef enum
{
    E_DSP_AUDIO_FILE_PCM,      ///< pcm文件
    E_DSP_AUDIO_FILE_WAV,      ///< wav文件
    E_DSP_AUDIO_FILE_G711U,    ///< G711U文件
    E_DSP_AUDIO_FILE_G711A,    ///< G711A文件
    E_DSP_AUDIO_FILE_TYPE_MAX,
} E_DSP_AUDIO_FILE_TYPE;

/**
 * @brief 智能检测类型
 *
 */
typedef enum
{
    E_DSP_DETECT_TYPE_MD,    ///< 动检
    E_DSP_DETECT_TYPE_HD,    ///< 人行
    E_DSP_DETECT_TYPE_DB,    ///< 声音
    E_DSP_DETECT_TYPE_MAX,
} E_DSP_DETECT_TYPE;

/**
 * @brief 归一化数据
 *
 */
typedef struct
{
    float x;         ///<
    float y;         ///<
    float width;     ///<
    float height;    ///<
} E_DSP_RECT;

/**
 * @brief 目标
 *
 */
typedef struct
{
    E_DSP_RECT drawRect;    ///< 坐标
    int score;              ///< 评分
} E_DSP_TARGET;

/**
 * @brief 动检结果
 *
 */
typedef struct
{
    int count;                                   ///<
    E_DSP_TARGET target[E_DSP_MD_TARGET_MAX];    ///<
} E_DSP_MD_DETECT_RESULT;

/**
 * @brief DB结果
 *
 */
typedef struct
{
    int pcm_db;    ///< DB值
} E_DSP_DB_DETECT_RESULT;

/**
 * @brief 智能检测类型
 *
 */
typedef enum
{
    E_DSP_DETECT_PARAM_ENABLE = 0,     ///< 使能开关
    E_DSP_DETECT_PARAM_SENSITIVITY,    ///< 灵敏度
    E_DSP_DETECT_PARAM_MAX,
} E_DSP_DETECT_PARAM_TYPE;

/**
 * @brief 人形参数
 *
 */
typedef struct
{
    E_DSP_DETECT_PARAM_TYPE type;    ///<
    int enable;                      ///<
    int sensitivity;                 ///<
} E_DSP_HD_DETECT_PARAM;

/**
 * @brief dsp 初始化函数
 *
 * @param pinfo Input，S_DSP_MEDIA_INFO 音视频的初始化参数
 * @return int <0 失败，=0 成功
 */
int ty_dsp_init(S_DSP_MEDIA_INFO *pinfo);

/**
 * @brief 播放文件接口
 *
 * @param fileName 文件路径
 * @param fileType 文件格式类型
 * @param count 播放次数，0，代表一直循环播放
 * @param timeOut 播放阻塞时间单位，ms
 * @return int <0 失败，=0 成功
 */
int ty_dsp_audio_play_file(const char *fileName, E_DSP_AUDIO_FILE_TYPE fileType, int count, int timeOut);

/**
 * @brief 设置播放时长s,音频文件持续播放模式时有效
 *
 * @param duration_time Input，int duration_time, 设置持续播放时长，单位s
 * @return int <0 失败，=0 成功
 */
int ty_dsp_audio_set_play_duration(int duration_time);

/**
 * @brief 停止播放文件
 *
 * @return int <0 失败，=0 成功
 */
int ty_dsp_audio_stop_play_file(void);

/**
 * @brief 获取文件播放状态
 *
 * @return int 0未播放文件，1正在播放文件
 */
int ty_dsp_audio_get_play_file_status(void);

int ty_dsp_audio_get(E_DSP_MEDIA_AUDIO_TYPE type, TAL_AUDIO_FRAME_INFO_T *frame);

/**
 * @brief 音频播放开始,使能ao 通道以及apk对应的引脚
 *
 * @param card card number
 * @param chn channel number
 * @param handle handle of start
 * @return int <0 失败，=0 成功
 */
int ty_dsp_ao_start(int card, int chn, VOID *handle);

/**
 * @brief 关闭ao, 关闭通道以及apk对应的引脚
 *
 * @param card card number
 * @param chn channel number
 * @param handle handle of start
 * @return int <0 失败，=0 成功
 */
int ty_dsp_ao_stop(int card, int chn, VOID *handle);

/**
 * @brief dsp去初始化
 *
 * @return int <0 失败，=0 成功
 */
int ty_dsp_uninit(void);

#ifdef __cplusplus
}    // extern "C"
#endif

#endif
