#ifndef __TUYA_LOG_H__
#define __TUYA_LOG_H__

#include <sys/syscall.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

enum LOGLEVEL {
    TY_LOG_DEBUG,
    TY_LOG_INFO,
    TY_LOG_NOTICE,
    TY_LOG_WARN,
    TY_LOG_ERROR,
    TY_LOG_CRIT,
    TY_LOG_ALERT,
    TY_LOG_EMERG
};

/*
显示：0(默认)、1(粗体/高亮)、22(非粗体)、4(单条下划线)、24(无下划线)、5(闪烁)、25(无闪烁)、7(反显、翻转前景色和背景色)、27(无反显)
颜色值：0(黑)、1(红)、2(绿)、 3(黄)、4(蓝)、5(紫)、6(深绿)、7(白)
颜色分为背景色和字体色，30~39（30+颜色值）用来设置字体色（前景色），40~49（40+颜色值）设置背景（背景色）：如31表示前景色为红色，41表示背景色为红色。
*/
#define LOG_COLOR_NONE                  "\033[0;m"  
#define LOG_RED                         "\033[0;31m"
#define LOG_LIGHT_RED                   "\033[1;31m"  //红色高亮
#define LOG_LIGHT_RED_INV               "\033[5;7;31m"  //红色高亮，并反白显示，字体闪烁
#define LOG_GREEN                       "\033[0;32m"
#define LOG_LIGHT_GREEN                 "\033[1;32m"
#define LOG_BLUE                        "\033[0;34m"
#define LOG_LIGHT_BLUE                  "\033[1;34m" //蓝色高亮
#define LOG_DARY_GRAY                   "\033[1;30m"
#define LOG_CYAN                        "\033[0;36m"
#define LOG_LIGHT_CYAN                  "\033[1;36m"
#define LOG_PURPLE                      "\033[0;35m"
#define LOG_LIGHT_PURPLE                "\033[1;35m"
#define LOG_YELLOW                      "\033[0;33m"
#define LOG_LIGHT_YELLOW                "\033[1;33m" //黄色高亮
#define LOG_WHITE                       "\033[0;37m"
#define LOG_LIGHT_WHITE                 "\033[1;37m" //白色高亮


void ty_log_init(const char *log_file, enum LOGLEVEL level);

void ty_log_uninit(void);

int ty_log_check_level(enum LOGLEVEL level);

void ty_log_print(enum LOGLEVEL level, const char *prefix, const char *fmt, ...);
void ty_log_print_col(enum LOGLEVEL level,char *col, const char *prefix, const char *fmt, ...);

#define LOG_BUFFER_LEN  256
#define __FILENAME__ (strrchr(__FILE__, '/') ? (strrchr(__FILE__, '/') + 1):__FILE__)

#define TUYA_LOG_PRINT(level, fmt, args...) \
    do {    \
        if(ty_log_check_level(level) == 0)  \
            break;  \
        char tylogbuf[LOG_BUFFER_LEN] = {0};    \
        snprintf(tylogbuf, LOG_BUFFER_LEN, "tid(%lu) %s %s(%d)", \
                    syscall(__NR_gettid), __FILENAME__, __FUNCTION__, __LINE__);    \
        ty_log_print(level, tylogbuf, fmt, ##args); \
    } while(0)


#define TUYA_LOG_COL_PRINT(level,col,fmt, args...)  \
    do {    \
        if(ty_log_check_level(level) == 0)  \
            break;  \
        char tylogbuf[LOG_BUFFER_LEN] = {0};    \
        snprintf(tylogbuf, LOG_BUFFER_LEN, " %s %s(%d)", \
                     __FILENAME__, __FUNCTION__, __LINE__);\
        ty_log_print_col(level, col,tylogbuf, fmt LOG_COLOR_NONE, ##args);  \
    } while(0)
    

#if 0
#define TYDEBUG(fmt, args...)       TUYA_LOG_PRINT(TY_LOG_DEBUG, fmt, ##args)
#define TYINFO(fmt, args...)        TUYA_LOG_PRINT(TY_LOG_INFO, fmt, ##args)
#define TYNOTICE(fmt, args...)      TUYA_LOG_PRINT(TY_LOG_NOTICE, fmt, ##args)
#define TYWARN(fmt, args...)        TUYA_LOG_PRINT(TY_LOG_WARN, fmt, ##args)
#define TYERROR(fmt, args...)       TUYA_LOG_PRINT(TY_LOG_ERROR, fmt, ##args)
#define TYCRIT(fmt, args...)        TUYA_LOG_PRINT(TY_LOG_CRIT, fmt, ##args)
#define TYALERT(fmt, args...)       TUYA_LOG_PRINT(TY_LOG_ALERT, fmt, ##args)
#define TYEMERG(fmt, args...)       TUYA_LOG_PRINT(TY_LOG_EMERG, fmt, ##args)
#else
#define TYDEBUG(fmt, args...)       TUYA_LOG_COL_PRINT(TY_LOG_DEBUG,LOG_LIGHT_YELLOW,fmt, ##args)
#define TYINFO(fmt, args...)        TUYA_LOG_COL_PRINT(TY_LOG_INFO,LOG_GREEN,fmt, ##args)
#define TYNOTICE(fmt, args...)      TUYA_LOG_COL_PRINT(TY_LOG_NOTICE,LOG_BLUE,fmt, ##args)
#define TYWARN(fmt, args...)        TUYA_LOG_COL_PRINT(TY_LOG_WARN,LOG_YELLOW,fmt, ##args)
#define TYERROR(fmt, args...)       TUYA_LOG_COL_PRINT(TY_LOG_ERROR,LOG_RED,fmt, ##args)
#define TYCRIT(fmt, args...)        TUYA_LOG_COL_PRINT(TY_LOG_CRIT,LOG_LIGHT_RED,fmt, ##args)
#define TYALERT(fmt, args...)       TUYA_LOG_COL_PRINT(TY_LOG_ALERT,LOG_LIGHT_RED_INV,fmt, ##args)
#define TYEMERG(fmt, args...)       TUYA_LOG_COL_PRINT(TY_LOG_EMERG,LOG_LIGHT_BLUE,fmt, ##args)
#endif
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif   // __TUYA_LOG_H__
