#include <pthread.h>
#include <semaphore.h>
#include <stdio.h>
#include <sys/prctl.h>
//#include "ty_dev_common.h"
//#include "tal_gpio.h"
//#include "tuya_g711_utils.h"
#include "ty_dsp_adapter.h"
#include "ty_dsp_adapter_wav.h"
#include "ty_log.h"
//#include "ty_sys_panel.h"
#include "ty_time.h"

#define DSP_AUDIO_FRAME_LEN         1280
#define DSP_AUDIO_THREAD_STACK_SIZE 0x100000
#define TUYA_G711_A_LAW             0
#define TUYA_G711_MU_LAW            1

typedef struct {
    int isInit;                     /* 音频初始化标志 */
    int start;                      /* 播放状态,0，未播放，1，播放中 */
    pthread_t stid;                 /* 播放线程pid   */
    char filename[128];             /* 播放的文件名，包含路径 */
    E_DSP_AUDIO_FILE_TYPE fileType; /* 文件类型    */
    int count;                      /* 播放次数，0，代表循环播放 */
    int fileStop;                   /* 文件播放停止    */
    int fileDurationTime;           /* 文件播放时长    */
    unsigned int audioStartType;    /* 音频打开者,采用移位保存状态 */
    int spkRequestCount;            /* 记录请求开启spk的数量*/
    sem_t spkSem;                   /* 多个线程对spkRequestCount操作时的保护*/
} ty_dsp_audio_handle_s;

typedef struct {
    uint64_t utc[E_DSP_MEDIA_VIDEO_MAX]; /* 记录上一帧I帧的utc时间*/
} ty_dsp_video_handle_s;

typedef struct {
    S_DSP_MEDIA_INFO *pstrDspInfo;     /* 媒体属性配置 */
    ty_dsp_audio_handle_s audioHandle; /* 音频属性 */
    ty_dsp_video_handle_s videoHandle; /* 视频属性 */
} TY_DSP_ADAPTER_S;

static TY_DSP_ADAPTER_S gdspAdapter = {0};

/***********************************************************
 *  函数名: ty_dsp_init
 *  功能：dsp 初始化函数
 *  参数: Input，S_DSP_MEDIA_INFO *pinfo，音视频的初始化参数
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
int ty_dsp_init(S_DSP_MEDIA_INFO *pinfo)
{
    int ret = -1;
    int i = 0;
    // ty_sys_gpio spk_gpio = { 0 };
    TUYA_CODEC_ID codectype = 0;

    if (pinfo == NULL) {
        TYERROR("ty_dsp_init pinfo is NULL\n");
        return -1;
    }

    for (i = 0; i < TY_DSP_MEDIA_AUDIO_MAX; i++) {
        if (pinfo->audio[i].enable) {
            codectype = pinfo->audio[i].codectype;
            if (((int)pinfo->audio[i].sample != TUYA_AUDIO_SAMPLE_8K) &&
                ((codectype == TUYA_CODEC_AUDIO_G711U) || (codectype == TUYA_CODEC_AUDIO_G711A))) {
                TYERROR("Not support:codetype:%d sample:%d\n", codectype, pinfo->audio[i].sample);
                return -1;
            }
        }
    }

    gdspAdapter.pstrDspInfo = pinfo;

    ret = tal_vi_init(pinfo->vi, 1);
    if (0 != ret) {
        TYERROR("ty_dsp_init failed,%d\n", ret);
        return -1;
    }

    ret = tal_venc_init(0, pinfo->venc, TY_DSP_MEDIA_VENC_MAX + TY_DSP_MEDIA_YUV_MAX);
    if (0 != ret) {
        TYERROR("ty_dsp_init failed,%d\n", ret);
        return -1;
    }

    ret = tal_ai_init(pinfo->audio, TY_DSP_MEDIA_AUDIO_MAX);
    if (0 != ret) {
        TYERROR("tal_ai_init failed,%d\n", ret);
        return -1;
    }

    // for (i = 0; i < E_DSP_MEDIA_VIDEO_MAX; i ++)
    // {
    //     if (pinfo->video[i].enable)
    //     {
    //         ret = ty_dev_venc_start(0, i);
    //         if (0 != ret)
    //         {
    //             TYERROR("ty_dev_venc_start failed,%d\n", ret);
    //             return -1;
    //         }
    //     }
    // }

    ret = tal_ao_init(pinfo->audio, TY_DSP_MEDIA_AUDIO_MAX, NULL);
    if (0 != ret) {
        TYERROR("tal_ao_init failed,%d\n", ret);
        return -1;
    }

    for (i = 0; i < TY_DSP_MEDIA_AUDIO_MAX; i++) {
        if (pinfo->audio[i].enable) {
            ret = tal_ai_start(0, i);
            if (0 != ret) {
                TYERROR("ty_dev_ai_start failed,%d\n", ret);
                return -1;
            }

            /* 喇叭功放引脚初始化 */
            // if (pinfo->audio[i].spk_gpio >= 0) {
            //     spk_gpio.dir = OUTPUT;
            //     spk_gpio.default_val = (pinfo->audio[i].spk_gpio_polarity == 0) ? 0 : 1;
            //     spk_gpio.gpio_num = pinfo->audio[i].spk_gpio;
            //     spk_gpio.mode = COMMON_MODE;
            //     ret = ty_sys_gpio_init(&spk_gpio);
            //     if (0 != ret) {
            //         TYERROR("tycam_gpio_init failed, %d\n", ret);
            //         return -1;
            //     }
            // }
        }
    }
    gdspAdapter.audioHandle.spkRequestCount = 0;     //初始化时 spk为关闭请求数量为0
    sem_init(&gdspAdapter.audioHandle.spkSem, 0, 1); //初始化 信号量
    gdspAdapter.audioHandle.isInit = 1;

    return 0;
}

int ty_dsp_audio_get(E_DSP_MEDIA_AUDIO_TYPE type, TAL_AUDIO_FRAME_INFO_T *frame)
{
    int ret = -1;
    char buf[DSP_AUDIO_FRAME_LEN] = {0};
    char *tmpBuf = NULL;
    int bufSize = 0;
    int outLen = 0;
    TUYA_CODEC_ID codectype = gdspAdapter.pstrDspInfo->audio[type].codectype;

    if (frame == NULL || frame->pbuf == NULL) {
        TYERROR("input is NULL\n");
        return -1;
    }

    switch (codectype) {
    case TUYA_CODEC_AUDIO_PCM:
        ret = tal_ai_get_frame(0, type, frame);
        if (0 != ret) {
            // TYERROR("get frame failed type:%d\n", type);
            return -1;
        }
        break;

    case TUYA_CODEC_AUDIO_G711U:
    case TUYA_CODEC_AUDIO_G711A:
        if (frame->buf_size < DSP_AUDIO_FRAME_LEN / 2) {
            TYERROR("bufSize:%d < %d, not enough\n", frame->buf_size, DSP_AUDIO_FRAME_LEN / 2);
            return -1;
        }
        tmpBuf = frame->pbuf;
        bufSize = frame->buf_size;
        frame->buf_size = DSP_AUDIO_FRAME_LEN;
        frame->pbuf = buf;
        ret = tal_ai_get_frame(0, type, frame);
        if (0 != ret) {
            frame->pbuf = tmpBuf;
            frame->buf_size = bufSize;
            return -1;
        }
        if (codectype == TUYA_CODEC_AUDIO_G711U) {
            //  ret = tuya_g711_encode(TUYA_G711_MU_LAW, (unsigned short *)buf, frame->used_size, tmpBuf, &outLen);
        } else {
            //   ret = tuya_g711_encode(TUYA_G711_A_LAW, (unsigned short *)buf, frame->used_size, tmpBuf, &outLen);
        }
        if (0 != ret) {
            TYERROR("codectype:%d failed\n", codectype);
            return -1;
        }
        frame->pbuf = tmpBuf;
        frame->buf_size = bufSize;
        frame->used_size = outLen;
        frame->codectype = codectype;
        break;

    default:
        TYERROR("input is NULL\n");
        return -1;
    }

    // TYDEBUG("[%d]type:%d Size:%d usedSize:%d sam:%d seq:%u codec:%d pts:%llu\n",
    // type, frame->type, frame->bufSize, frame->usedSize, frame->sample, frame->seq,frame->codectype,frame->pts);
    return 0;
}

/***********************************************************
*  函数名: ty_dsp_audio_put
*  功能：音频播放
*  参数:  Input，E_DSP_AUDIO_SOURCE_TYPE type，音频源
        Input，S_DSP_AUDIO_FRAME *pframe，音频帧数据
*
*  返回值: <0 失败，=0 成功
***********************************************************/
int ty_dsp_audio_put(E_DSP_AUDIO_SOURCE_TYPE type, TAL_AUDIO_FRAME_INFO_T *pframe)
{
    int ret = -1;
    char buf[DSP_AUDIO_FRAME_LEN] = {0};
    // char *tmpBuf = NULL;
    // int buf_size = 0;
    int outLen = 0;
    // unsigned char g711Type = TUYA_G711_A_LAW;    //默认初始化为 TUYA_G711_A_LAW
    TUYA_CODEC_ID codectype = pframe->codectype;
    TUYA_AUDIO_SAMPLE_E sample = gdspAdapter.pstrDspInfo->audio[0].sample;
    TAL_AUDIO_FRAME_INFO_T frame = {0};
    TAL_AUDIO_FRAME_INFO_T *ptmpFrame = NULL;

    if (pframe == NULL || pframe->pbuf == NULL) {
        TYERROR("input is NULL\n");
        return -1;
    }

    if ((uint32_t)pframe->sample != sample) {
        TYERROR("sample not support, %d %d\n", pframe->sample, sample);
        return -1;
    }

    switch (codectype) {
    case TUYA_CODEC_AUDIO_PCM:
        ptmpFrame = pframe;
        break;

    case TUYA_CODEC_AUDIO_G711U:
        // g711Type = TUYA_G711_MU_LAW;    //不进行break
    case TUYA_CODEC_AUDIO_G711A:
        if (pframe->used_size * 2 > DSP_AUDIO_FRAME_LEN) {
            TYERROR("frame->used_size:%d > %d\n", pframe->used_size, DSP_AUDIO_FRAME_LEN);
            return -1;
        }

        // ret = tuya_g711_decode(g711Type, (unsigned short *)pframe->pbuf, pframe->used_size, buf, &outLen);
        // if (0 != ret) {
        //     TYERROR("tuya_g711_decode codectype:%d failed\n", codectype);
        //     return -1;
        // }
        frame = *pframe;
        frame.pbuf = buf;
        frame.used_size = outLen;
        frame.buf_size = DSP_AUDIO_FRAME_LEN;
        ptmpFrame = &frame;
        break;

    default:
        TYERROR("input is NULL\n");
        return -1;
    }

    ret = tal_ao_put_frame(0, 0, NULL, ptmpFrame);
    if (0 != ret) {
        TYERROR("tal_ao_put_frame frame failed type:%d\n", type);
        return -1;
    }

    return 0;
}

int ty_dsp_audio_play_wav(ty_dsp_audio_handle_s *phdl)
{
    int ret = -1;
    char buffer[DSP_AUDIO_FRAME_LEN / 4] = {0};
    unsigned long long startTime = 0; //播放开始时间
    unsigned long long Currtime = 0;
    unsigned long long playTime = 0; //需要的播放时间
    unsigned long long fileLen = 0;
    ty_dsp_audio_wav_s *pwav = NULL;
    TAL_AUDIO_FRAME_INFO_T frame = {0};
    int count = phdl->count;
    int flag = 0;
    uint64_t current_time = ty_time_tick_ms() / 1000;
    uint64_t start_time = ty_time_tick_ms() / 1000;
    int i = 0;

    pwav = ty_dsp_audio_wav_open(phdl->filename);
    if (pwav == NULL) {
        TYERROR("open failed\n");
        return -1;
    }

    if (count == 0) {
        flag = 1;
    }

    frame.buf_size = DSP_AUDIO_FRAME_LEN / 4;
    frame.pbuf = buffer;
    frame.sample = pwav->format.samples_per_sec;
    frame.type = E_AUDIO_FRAME;

    fseek(pwav->fp, pwav->data_offset * (-1), SEEK_END); //求不带头的数据的长度
    fileLen = ftell(pwav->fp);
    fseek(pwav->fp, pwav->data_offset, SEEK_SET);
    if (pwav->format.compression_code == 1) {
        frame.codectype = TUYA_CODEC_AUDIO_PCM;
        playTime = fileLen / ((frame.sample * 2) / 1000); //除1000 算出来的是ms
    } else if (pwav->format.compression_code == 6) {
        frame.codectype = TUYA_CODEC_AUDIO_G711A;
        playTime = fileLen / (frame.sample / 1000); //除1000 算出来的是ms
    } else if (pwav->format.compression_code == 7) {
        frame.codectype = TUYA_CODEC_AUDIO_G711U;
        playTime = fileLen / (frame.sample / 1000); //除1000 算出来的是ms
    } else {
        TYERROR("wav not support\n");
        ty_dsp_audio_wav_close(&pwav);
        return -1;
    }
    TYINFO("codectype, sample:%d %d, buf_size:%d\n", frame.codectype, frame.sample, frame.buf_size);
    startTime = ty_time_utc_ms();

    while ((flag == 1) || (count > 0)) {
        count--;
        fseek(pwav->fp, pwav->data_offset, SEEK_SET);
        while ((frame.used_size = fread(buffer, 1, sizeof(buffer), pwav->fp)) > 0) {
            ret = ty_dsp_audio_put(E_DSP_AUDIO_SOURCE_TYPE_FILE, &frame);
            if (0 != ret) {
                TYERROR("ty_dsp_audio_play_wav failed\n");
                ty_dsp_audio_wav_close(&pwav);
                return -1;
            }

            if (phdl->fileStop) {
                TYDEBUG("fileStop\n");
                ty_dsp_audio_wav_close(&pwav);
                memset(frame.pbuf, 0,
                       frame.used_size); //在音频播放结束时，防止有音频数据残留，塞入空数据，将残留音频数据挤出缓存区

                for (i = 0; i < 2; i++) {
                    tal_ao_put_frame(0, 0, NULL, &frame);
                }
                return 0;
            }

            if (flag == 1) {
                if (phdl->fileDurationTime > 0) {
                    current_time = ty_time_tick_ms() / 1000;
                    if (current_time >= start_time + phdl->fileDurationTime) {
                        TYDEBUG("fileDurationTime fileStop\n");
                        ty_dsp_audio_wav_close(&pwav);
                        return 0;
                    }
                }
            }
        }
    }

    ty_dsp_audio_wav_close(&pwav);

    playTime *= phdl->count;
    Currtime = ty_time_utc_ms();
    while (Currtime - startTime < playTime) {
        usleep(40 * 1000);
        Currtime = ty_time_utc_ms();
    }
    return 0;
}

int ty_dsp_audio_play_naked_stream(ty_dsp_audio_handle_s *phdl)
{
    int ret = 0;
    int flag = 0;
    uint64_t startTime = 0; //播放开始时间
    uint64_t Currtime = 0;
    uint64_t playTime = 0; //需要的播放时间
    uint64_t fileLen = 0;
    TAL_AUDIO_FRAME_INFO_T frame = {0};
    char buffer[DSP_AUDIO_FRAME_LEN / 2] = {0};
    FILE *fp = NULL;
    int count = phdl->count;
    int i = 0;
    fp = fopen(phdl->filename, "rb");
    if (NULL == fp) {
        TYERROR("open audio file %s fail\n", phdl->filename);
        return -1;
    }

    if (count == 0) {
        flag = 1;
    }

    frame.pbuf = buffer;
    frame.sample = gdspAdapter.pstrDspInfo->audio[0].sample; /* 与配置的音频采样率一致 */
    fseek(fp, 0l, SEEK_END);
    fileLen = ftell(fp);
    fseek(fp, 0l, SEEK_SET);
    switch (phdl->fileType) {
    case E_DSP_AUDIO_FILE_PCM:
        frame.buf_size = DSP_AUDIO_FRAME_LEN / 2;
        frame.codectype = TUYA_CODEC_AUDIO_PCM;
        playTime = fileLen / ((frame.sample * 2) / 1000); //除1000 算出来的是ms
        break;
    case E_DSP_AUDIO_FILE_G711U:
        frame.buf_size = DSP_AUDIO_FRAME_LEN / 4;
        frame.codectype = TUYA_CODEC_AUDIO_G711U;
        playTime = fileLen / (frame.sample / 1000); //除1000 算出来的是ms
        break;
    case E_DSP_AUDIO_FILE_G711A:
        frame.buf_size = DSP_AUDIO_FRAME_LEN / 4;
        frame.codectype = TUYA_CODEC_AUDIO_G711A;
        playTime = fileLen / (frame.sample / 1000); //除1000 算出来的是ms
        break;
    default:
        TYERROR("Not support filetype:%d\n", phdl->fileType);
        fclose(fp);
        return -1;
    }

    startTime = ty_time_utc_ms();

    while ((flag == 1) || (count > 0)) {
        count--;
        while ((frame.used_size = fread(buffer, 1, frame.buf_size, fp)) > 0) {
            ret = ty_dsp_audio_put(E_DSP_AUDIO_SOURCE_TYPE_FILE, &frame);
            if (0 != ret) {
                TYERROR("ty_dsp_audio_play_wav failed\n");
                fclose(fp);
                return -1;
            }

            if (phdl->fileStop) {
                TYDEBUG("fileStop\n");
                fclose(fp);
                memset(frame.pbuf, 0, frame.used_size);

                for (i = 0; i < 2; i++) {
                    tal_ao_put_frame(0, 0, NULL, &frame);
                }
                return 0;
            }
        }

        fseek(fp, 0l, SEEK_SET);
    }

    fclose(fp);

    playTime *= phdl->count;
    Currtime = ty_time_utc_ms();
    while (Currtime - startTime < playTime) {
        usleep(40 * 1000);
        Currtime = ty_time_utc_ms();
    }

    return 0;
}

void *ty_dsp_audio_play_file_proc(void *arg)
{
    (void)arg;
    int ret = 0;
    ty_dsp_audio_handle_s *phdl = &gdspAdapter.audioHandle;

    prctl(PR_SET_NAME, "audio_play_file");
    TYINFO("play %s, type:%d, count:%d\n", phdl->filename, phdl->fileType, phdl->count);

    ret = ty_dsp_ao_start(0, 0, NULL);
    if (0 != ret) {
        TYERROR("ty_dsp_ao_start failed\n");
        phdl->start = 0;
        return NULL;
    }

    switch (phdl->fileType) {
    case E_DSP_AUDIO_FILE_PCM:
    case E_DSP_AUDIO_FILE_G711U:
    case E_DSP_AUDIO_FILE_G711A:
        ret = ty_dsp_audio_play_naked_stream(phdl);
        if (0 != ret) {
            TYERROR("ty_dsp_audio_play_pcm failed\n");
            goto err;
        }
        break;

    case E_DSP_AUDIO_FILE_WAV:
        ret = ty_dsp_audio_play_wav(phdl);
        if (0 != ret) {
            TYERROR("ty_dsp_audio_play_wav failed\n");
            goto err;
        }
        break;

    default:
        TYERROR("Not support filetype:%d\n", phdl->fileType);
        goto err;
    }

    ret = ty_dsp_ao_stop(0, 0, NULL);
    if (0 != ret) {
        TYERROR("ty_dsp_ao_stop\n");
        goto err;
    }

err:
    phdl->start = 0;
    return NULL;
}

/***********************************************************
*  函数名: ty_dsp_audio_play_file
*  功能：播放文件接口
*  参数:  const char* fileName, 文件路径
       E_DSP_AUDIO_FILE_TYPE fileType, 文件格式类型
       int count, 播放次数，0，代表一直循环播放
       int timeOut， 播放阻塞时间单位，ms
*
*  返回值: <0 失败，=0 成功
***********************************************************/
int ty_dsp_audio_play_file(const char *fileName, E_DSP_AUDIO_FILE_TYPE fileType, int count, int timeOut)
{
    int ret = 0;
    pthread_attr_t attr = {0};

    if (fileName == NULL) {
        TYERROR("the file is null\n");
        return -1;
    }

    while (gdspAdapter.audioHandle.isInit == 0) {
        usleep(100 * 1000);
        TYWARN("wait dsp init:%d\n", gdspAdapter.audioHandle.isInit);
    }

    ty_dsp_audio_handle_s *phdl = &gdspAdapter.audioHandle;

    while (phdl->start && timeOut > 0) {
        TYDEBUG("The last play is not over, timeOut:%d\n", timeOut);
        usleep(200 * 1000);
        timeOut = timeOut - 200;
    }

    if (phdl->start) {
        TYERROR("speaker thread have start...\n");
        return -1;
    }

    strncpy(phdl->filename, fileName, 127);
    phdl->fileType = fileType;
    phdl->count = count;
    phdl->fileStop = 0;
    phdl->start = 1;
    pthread_attr_init(&attr);
    pthread_attr_setstacksize(&attr, DSP_AUDIO_THREAD_STACK_SIZE);
    ret = pthread_create(&phdl->stid, &attr, ty_dsp_audio_play_file_proc, NULL);
    if (ret != 0) {
        printf(" tycam_hisi_speaker_play_file end___,create fail!  ret:%d\n", ret);
        phdl->start = 0;
        return -1;
    }

    pthread_detach(phdl->stid);

    return 0;
}

/***********************************************************
 *  函数名: ty_dsp_audio_set_play_duration
 *  功能：设置播放时长s,音频文件持续播放模式时有效
 *  参数:  Input，int duration_time, 设置持续播放时长，单位s
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
int ty_dsp_audio_set_play_duration(int duration_time)
{
    TYINFO("duration_time %d\n", duration_time);
    ty_dsp_audio_handle_s *phdl = &gdspAdapter.audioHandle;
    phdl->fileDurationTime = duration_time;
    return 0;
}

/***********************************************************
 *  函数名: ty_dsp_audio_stop_play_file
 *  功能：停止播放文件
 *  参数:  无
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
int ty_dsp_audio_stop_play_file(void)
{
    TYINFO("stop to play file\n");
    ty_dsp_audio_handle_s *phdl = &gdspAdapter.audioHandle;
    phdl->fileStop = 1;
    return 0;
}

/***********************************************************
 *  函数名: ty_dsp_audio_get_play_file_status
 *  功能：获取文件播放状态
 *  参数:  无
 *
 *  返回值: 0未播放文件，1正在播放文件
 ***********************************************************/
int ty_dsp_audio_get_play_file_status(void)
{
    ty_dsp_audio_handle_s *phdl = &gdspAdapter.audioHandle;
    return phdl->start;
}

/***********************************************************
 *  函数名: ty_dsp_ao_start
 *  功能：音频播放开始,使能ao 通道以及apk对应的引脚
 *  参数: @param[in] card: card number
 *        @param[in] chn: channel number
 *        @param[out] handle: handle of start
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
int ty_dsp_ao_start(int card, int chn, VOID *handle)
{
    int ret = 0;
    ty_dsp_audio_handle_s *phdl = &gdspAdapter.audioHandle;
    S_DSP_MEDIA_INFO *mediaInfo = gdspAdapter.pstrDspInfo;
    // TAL_GPIO_LEVEL_E level = 0;

    sem_wait(&phdl->spkSem);
    if (phdl->spkRequestCount == 0) {
        ret = tal_ao_start(card, chn, handle);
        if (0 != ret) {
            TYERROR("tal_ao_start failed:%d\n", ret);
            sem_post(&phdl->spkSem);
            return ret;
        }
        if (mediaInfo->audio[chn].spk_gpio < 0) {
            TYDEBUG("no spk gpio\n");
            goto exit;
            return 0;
        }

        // if (mediaInfo->audio[chn].spk_gpio_polarity == 0) {
        //     level = TAL_GPIO_LEVEL_HIGH;
        // } else {
        //     level = TAL_GPIO_LEVEL_LOW;
        // }

        // ret = tal_gpio_write(mediaInfo->audio[chn].spk_gpio, level);
        // if (0 != ret) {
        //     TYERROR("tal_gpio_write failed:%d\n", ret);
        //     tal_ao_stop(card, chn, handle);
        //     sem_post(&phdl->spkSem);
        //     return -1;
        // }
    }
exit:
    phdl->spkRequestCount++;
    sem_post(&phdl->spkSem);
    return 0;
}

/***********************************************************
 *  函数名: ty_dsp_ao_stop
 *  功能：关闭ao, 关闭通道以及apk对应的引脚
 *  参数: @param[in] card: card number
 *        @param[in] chn: channel number
 *        @param[out] handle: handle of start
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
int ty_dsp_ao_stop(int card, int chn, VOID *handle)
{
    int ret = 0;
    ty_dsp_audio_handle_s *phdl = &gdspAdapter.audioHandle;
    S_DSP_MEDIA_INFO *mediaInfo = gdspAdapter.pstrDspInfo;
    // TAL_GPIO_LEVEL_E level = 0;

    sem_wait(&phdl->spkSem);
    if (phdl->spkRequestCount == 0) {
        sem_post(&phdl->spkSem);
        return 0;
    }
    phdl->spkRequestCount--;
    if (phdl->spkRequestCount == 0) {
        ret = tal_ao_stop(card, chn, handle);
        if (0 != ret) {
            TYERROR("tal_ao_stop failed:%d\n", ret);
            sem_post(&phdl->spkSem);
            return -1;
        }

        if (mediaInfo->audio[chn].spk_gpio < 0) {
            TYDEBUG("no spk gpio\n");
            sem_post(&phdl->spkSem);
            return 0;
        }

        // if (mediaInfo->audio[chn].spk_gpio_polarity == 0) {
        //     level = TAL_GPIO_LEVEL_LOW;
        // } else {
        //     level = TAL_GPIO_LEVEL_HIGH;
        // }
        // ret = tal_gpio_write(mediaInfo->audio[chn].spk_gpio, level);
        // if (0 != ret) {
        //     TYERROR("tal_gpio_write failed:%d\n", ret);
        //     sem_post(&phdl->spkSem);
        //     return -1;
        // }
    }
    sem_post(&phdl->spkSem);
    return 0;
}

/***********************************************************
 *  函数名: ty_dsp_uninit
 *  功能：dsp去初始化
 *  参数:  无
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
int ty_dsp_uninit(void)
{
    return 0;
}
