#include <sys/timeb.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <time.h>

#include "ty_log.h"

const char *log_info[] = {
    [TY_LOG_DEBUG] = "Debug", [TY_LOG_INFO] = "Info", [TY_LOG_NOTICE] = "Notice", [TY_LOG_WARN] = "Warn",
    [TY_LOG_ERROR] = "Error", [TY_LOG_CRIT] = "Crit", [TY_LOG_ALERT] = "Alert",   [TY_LOG_EMERG] = "Emerg"};

typedef struct rb_log_st {
    FILE *file;
    enum LOGLEVEL level;
} rb_log;

rb_log *g_rb_log = NULL;

void ty_log_init(const char *log_file, enum LOGLEVEL level)
{
    if (g_rb_log != NULL) {
        return;
    }

    g_rb_log = (rb_log *)malloc(sizeof(rb_log));
    if (g_rb_log == NULL) {
        return;
    }

    memset(g_rb_log, 0, sizeof(rb_log));

    if (log_file == NULL) {
        g_rb_log->file = stderr;
    } else {
        g_rb_log->file = fopen(log_file, "a+");
        if (g_rb_log->file == NULL) {
            g_rb_log->file = stderr;
        }
    }

    if (level < TY_LOG_DEBUG || level > TY_LOG_EMERG) {
        g_rb_log->level = TY_LOG_ERROR;
    } else {
        g_rb_log->level = level;
    }

    return;
}

void ty_log_uninit(void)
{
    if (g_rb_log == NULL) {
        return;
    }

    if (g_rb_log->file != stderr) {
        fclose(g_rb_log->file);
    }

    g_rb_log->file = NULL;

    free(g_rb_log);
    g_rb_log = NULL;

    return;
}

int ty_log_check_level(enum LOGLEVEL level)
{
    if (g_rb_log == NULL) {
        return 1;
    }

    return (g_rb_log->level > level ? 0 : 1);
}

void ty_log_print_col(enum LOGLEVEL level, char *col, const char *prefix, const char *fmt, ...)
{
    FILE *fp = NULL;
    va_list args;
    struct tm t_info;

    if (g_rb_log == NULL) {
        fp = stderr;
    } else {
        fp = g_rb_log->file;
    }
    struct timeb stTimeb;
    ftime(&stTimeb);
    localtime_r(&stTimeb.time, &t_info);
    fprintf(fp, "%s[%d-%d-%d %d:%d:%d.%03d %s %s] ", col, t_info.tm_year + 1900, t_info.tm_mon + 1, t_info.tm_mday,
            t_info.tm_hour, t_info.tm_min, t_info.tm_sec, stTimeb.millitm, prefix, log_info[level]);

    va_start(args, fmt);
    vfprintf(fp, fmt, args);
    va_end(args);
    return;
}

void ty_log_print(enum LOGLEVEL level, const char *prefix, const char *fmt, ...)
{
    FILE *fp = NULL;
    va_list args;
    struct tm t_info;

    if (g_rb_log == NULL) {
        fp = stderr;
    } else {
        fp = g_rb_log->file;
    }

#if 1
    struct timeb stTimeb;

    ftime(&stTimeb);
    localtime_r(&stTimeb.time, &t_info);

    fprintf(fp, "[%d-%d-%d %d:%d:%d.%03d %s %s] ", t_info.tm_year + 1900, t_info.tm_mon + 1, t_info.tm_mday,
            t_info.tm_hour, t_info.tm_min, t_info.tm_sec, stTimeb.millitm, prefix, log_info[level]);
#else
    time_t timep;

    time(&timep);
    t_info = localtime(&timep);

    fprintf(fp, "[%d-%d-%d %d:%d:%d %s %s] ", t_info->tm_year + 1900, t_info->tm_mon + 1, t_info->tm_mday,
            t_info->tm_hour, t_info->tm_min, t_info->tm_sec, prefix, log_info[level]);
#endif

    va_start(args, fmt);
    vfprintf(fp, fmt, args);
    va_end(args);

    return;
}
