#include <sys/prctl.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>

#include "ty_log.h"
#include "ty_shell_cmd.h"
#include "ty_time.h"

#define UTC_MATCH_TIME_S (1000000000) // 7682缺省下发 946684801-》2000/1/1 8:0:1
// 时间转换为hh:mm:ss
// echo GMT+hh:mm:ss > /var/conf/TZ
static time_t g_time_utc = 0;
static int g_timezone = 0;

void ty_time_wait_sync()
{
    int cnt = 0;
    while (g_time_utc < UTC_MATCH_TIME_S) {
        usleep(50 * 1000);
        cnt++;
        if (cnt % 20 == 0) {
            printf("wait for time sync");
        }
    }
    return;
}

uint64_t ty_time_tick_us()
{
    struct timespec tsnow = {0};
    uint64_t u64us;

    clock_gettime(CLOCK_MONOTONIC, &tsnow);
    u64us = tsnow.tv_sec;
    u64us *= 1000000ULL;
    u64us += tsnow.tv_nsec / 1000;
    return u64us;
}

uint64_t ty_time_tick_ms()
{
    struct timespec tsnow = {0};
    uint64_t u64ms;

    clock_gettime(CLOCK_MONOTONIC, &tsnow);
    u64ms = tsnow.tv_sec;
    u64ms *= 1000ULL;
    u64ms += tsnow.tv_nsec / (1000 * 1000);
    return u64ms;
}

uint64_t ty_time_utc_ms()
{
    struct timeval now;

    gettimeofday(&now, NULL);

    return ((uint64_t)now.tv_sec * 1000 + (uint64_t)now.tv_usec / 1000);
}

uint32_t ty_time_utc_s()
{
    struct timeval now;

    gettimeofday(&now, NULL);

    return now.tv_sec;
}

void ty_time_set_timezone(char *tzfile, int timezone)
{
    unsigned int tzValue = 0;
    int hour = 0, min = 0, sec = 0;
    char cmd[128] = {0};

    g_timezone = timezone;

    tzValue = (timezone > 0) ? timezone : (-timezone);

    hour = tzValue / 3600;
    min = (tzValue % 3600) / 60;
    sec = tzValue % 60;

    if (timezone > 0) {
        snprintf(cmd, sizeof(cmd), "echo GMT-%02d:%02d:%02d > %s", hour, min, sec, tzfile);
    } else {
        snprintf(cmd, sizeof(cmd), "echo GMT+%02d:%02d:%02d > %s", hour, min, sec, tzfile);
    }

    ty_cmd_excute_shell(cmd, NULL, 0);

    return;
}

void ty_time_set_systime(time_t time_utc)
{
    struct timeval tv;

    g_time_utc = time_utc;
    tv.tv_sec = time_utc;
    tv.tv_usec = 0;

    settimeofday(&tv, NULL);

    return;
}

void ty_time_get_systime(time_t *time_utc)
{
    *time_utc = g_time_utc;
    return;
}

int ty_time_get_timezone()
{
    return g_timezone;
}
