#if 0

#include <sys/prctl.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <time.h>

#include "ty_log.h"
#include "ty_timer.h"


typedef struct ty_timer_handle_ {
    void*       ptr;
    char        name[20];
    timer_t     timerid;
    TIMERLOOP   timer_proc;
} ty_timer_handle_s;



static void ty_timer_proc(union sigval v)  
{
    if(v.sival_ptr == NULL) {
        return;
    }

    ty_timer_handle_s* pthdl = (ty_timer_handle_s*)v.sival_ptr;

    prctl(PR_SET_NAME, pthdl->name);

    pthdl->timer_proc(pthdl->ptr);

    return;
}

int ty_timer_create(void** pphdl, ty_timer_param_s* param)
{
    struct sigevent evp;
    ty_timer_handle_s* pthdl = NULL;

    if(param == NULL) {
        return -1;
    }

    TYDEBUG("begin___, name(%s)\n", param->name);

    pthdl = malloc(sizeof(ty_timer_handle_s));
    if(pthdl == NULL) {
        return -1;
    }

    memset(pthdl, 0, sizeof(ty_timer_handle_s));
    memset(&evp, 0, sizeof(struct sigevent));

    evp.sigev_value.sival_ptr = pthdl;
    evp.sigev_notify = SIGEV_THREAD;
    evp.sigev_notify_function = ty_timer_proc;
 
    if(timer_create(CLOCK_REALTIME, &evp, &pthdl->timerid) == -1) {  
        TYERROR("end___, fail to timer create\n");
        free(pthdl);
        return -1;
    }

    pthdl->ptr = param->ptr;
    pthdl->timer_proc = param->timer_proc;
    strncpy(pthdl->name, param->name, sizeof(param->name));

    TYDEBUG("end___\n");

    *pphdl = pthdl;

    return 0;
}

int ty_timer_destory(void** pphdl)
{
    ty_timer_handle_s* pthdl = (ty_timer_handle_s*)*pphdl;

    if(pthdl == NULL) {
        return -1;
    }

    TYDEBUG("begin___, name(%s)\n", pthdl->name);

    timer_delete(pthdl->timerid);

    free(pthdl);
    pthdl = NULL;

    TYDEBUG("end___\n");

    return 0;
}

int ty_timer_set_time(void* phdl, char abstime, uint64_t value, uint64_t interval)
{
    int flags = 0;
    struct itimerspec it;
    ty_timer_handle_s* pthdl = (ty_timer_handle_s*)phdl;

    if(pthdl == NULL) {
        return -1;
    }

    if(abstime) {
        flags |= TIMER_ABSTIME;
    }

    it.it_value.tv_sec = value/1000;     // 最初开始时间间隔
    it.it_value.tv_nsec = (value%1000) * 1000;
    it.it_interval.tv_sec = interval/1000;  // 后续按照该时间间隔
    it.it_interval.tv_nsec = (interval%1000) * 1000;

    if(timer_settime(pthdl->timerid, flags, &it, NULL) == -1) {
        TYERROR("fail to timer settime\n");
        return -1;
    }

    return 0;
}

int ty_timer_unset_time(void* phdl)
{
    struct itimerspec ts;
    ty_timer_handle_s* pthdl = (ty_timer_handle_s*)phdl;

    if(pthdl == NULL) {
        return -1;
    }
    TYDEBUG("begin___, name(%s)\n", pthdl->name);

    ts.it_value.tv_sec  = 0;
    ts.it_value.tv_nsec = 0;
    ts.it_interval = ts.it_value;

    timer_settime(pthdl->timerid, 0, &ts, NULL);
    TYDEBUG("end___\n");

    return 0;
}
#endif

