/**
 * @file tal_video_in.h
 * @brief video input
 * @version 0.1
 * @date 2021-11-09
 *
 * @copyright Copyright 2019-2021 Tuya Inc. All Rights Reserved.
 *
 * Video input configuration: for sensor, ISP or VI attribute configuration,
 * such as image flip, anti flicker, compensation mode, profile, etc.
 */

#include <stdio.h>
#include "tal_video_in.h"
#include "tuya_iot_config.h"
#include "tuya_error_code.h"
#include "uni_log.h"

/**
 * @brief vi init
 *
 * @param[in] pconfig: vi config
 * @param[in] count: count of pconfig
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_vi_init(TAL_VI_CONFIG_T *pconfig, INT32_T count)
{
    INT32_T ret = 0;
    ret = tkl_vi_init(pconfig, count);
    if (ret != 0) {
        PR_ERR("tkl_vi_init failed, ret = %d\n", ret);
        return OPRT_COM_ERROR;
    }

    return OPRT_OK;
}

/**
 * @brief vi set
 *
 * @param[in] chn: vi chn
 * @param[in] cmd: cmd
 * @param[in] parg: parg
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_vi_set(INT32_T chn, TAL_VI_CMD_E cmd, VOID *parg)
{
    INT32_T ret = -1;
    if (NULL == parg) {
        PR_ERR("parg is null, cmd:%d\n", cmd);
        return OPRT_COM_ERROR;
    }
    switch (cmd) {
    case TAL_VI_CMD_MIRROR_FLIP:
        ret = tkl_vi_set_mirror_flip((TKL_VI_CHN_E)chn, *(TKL_VI_MIRROR_FLIP_E *)parg);
        if (ret != 0) {
            PR_ERR("tkl_vi_set_mirror_flip  failed, ret = %d\n", ret);
            return OPRT_COM_ERROR;
        }
        break;
    case TAL_VI_CMD_SENSOR_REG:
        ret = tkl_vi_sensor_reg_set((TKL_VI_CHN_E)chn, (TAL_VI_SENSOR_REG_CONFIG_T *)parg);
        if (ret != 0) {
            PR_ERR("tkl_vi_sensor_reg_set failed, ret = %d\n", ret);
            return OPRT_COM_ERROR;
        }
        break;

    default:
        PR_ERR("not support cmd:%d\n", cmd);
        break;
    }
    return OPRT_OK;
}

/**
 * @brief vi get
 *
 * @param[in] chn: vi chn
 * @param[in] cmd: cmd
 * @param[out] parg: parg
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_vi_get(INT32_T chn, TAL_VI_CMD_E cmd, VOID *parg)
{
    INT32_T ret = -1;
    if (NULL == parg) {
        PR_ERR("parg is null, cmd:%d\n", cmd);
        return OPRT_COM_ERROR;
    }
    switch (cmd) {
    case TAL_VI_CMD_MIRROR_FLIP:
        ret = tkl_vi_get_mirror_flip((TKL_VI_CHN_E)chn, (TKL_VI_MIRROR_FLIP_E *)parg);
        if (ret != 0) {
            PR_ERR("tkl_vi_get_mirror_flip  failed, ret = %d\n", ret);
            return OPRT_COM_ERROR;
        }
        break;
    case TAL_VI_CMD_SENSOR_REG:
        ret = tkl_vi_sensor_reg_get((TKL_VI_CHN_E)chn, (TAL_VI_SENSOR_REG_CONFIG_T *)parg);
        if (ret != 0) {
            PR_ERR("tkl_vi_sensor_reg_get failed, ret = %d\n", ret);
            return OPRT_COM_ERROR;
        }
        break;

    default:
        PR_ERR("not support cmd:%d\n", cmd);
        break;
    }
    return OPRT_OK;
}

/**
 * @brief vi uninit
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tal_vi_uninit(VOID)
{
    INT32_T ret = -1;
    ret = tkl_vi_uninit();
    if (ret != 0) {
        PR_ERR("tkl_vi_uninit failed \n");
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

