#ifndef _TKL_FH8636_VIDEO_H_
#define _TKL_FH8636_VIDEO_H_

#include <stdio.h>
#include <stdint.h>
#include "fh_common.h"
#include "isp_enum.h"
#include "fh_vpu_mpipara.h"
#include "fh_venc_mpipara.h"
#include "isp_sensor_if.h"
#include "tkl_video_enc.h"

#if defined(__cplusplus)
extern "C" {
#endif

#define VPU_CHN_MAX 4 
#define FH_GRP_ID 0

/** Return Pointer Check */
#define TY_CHECK_POINTER(p, errcode)                                                                                   \
    do {                                                                                                               \
        if (!(p)) {                                                                                                    \
            printf("pointer[%s] is NULL\n", #p);                                                                       \
            return errcode;                                                                                            \
        }                                                                                                              \
    } while (0)

/** Return Result Check */
#define TY_CHECK_RETURN(ret, errcode)                                                                                  \
    do {                                                                                                               \
        if (FH_SUCCESS != ret) {                                                                                       \
            printf("[%s:%d]Error Code: [0x%08X]\n\n", __FUNCTION__, __LINE__, ret);                                                                   \
            return errcode;                                                                                            \
        }                                                                                                              \
    } while (0)

/** Return Result Check With Error String */
#define TY_CHECK_RETURN_WITH_ERRINFO(ret, errcode, errstring)                                                          \
    do {                                                                                                               \
        if (FH_SUCCESS != ret) {                                                                                       \
            printf("[%s] failed[0x%08X]\n", errstring, ret);                                                           \
            return errcode;                                                                                            \
        }                                                                                                              \
    } while (0)

/*
    vi相关参数
*/
typedef struct ty_fh8636_vi_param_ {
    struct isp_sensor_if *pSensorinfo;
    uint32_t ispFormat;
    FH_VPU_SIZE viSize; // vi的参数
} TY_FH8636_VI_PARAM_T;

typedef struct {
    int ref_luma[4];
    TKL_VI_CONFIG_T config;
} TY_FH8636_VI_HANDLE_T;

/*
    vpu相关参数
*/
typedef struct ty_fh8636_vpu_param_ {
    int enable;
    FH_VPU_VO_MODE vpuType;
    FH_VPU_CHN_CONFIG chnAttr;
    int fps;
} TY_FH8636_VPU_PARAM_T;

/*
    venc相关参数
*/
typedef struct ty_fh8636_venc_param_ {
    int enable;
    int vpuChn; //绑定的 vpu的通道
    int encChn; //编码通道
    uint32_t width;
    uint32_t height;
    FH_VENC_CHN_CAP vencCap;
    FH_VENC_CHN_CONFIG vencCfg;
    TKL_VENC_DATA_TRANS_MODE_E trans_mode;
    TKL_VENC_PUT_CB put_cb;
} TY_FH8636_VENC_PARAM_T;

/*
    osd相关参数
*/
typedef struct ty_fh8636_osd_param_ {
    int enable;
    uint32_t left;
    uint32_t top;
    uint32_t fontW;
    uint32_t fontH;
} TY_FH8636_OSD_PARAM_T;

//#define FH_VENC_QUEUE     // 本地编码缓存
/*
    视频模块的头结点信息包含 编码、osd参数和首图信息
*/
typedef struct ty_fh8636_video_handle_ {
    int isInit;
    int runing;
    pthread_mutex_t yuv_mutex;
    pthread_mutex_t enc_mutex;
    int jpeg_channel_inited;
    int thread_status;
    TY_FH8636_VI_PARAM_T viParam;
    TY_FH8636_VPU_PARAM_T vpuParam[VPU_CHN_MAX];
    TY_FH8636_VENC_PARAM_T encParam[TKL_VENC_TYPE_MAX];
    TY_FH8636_OSD_PARAM_T osdParam[TKL_VENC_TYPE_MAX];
    int isp_thread_stop;
    int isp_thread_running;
    int isp_anti_flicker;
    int isp_wdr_mode;
    char *frame_buf;
    TKL_VENC_FORMAT_T format[TKL_VENC_TYPE_MAX];                     // 当前只支持主子码流动态修改
    int format_flag[TKL_VENC_TYPE_MAX];                              // 表示是否需要更新编码参数，1，需要更新，0 不需要更新

    int qr_chn;                                      // 区别普通产品和homekit产品，普遍产品MD QR的yuv通道用1，homekit 产品用2
    TKL_VENC_TIME_CB cb;
} TY_FH8636_VIDEO_HANDLE_T;

TY_FH8636_VIDEO_HANDLE_T *__get_video_handle();
TY_FH8636_VI_HANDLE_T *tkl_vi_get_handle();
FH_SINT32 fh8636_isp_init(FH_VOID);
FH_VOID fh8636_isp_deinit(FH_VOID);

OPERATE_RET tkl_vi_isp_show_fw_version(PCHAR_T version, PCHAR_T name);
OPERATE_RET tkl_vi_isp_set_mirror_flip(TKL_VI_CHN_E chn, int mirror, int flip);
OPERATE_RET tkl_vi_isp_get_mirror_flip(TKL_VI_CHN_E chn, int *pmirror, int *pflip);
OPERATE_RET tkl_vi_isp_init(TKL_VI_CHN_E chn, TKL_ISP_PARAM_INFO_T *pisp_info);

#if defined(__cplusplus)
}
#endif

#endif
