﻿#ifndef FHASD_H
#define FHASD_H

#ifdef __cplusplus
extern "C" {
#endif

/*
* 功能:异常声音检测初始化函数
* 参数:
*	p_handle为初始化结构体指针
*	sampleRate为采样率，支持16000和8000
*
* 返回值：0表示成功，其他表示不成功
*/

int FHASD_Init(void** p_handle, unsigned int sampleRate);

/*
* 功能：异常声音检测处理函数
* 参数：
*	in_buffer:输入音频buffer,仅支持单声道
*	samples:输入音频采样点数，16k采样对应160个点，8k采样对应80个点
* 返回值：
*		0:处理成功
*		-1:处理失败
*/
int FHASD_Process(void* handle, short* in_buffer, int samples);

/*
* 功能：异常声音检测结束内存释放函数
* 参数：
*
* 返回值: 0表示成功,其他表示错误
*/
int FHASD_DeInit(void* handle);

/*
* 功能：异常声音检测算法版本
* 返回值：当前算法版本号
*/
int FHASD_Version(void);

/*
* 功能：设置异常声音检测参数
* 参数：
*		request：选择设置项，详细见下方宏定义
*		*ptr：具体配置值的指针
*/
int FHASD_Set(void* handle, int request, void* ptr);


/* 设置异常声音检测详细参数 */
/* 设置婴儿啼哭检测门限，门限值默认0.85,建议取值[0.6-1)*/
#define FHASD_SET_BABYCRY_DETECTOR_THR 0
/* 设置婴儿啼哭检测时长门限，门限值默认30,建议取值[30-50)*/
#define FHASD_SET_BABYCRY_DETECTOR_HIST_LEN 1
/* 设置婴儿啼哭检测间隔，单位为秒*/
#define FHASD_SET_BABYCRY_DETECTOR_SILENCE_TIME 2
/* 设置玻璃破碎检测门限，门限值默认0.85,建议取值[0.6-1)*/
#define FHASD_SET_GLASSBREAK_DETECTOR_THR 100
/* 设置玻璃破碎检测时长门限，门限值默认20,建议取值[10-30)*/
#define FHASD_SET_GLASSBREAK_DETECTOR_HIST_LEN 101
/* 设置玻璃破碎检测间隔，单位为秒*/
#define FHASD_SET_GLASSBREAK_DETECTOR_SILENCE_TIME 102

/* 设置低能量保护使能，默认值为1，开启*/
#define FHASD_SET_LOW_ENERGY_PROTECTOR_EN 10
/* 设置低能量门限，低于门限不检测，默认门限取值40，意为小于(40-90)dB信号不检测*/
#define FHASD_SET_LOW_ENERGY_PROTECTOR_THR 11
/* 设置信号放大使能，默认值为0，关闭*/
#define FHASD_SET_AGC_EN 22
/* 设置信号放大增益*/
#define FHASD_SET_AGC_GAIN 23

/*
* 功能：获取异常声音检测结果
*		request：选择设置项，详细见下方宏定义
*		*ptr：返回结果指针
*/
int FHASD_Get(void* handle, int request, void* ptr);

/* 获取婴儿啼哭检测结果 */
#define FHASD_GET_BABYCRY_DETECTOR_RESULT 0
/* 获取玻璃破碎检测结果 */
#define FHASD_GET_GLASSBREAK_DETECTOR_RESULT 100

 

#ifdef __cplusplus
}
#endif
#endif