#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stdarg.h>
#include "mat.h"
#include <string.h>

void PrintMatrix(Matrix *T)
{
    int i,j;
    if (T == NULL) {
        printf("the matrix is null!!!\n");
        return;
    }
    printf("ans=\n");
    for (i = 0; i < T->row; i++) {
        printf("\t");
        for (j = 0; j < T->col; j++) {
            printf("%f\t", T->mat[i][j]);
        }
        printf("\n");
    }
    printf("\n");
}

MatPtr CreateMatrix( int row, int col)
{
    int i,j;
    MatPtr T = NULL;
    
    T = (MatPtr) malloc(sizeof(Matrix));
    if (T == NULL) {
        return NULL;
    }
    
    T->mat = (MatEleType **)malloc(row * sizeof(MatEleType *));
    if (T->mat == NULL) {
        free(T);
        return NULL;
    }
    for (i = 0; i < row; i++) {
        T->mat[i] = (MatEleType *)malloc(col * sizeof(MatEleType));
        if (T->mat[i] == NULL) {
            for (j = 0; j < i; j++) {
                free(T->mat[j]);
            }
            free(T->mat);
            free(T);
            return NULL;
        }
        memset(T->mat[i], 0, col * sizeof(MatEleType));
    }
    T->row = row;
    T->col = col;
    
    return T;
}

void DestroyMatrix(MatPtr T)
{
    if(T == NULL) {
        return;
    }
    
    int i;
    for (i = 0; i < T->row; i++) {
        free(T->mat[i]);
    }
    free(T->mat);
    free(T);
}

void InitMatrix(MatPtr T, MatEleType arr[])
{
    int i, j;
    for (i = 0; i < T->row; i++) {
        for (j = 0; j < T->col; j++) {
            T->mat[i][j] = arr[ i * T->col + j];
        }
    }
}

void SetMatrix(MatPtr T, ...)
{
    va_list ap;
    int i, j;
    va_start(ap, T);
    for (i = 0; i < T->row; i++) {
        for (j = 0; j < T->col; j++) {
            T->mat[i][j] = va_arg(ap, double);
        }
    }
    va_end(ap);
}

void SetIdentityMatrix(MatPtr T, MatEleType idn)
{
    int i;
    for (i = 0; (i < T->row) && (i < T->col); i++) {
        T->mat[i][i] = idn;
    }
}

MatPtr CopyMatrix(MatPtr src)
{
    int i, j;
    MatPtr T = NULL;
    if (src == NULL) {
        return NULL;
    }
    T = CreateMatrix(src->row, src->col);
    if (T == NULL) {
        return NULL;
    }
    for (i = 0; i < src->row; i++) {
        for (j = 0; j < src->col; j++) {
            T->mat[i][j] = src->mat[i][j];
        }
    }
    return T;
}

MatPtr TransMatrix(MatPtr A)
{
    int i, j;
    MatPtr T = NULL;
    if (A == NULL) {
        return NULL;
    }
    T = CreateMatrix(A->col, A->row);
    if (T == NULL) {
        return NULL;
    }
    for (i = 0; i < A->row; i++) {
        for (j = 0; j < A->col; j++) {
            T->mat[j][i] = A->mat[i][j];
        }
    }
    return T;
}

MatPtr AddMatrix(MatPtr A, MatPtr B)
{
    int i, j;
    MatPtr T = NULL;
    if (A == NULL || B == NULL) {
        return NULL;
    }
    if ( (A->row != B->row) || (A->col != B->col) ) {
        return NULL;
    }
    T = CreateMatrix(A->row, A->col);
    if (T == NULL) {
        return NULL;
    }
    for (i = 0; i < A->row; i++) {
        for (j = 0; j < A->col; j++) {
            T->mat[i][j] = A->mat[i][j] + B->mat[i][j];
        }
    }
    return T;
}


MatPtr SubMatrix(MatPtr A, MatPtr B)
{
    int i, j;
    MatPtr T = NULL;
    if (A == NULL || B == NULL) {
        return NULL;
    }
    if ( (A->row != B->row) || (A->col != B->col) ) {
        return NULL;
    }
    T = CreateMatrix(A->row, A->col);
    if (T == NULL) {
        return NULL;
    }
    for (i = 0; i < A->row; i++) {
        for (j = 0; j < A->col; j++) {
            T->mat[i][j] = A->mat[i][j] - B->mat[i][j];
        }
    }
    return T;
}

MatPtr ProductMatrix(MatPtr A, MatPtr B)
{
    int i, j, k;
    MatPtr T = NULL;
    if (A == NULL || B == NULL) {
        printf("inpunt null\n");
        return NULL;
    }
    if ( (A->col != B->row) ) {
        printf("inpunt col:%d row:%d\n", A->col , B->row);
        return NULL;
    }
    T = CreateMatrix(A->row, B->col);
    if (T == NULL) {
        return NULL;
    }
    for (i = 0; i < A->row; i++) {
        for (j = 0; j < B->col; j++) {
            T->mat[i][j] = 0.0;
            for (k = 0; k < A->col; k++) {
                T->mat[i][j] += A->mat[i][k] * B->mat[k][j];
            }
        }
    }
    return T;
}

MatPtr IdentitySubMatrix(MatPtr T)
{
 int i, j;
 MatPtr tmp;
    if (T->row != T->col) {
        return NULL;
    }
    tmp = CreateMatrix(T->row, T->col);
    for (i = 0; i < T->row; i++) {
        for (j = 0; j < T->col; j++) {
            if (i == j) {
                tmp->mat[i][j] = 1.0 - T->mat[i][j];
            } else {
                tmp->mat[i][j] = 0.0 - T->mat[i][j];
            }
        }
    }
    return tmp;
}

static void identityMatrix(MatPtr T)
{
    int i, j;
    if (T->row != T->col) {
        return;
    }
    for (i = 0; i < T->row; i++) {
        for (j = 0; j < T->col; j++) {
            if (i == j) {
                T->mat[i][j] = 1.0;
            } else {
                T->mat[i][j] = 0.0;
            }
        }
    }
}

/* I - T */
// static void SubMatrixFromIdentity(MatPtr T)
// {
//     int i, j;
//     if (T->row != T->col) {
//         return;
//     }
//     for (i = 0; i < T->row; i++) {
//         for (j = 0; j < T->col; j++) {
//             if (i == j) {
//                 T->mat[i][j] = 1.0 - T->mat[i][j];
//             } else {
//                 T->mat[i][j] = 0.0 - T->mat[i][j];
//             }
//         }
//     }
// }

/*
  this function is to determine whether the two matrix are equal or not
    if A==B then the returned vaule is 1,else the returned vaule is 0
*/
// static int isEqualMatrix(MatPtr A, MatPtr B, MatEleType tolerance)
// {
//     int i, j;
//     if (A == NULL || B == NULL) {
//         return 0;
//     }
//     if ( (A->col != B->col) || (A->row != B->row) ) {
//         return 0;
//     }
//     for (i = 0; i < A->row; i++) {
//         for (j = 0; j < A->col; j++) {
//             if (fabs(A->mat[i][j] - B->mat[i][j]) > tolerance) {
//                 return 0;
//             }
//         }
//     }
//     return 1;
// }

/*
 * A = a*A, a is a scalar
*/
// static void scaleMatrix(MatPtr T, MatEleType scalar)
// {
//     int i, j;
//     if (T == NULL) {
//         return;
//     }
//     for (i = 0; i < T->row; i++) {
//         for (j = 0; j < T->col; j++) {
//             T->mat[i][j] *= scalar;
//         }
//     }
// }

/*
 * exchange two rows of the matrix
*/
static void swapRowsMatrix(MatPtr T, int r1, int r2)
{
    MatEleType *tmp;
    if (r1 == r2) {
        return;
    }
    tmp = T->mat[r1];
    T->mat[r1] = T->mat[r2];
    T->mat[r2] = tmp;
}

/*
 * the specified matrix's row product a scalar
*/
static void scaleRow(MatPtr T, int r, MatEleType scalar)
{
    int i;
    for (i = 0; i < T->col; i++) {
        T->mat[r][i] *= scalar;
    }
}

/*
 * Add scalar*row r2 to row r1.
*/
static void shearRow(MatPtr T, int r1, int r2, MatEleType scalar)
{
    int i;
    if (r1 == r2) {
        return;
    }
    for (i = 0; i < T->col; i++) {
        T->mat[r1][i] += scalar * T->mat[r2][i];
    }
}

/* Uses Gauss-Jordan elimination.
   The elimination procedure works by applying elementary row
   operations to our input matrix until the input matrix is reduced to
   the identity matrix.
   Simultaneously, we apply the same elementary row operations to a
   separate identity matrix to produce the inverse matrix.
   If this makes no sense, read wikipedia on Gauss-Jordan elimination.
   This is not the fastest way to invert matrices, so this is quite
   possibly the bottleneck. */
MatPtr InvertMatrix(MatPtr input)
{
    int i, j, r;
    MatEleType scalar;
    MatEleType shearNeeded;
    MatPtr output = NULL;
    MatPtr tmp = NULL;
    MatEleType eps = 0.0000001;
    if (input->row != input->col) {
        return NULL;
    }
    output = CreateMatrix(input->row, input->col);
    tmp = CopyMatrix(input);
    if (output == NULL || tmp == NULL) {
        return NULL;
    }
    identityMatrix(output);
    /* Convert input to the identity matrix via elementary row operations.
         The ith pass through this loop turns the element at i,i to a 1
         and turns all other elements in column i to a 0. */
    for (i = 0 ; i < tmp->row; i++) {
        if (fabs(tmp->mat[i][i]) < eps) {
            for (r = i + 1; r < tmp->row; r++) {
                if (fabs(tmp->mat[r][i]) > eps) {
                    /* We must swap rows to get a nonzero diagonal element. */
                    break;
                }
            }
            if (r == tmp->row) {
                /* Every remaining element in this column is zero, so this
                       matrix cannot be inverted. */
                printf("cannot be inverted!!\n");
                printf("the matrix is :\n");
                PrintMatrix(input);
                return NULL;
            }
            swapRowsMatrix(tmp, i, r);
            swapRowsMatrix(output, i, r);
        }
        /* Scale this row to ensure a 1 along the diagonal.
               We might need to worry about overflow from a huge scalar here. */
        scalar = 1.0 / tmp->mat[i][i];
        scaleRow(tmp, i, scalar);
        scaleRow(output, i, scalar);
        /* Zero out the other elements in this column. */
        for (j = 0; j < tmp->row; j++) {
            if (i == j) {
                continue;
            }
            shearNeeded = -tmp->mat[j][i];
            shearRow(tmp, j, i, shearNeeded);
            shearRow(output, j, i, shearNeeded);
        }
    }
    DestroyMatrix(tmp);
    return output;
}