/*
 * sc3336p_mipi.c
 *
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "clock_ops.h"
#include "isp_sensor_if.h"
#include "sensor_ops.h"

#include "isp_enum.h"
#include "mipi_api.h"
#include "sc3336p_mipi_param.h"
#include "sc3336p_mipi_wdr_param.h"
#include "sensor.h"
#include "sc3336p_mipi.h"
#include "sensor_std.h"
#define _NAME(n, s) n##_##s
#define NAME(n)     _NAME(n, sc3336p_mipi)
static SNS_CTRL_PARA_S stSnsDefaultCtrlPara = {
                                       .bInited = 0,
                                       .u32CurrExposureRatio = 256, // 默认初始化为16倍
                                       .u32MinRstLine = 4,
                                       .u32MinAgain = 0x40,
                                   };

static FH_UINT32 FW_ori = 0x4e2;
static FH_UINT32 FH_ori = 0x550;
// static FH_UINT32 FPS_ori = 30;


// static int enable = 0;
// static unsigned int status_mirror=0;
// static unsigned int status_flip=0;

static int SetIntt(void *pvObj, FH_UINT32 intt, FH_UINT8 fNo);
static int GetIntt(void *pvObj);
static int SetGain(void *pvObj, FH_UINT32 again, FH_UINT8 fNo);
static int GetGain(void *pvObj);
static int SetExposureRatio(void *pvObj, FH_UINT32 exposure_ratio);
static int GetExposureRatio(void *pvObj, FH_UINT32 *exposure_ratio);
/********************sensor flow related start****************************/
static void SensorReset(void *pvObj)
{
    (void)pvObj;
    return;
}

static int Sensor_Init(void *pvObj, Sensor_Init_t *stInitConf)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    pstCtrlPara->u8CciDeviceId = stInitConf->u8CciDeviceId;
    pstCtrlPara->u8CsiDeviceId = stInitConf->u8CsiDeviceId;
    SensorDevice_Init(pstCci, (0x60 >> 1), 2, pstCtrlPara->u8CciDeviceId);
    SensorReset(pvObj);
    pstCtrlPara->bInited = 1;
    return 0;
}

static int Sensor_Isconnect(void *pvObj, FH_UINT8 deviceId)
{
    (void)pvObj;
    (void)deviceId;
    return 1;
#if 0
    unsigned int sensor_chip_id;
    SensorDevice_Init((0x60 >> 1), 2);

    Sensor_Write(0x440d,0x10);
    Sensor_Write(0x4400,0x11);
    usleep(12000);
    sensor_chip_id = Sensor_Read(0x3108);
    sensor_chip_id |= Sensor_Read(0x3107) << 8;

    fh_printf("@@@SC3336P sensor_chip_id %x \n",sensor_chip_id);

    SensorDevice_Close();

    if (sensor_chip_id == 0x9c41)	//SC3336P
	//if (sensor_chip_id == 0xcc41)   SC3336
        return 1;
    else
        return 0;
#endif
}

static int Sensor_DeInit(void *pvObj)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);

    SensorReset(pvObj);

    return SensorDevice_Close(pstCci);
}

static void SetSensorMipiCfg(void *pvObj)
{
    // printf("------SetSensorMipiCfg-------\n");
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    struct mipi_conf config;

    config.frq_range = pstCtrlPara->pstCurrConf->u8MipiClkRate;
    config.raw_type = RAW10;
    config.lane_num = pstCtrlPara->pstCurrConf->u8MipiLaneNum;
    config.mipi_id = pstCtrlPara->u8CsiDeviceId;

    if (pstCtrlPara->u32WdrFlag == 1) {
        config.sensor_mode = NOT_SONY_WDR_USE_WDR;
        config.lf_vc_id = 0;
        config.sf_vc_id = 1;
    } else {
        config.sensor_mode = NON_WDR;
        config.lf_vc_id = 0;
        config.sf_vc_id = 0;
    }
    mipi_init(&config);
    return;
}

static int SetLaneNumMax(void *pvObj, FH_UINT32 lane_num)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    pstCtrlPara->u32MaxLaneNum = lane_num;
    return 0;
}

static SNS_CONFIG_S *getSensorConfType(void *pvObj)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    pstCtrlPara->u32CurrConfSize = sizeof(stSnsConf0) / sizeof(SNS_CONFIG_S);
    return stSnsConf0;
}

static int getSensorConf(void *pvObj)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SNS_CONFIG_S *pstConfGrp = getSensorConfType(pvObj);
    FH_UINT32 i;

    for (i = 0; i < pstCtrlPara->u32CurrConfSize; i++) {
        if (pstCtrlPara->u32SnsFmt == pstConfGrp[i].u32SnsFmt) {
            if (pstCtrlPara->u32MaxLaneNum >= pstConfGrp[i].u8MipiLaneNum) {
                set_clk_rate(SNS_CLK_NAME(pstCtrlPara->u8CsiDeviceId), pstConfGrp[i].u32SnsClk);

                pstCtrlPara->pstCurrConf = &pstConfGrp[i];

                return FH_RET_SENSOR_OK;
            }
        }
    }

    return FH_RET_SENSOR_NO_SUPPORT_FORMAT;
}

static int GetSensorViAttr(void *pvObj, ISP_VI_ATTR_S *stViAttr)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    FH_SINT32 s32Ret;

    if (stViAttr == NULL)
        return FH_RET_SENSOR_NULL_POINTER;

    if (pstCtrlPara->pstCurrConf == NULL) {
        s32Ret = getSensorConf(pvObj);

        if (s32Ret != FH_RET_SENSOR_OK)
            return s32Ret;
    }

    stViAttr->u16WndWidth = pstCtrlPara->pstCurrConf->stSnsAttr->u16WndWidth;
    stViAttr->u16WndHeight = pstCtrlPara->pstCurrConf->stSnsAttr->u16WndHeight;
    stViAttr->u16InputWidth = pstCtrlPara->pstCurrConf->stSnsAttr->u16InputWidth;
    stViAttr->u16InputHeight = pstCtrlPara->pstCurrConf->stSnsAttr->u16InputHeight;
    stViAttr->u16PicWidth = pstCtrlPara->pstCurrConf->stSnsAttr->u16PicWidth;
    stViAttr->u16PicHeight = pstCtrlPara->pstCurrConf->stSnsAttr->u16PicHeight;
    stViAttr->u16OffsetX = pstCtrlPara->pstCurrConf->stSnsAttr->u16OffsetX;
    stViAttr->u16OffsetY = pstCtrlPara->pstCurrConf->stSnsAttr->u16OffsetY;
    stViAttr->enBayerType = pstCtrlPara->pstCurrConf->stSnsAttr->enBayerType;
    stViAttr->u16FrameRate = pstCtrlPara->pstCurrConf->stSnsAttr->u16FrameRate;
    pstCtrlPara->u32CurrHsf = pstCtrlPara->pstCurrConf->stSnsAttr->u16FrameRate * stViAttr->u16WndHeight;

    if (pstCtrlPara->pstCurrConf->stSnsAttr->u16FrameRate == 12)
        pstCtrlPara->u32CurrHsf = 25 * stViAttr->u16WndHeight / 2;
    else
        pstCtrlPara->u32CurrHsf = pstCtrlPara->pstCurrConf->stSnsAttr->u16FrameRate * stViAttr->u16WndHeight;

    return 0;
}

static int SetSensorFmt(void *pvObj, int format)
{
    printf("SetSensorFmt format:%d\n", format);
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    SensorDevice_Close(pstCci);
    pstCtrlPara->u32SnsFmt = format;
    pstCtrlPara->u32WdrFlag = (format >> 16) & 1;

    SensorDevice_Init(pstCci, (0x60 >> 1), 2, pstCtrlPara->u8CciDeviceId);
    SensorReset(pvObj);

    if (pstCtrlPara->bInited) {
        FH_SINT32 s32Ret;
        FH_UINT32 i;
        fh_printf("sc3336p 20230907  0x%x \n", format);

        s32Ret = getSensorConf(pvObj);

        if (s32Ret)
            return s32Ret;

        SetSensorMipiCfg(pvObj);
        for (i = 0; i < pstCtrlPara->pstCurrConf->u32ConfSize; i++) {
            Sensor_Write(pstCci, pstCtrlPara->pstCurrConf->pu16CciConf[2 * i],
                         pstCtrlPara->pstCurrConf->pu16CciConf[2 * i + 1]);
        }
#if 0
        //usleep(10000);
        fh_printf("@@@@@@finished\n\n");
        if (enable)
        {
            FH_UINT32 tmp,flip, mirror;

            //Sensor_Write(0x0100, 0x00); //解决FLIP功能导致的画面割裂问题
            flip = status_flip & 0x1;
            mirror = status_mirror & 0x1;
            tmp = Sensor_Read(0x3221);
            tmp &=(~((0x03<<1)|(0x03<<5)));
            tmp = tmp | (flip << 6) | (flip << 5) | (mirror << 2) | (mirror << 1); 
            Sensor_Write(0x3221, tmp);
            //Sensor_Write(0x0100, 0x01); //解决FLIP功能导致的画面割裂问题

            enable = 0;
        }
#endif
    }
    else
        return -1;

    usleep(20000);
    //Sensor_Write(0x3937,0x80);//blc offset
   // Sensor_Write(0x3938,0x80);//blc offset

pstCtrlPara->u32Vcycle = (Sensor_Read(pstCci,0x320e) << 8) | (Sensor_Read(pstCci, 0x320f));
    FW_ori= (Sensor_Read(pstCci, 0x320c) << 8) | (Sensor_Read(pstCci, 0x320d));
    FH_ori= pstCtrlPara->u32Vcycle;
    pstCtrlPara->u32FullLineStd = pstCtrlPara->u32Vcycle;
    pstCtrlPara->u32MinRstLine = 8;

    pstCtrlPara->u32MaxIntt  =  pstCtrlPara->u32Vcycle - pstCtrlPara->u32MinRstLine;
    SetIntt(pvObj, pstCtrlPara->u32MaxIntt, 0);
    SetExposureRatio(pvObj, pstCtrlPara->u32CurrExposureRatio);
    SetGain(pvObj, pstCtrlPara->u32MinAgain, 0);
    pstCtrlPara->u32CurrSnsGain = pstCtrlPara->u32MinAgain;

    return 0;
}
/********************sensor flow related end*************************************/

/************************AE&WDR Related start************************************/
static int GetAEInfo(void *pvObj, Sensor_AE_INFO_S *stSnsAEInfo)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    stSnsAEInfo->currIntt = GetIntt(pvObj);
    stSnsAEInfo->currAGain = GetGain(pvObj);
    stSnsAEInfo->currHsf = pstCtrlPara->u32CurrHsf;
    stSnsAEInfo->currFrameH = pstCtrlPara->u32Vcycle;
    return 0;
}

static int GetAEDefault(void *pvObj, Sensor_AE_Default_S *sensAEDefault)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    sensAEDefault->minIntt = 2;
    sensAEDefault->maxIntt = pstCtrlPara->u32MaxIntt;
    sensAEDefault->minAGain = pstCtrlPara->u32MinAgain;
    sensAEDefault->maxAGain = 0x3ff7;
    sensAEDefault->fullLineStd = pstCtrlPara->u32FullLineStd;
    sensAEDefault->MinRstLine = pstCtrlPara->u32MinRstLine;

    return 0;
}

// GetGain仅在ae refresh时会重新获取,所以尽量直接从sensor端获取,保证intt的正确性
static int GetGain(void *pvObj)
{
    // SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SensorDevice* pstCci = _sns_get_cci_(pvObj);
    FH_UINT16 d_gain = 0, a_gain = 0;
    FH_UINT16 gain_3e06 = Sensor_Read(pstCci, 0x3e06);
    FH_UINT16 gain_3e07 = Sensor_Read(pstCci, 0x3e07);
    FH_UINT16 gain_3e09 = Sensor_Read(pstCci, 0x3e09);

    if (gain_3e09 == 0x00)
        a_gain = 64;
    else if (gain_3e09 == 0x40)
        a_gain = 97;
    else if (gain_3e09 == 0x48)
        a_gain = 194;
    else if (gain_3e09 == 0x49)
        a_gain = 389;
    else if (gain_3e09 == 0x4b)
        a_gain = 778;
    else if (gain_3e09 == 0x4f)
        a_gain = 1556;
    else if (gain_3e09 == 0x5f)
        a_gain = 3112;

    FH_UINT8 i = 0; //i: 1, 2, 3, 4
    FH_UINT8 temp = gain_3e06 + 1;
    for (i = 0; (temp >> i) > 0; i++)
    {
    };
    d_gain = (gain_3e07 >> 1) << (i - 1);
    FH_UINT32 total_gain = a_gain * d_gain >> 6;
    return total_gain;
}

static int CalcSnsValidGain(void *pvObj, FH_UINT32 *u32Gain)
{
    (void)pvObj;
    (void)u32Gain;
    return 0;
}

static int SetGain(void *pvObj, FH_UINT32 again, FH_UINT8 fNo)
{
    (void)fNo;
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SensorDevice* pstCci = _sns_get_cci_(pvObj);
    FH_UINT32 total_gain = again;
    FH_UINT16 a_gain = 0, d_gain = 0;
    total_gain = total_gain < 0x40 ? 0x40 : total_gain;
    FH_UINT16 gain_3e09 = 0x00;

    if (total_gain < 97)
    {
        gain_3e09 = 0x00;
        a_gain = 64;
    }
    else if (total_gain < 194)
    {
        gain_3e09 = 0x40;
        a_gain = 97;
    }
    else if (total_gain < 389)
    {
        gain_3e09 = 0x48;
        a_gain = 194;
    }
    else if (total_gain < 778)
    {
        gain_3e09 = 0x49;
        a_gain = 389;
    }
    else if (total_gain < 1556)
    {
        gain_3e09 = 0x4b;
        a_gain = 778;
    }
    else if (total_gain < 3112)
    {
        gain_3e09 = 0x4f;
        a_gain = 1556;
    }
    else
    {
        gain_3e09 = 0x5f;
        a_gain = 3112;
    }

    d_gain = (total_gain << 6) / a_gain;
    if (d_gain % 2)
        d_gain++;

    d_gain = d_gain < 64 ? 64 : d_gain;
    d_gain = d_gain > 1008 ? 1008 : d_gain;

    FH_UINT8 i = 0;
    for (i = 0; (d_gain >> i) > 0; i++)
    {
    };
    FH_UINT16 gain_3e06 = (1 << (i - 7)) - 1;
    FH_UINT16 gain_3e07 = (d_gain << 1) >> (i - 7);

    Sensor_Write(pstCci, 0x3e06, gain_3e06);
    Sensor_Write(pstCci, 0x3e07, gain_3e07);
    Sensor_Write(pstCci, 0x3e09, gain_3e09);
    pstCtrlPara->u32CurrSnsGain = total_gain;
    return 0;
}

static int GetIntt(void *pvObj)
{
    // SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SensorDevice* pstCci = _sns_get_cci_(pvObj);
    FH_UINT8 Reg3e02 = (Sensor_Read(pstCci, 0x3e02) & 0xf0) >> 4;
    FH_UINT8 Reg3e01 = Sensor_Read(pstCci, 0x3e01) & 0xff;
    FH_UINT8 Reg3e00 = Sensor_Read(pstCci, 0x3e00) & 0x0f;

    return ((Reg3e00 << 12) | (Reg3e01 << 4) | (Reg3e02));
}

static int CalcSnsValidIntt(void *pvObj, FH_UINT32 *u32Intt)
{
    (void)pvObj;
    (void)u32Intt;
    return 0;
}
// static int Set_short_exp_intt(void *pvObj, FH_UINT32 intt)
// {
//     (void)pvObj;
//     (void)intt;
//     return 0;
// }

static int SetIntt(void *pvObj, FH_UINT32 intt, FH_UINT8 fNo)
{
    (void)fNo;
    FH_UINT8 exposure_3e00 = 0x00;
    FH_UINT8 exposure_3e01 = 0x00;
    FH_UINT8 exposure_3e02 = 0x10;
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    if (intt < 2) {
        intt = 2;
    }

    pstCtrlPara->u32CurrSnsIntt = intt;
    exposure_3e00 = (intt >> 12) & 0xf;
    exposure_3e01 = (intt >> 4) & 0xff;
    exposure_3e02 = (intt & 0xf) << 4;

    Sensor_Write(pstCci, 0x3e02, exposure_3e02);
    Sensor_Write(pstCci, 0x3e01, exposure_3e01);
    Sensor_Write(pstCci, 0x3e00, exposure_3e00);

    if (pstCtrlPara->u32WdrFlag) {

        //FH_UINT32 short_exp_intt;
        //short_exp_intt = MAX(1, ((MIN(intt, sns_cfg.max_intt) << 4) + (sns_cfg.curr_exposure_ratio >> 1)) / sns_cfg.curr_exposure_ratio);
        //Set_short_exp_intt(short_exp_intt);
    }
    return 0;
}

static int SetSnsFrameH(void *pvObj, FH_UINT32 frameH)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    Sensor_Write(pstCci, 0x320e, (frameH >> 8) & 0x7f); //{16’h320e[6:0],16’h320f}
    Sensor_Write(pstCci, 0x320f, frameH & 0xff);
    pstCtrlPara->u32Vcycle = frameH;
    return 0;
}

static int SetExposureRatio(void *pvObj, FH_UINT32 exposure_ratio)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    pstCtrlPara->u32CurrExposureRatio = exposure_ratio;
    return 0;
}

static int GetExposureRatio(void *pvObj, FH_UINT32 *exposure_ratio)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    *exposure_ratio = (pstCtrlPara->u32CurrSnsIntt << 4) / MAX(1, pstCtrlPara->u32CurrSnsShortIntt);
    return 0;
}
/************************AE&WDR Related End************************************/

/************************sensor attr related start*****************************/
static int GetSensorAttribute(void *pvObj, char *name, FH_UINT32 *value)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    if (strcmp(name, "WDR") == 0) {
        *value = pstCtrlPara->u32WdrFlag;
        return 0;
    }
    return -1;
}
/************************sensor attr related end*******************************/

/***********************fastboot related start*********************************/
static int Sensor_Kick(void *pvObj)
{
    (void)pvObj;
    return 0;
}
/***********************fastboot related end**********************************/

/************************sensor func related start****************************/
static int SetSensorReg(void *pvObj, unsigned short addr, unsigned short data)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    Sensor_Write(pstCci, addr, data);
    return 0;
}

static int GetSensorReg(void *pvObj, unsigned short addr, unsigned short *data)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    *data = Sensor_Read(pstCci, addr);
    return 0;
}

static int GetSensorFlipMirror(void *pvObj, FH_UINT32 *sensor_en_stat)
{
    FH_UINT8 mirror, flip;
    SensorDevice *pstCci = _sns_get_cci_(pvObj);

    flip = ((Sensor_Read(pstCci, 0x3221) >> 5) & 0x1);
    mirror = ((Sensor_Read(pstCci, 0x3221) >> 1) & 0x1);
    *sensor_en_stat = (mirror << 1) | flip;
    //fh_printf("now mirror %d, flip %d\n", mirror, flip);
    return 0;
}
#if 0
static int SetFps(FH_UINT32 fps)
{
    if(fps<31)
    {
        FH_UINT32 FW_new=FW_ori;
        FH_UINT32 FH_new=FH_ori;
        FH_new=FH_ori*FPS_ori/fps;
        sns_cfg.v_cycle=FH_new;
        sns_cfg.fullLineStd=sns_cfg.v_cycle;
        sns_cfg.max_intt = sns_cfg.v_cycle - sns_cfg.MinRstLine;
        Sensor_Write(0x320c, (FW_new >> 8) & 0x7f); //{16’h320e[6:0],16’h320f}
        Sensor_Write(0x320d, FW_new & 0xff);
        Sensor_Write(0x320e, (FH_new >> 8) & 0x7f); //{16’h320e[6:0],16’h320f}
        Sensor_Write(0x320f, FH_new & 0xff);
        fh_printf("@@Set FPS to %d\n",fps);
    }
    else if(fps<301)
    {
        float Fps_float;
        Fps_float=(float)fps;
        Fps_float=Fps_float/10;
        FH_UINT32 FW_new=FW_ori;
        FH_UINT32 FH_new=FH_ori;
        FH_new=10*FH_ori*FPS_ori/fps;
        sns_cfg.v_cycle=FH_new;
        sns_cfg.fullLineStd=sns_cfg.v_cycle;
        sns_cfg.max_intt = sns_cfg.v_cycle - sns_cfg.MinRstLine;
        Sensor_Write(0x320c, (FW_new >> 8) & 0x7f); //{16’h320e[6:0],16’h320f}
        Sensor_Write(0x320d, FW_new & 0xff);
        Sensor_Write(0x320e, (FH_new >> 8) & 0x7f); //{16’h320e[6:0],16’h320f}
        Sensor_Write(0x320f, FH_new & 0xff);
        fh_printf("@@Set FPS to %2.2f\n",Fps_float);
    }
    else
    {
        fh_printf("@@ FPS error!\n");
    }
    
    return 0;
}
#endif

static int SensorCommonIf(void *pvObj, FH_UINT32 cmd, ISP_SENSOR_COMMON_CMD_DATA0* data0, ISP_SENSOR_COMMON_CMD_DATA1* data1)
{
    (void)pvObj;
    (void)cmd;
    (void)data0;
    (void)data1;
    int ret = -1;
#if 0
    int ret = -1;
    FH_UINT32 fps;
    switch (cmd)
    {
    case 0x101:
        status_mirror = *data0->dw;
        status_flip = *data1->dw;
        enable = 1;
        ret = 0;
      //  fh_printf("ori mirror %d, flip %d\n", status_mirror, status_flip);
        break;
    case 0x2000:
        fps=data0->dw[0];
        if(fps>1)
        {
            SetFps(fps);
        }
        else
        {
            //SetFps(FPS_ori);
        }
        break;
    default:
        break;
    }
#endif
    printf("%s:%d, not support\n", __func__, __LINE__);
    return ret;

}
/***********************sensor func related end**********************************/

/***********************adv api related start************************************/
static FH_UINT32 *GetMirrorFlipBayerFormat(void *pvObj)
{
    (void)pvObj;
    return 0;
}
static FH_UINT32 *GetUserSensorAwbGain(void *pvObj, FH_UINT32 idx)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    if (pstCtrlPara->u32WdrFlag)
        return (AwbGainWdr)[idx];
    else
        return (AwbGain)[idx];
}
static FH_UINT32 *GetSensorLtmCurve(void *pvObj, FH_UINT32 idx)
{ // printf("------GetSensorLtmCurve-------\n");
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    if (!pstCtrlPara->u32WdrFlag) {
        switch (idx) {
        case 0:
            return (LtmMapping00);
            break;
        case 1:
            return (LtmMapping01);
            break;
        case 2:
            return (LtmMapping02);
            break;
        case 3:
            return (LtmMapping03);
            break;
        case 4:
            return (LtmMapping04);
            break;
        case 5:
            return (LtmMapping05);
            break;
        case 6:
            return (LtmMapping06);
            break;
        case 7:
            return (LtmMapping07);
            break;
        case 8:
            return (LtmMapping08);
            break;
        case 9:
            return (LtmMapping09);
            break;
        case 10:
            return (LtmMapping10);
        }
        return (LtmMapping00);
    }
    return 0;
}
/***********************adv api related end*************************************/

static int SetSensorFlipMirror(void *pvObj, FH_UINT32 sensor_en_stat)
{
    SensorDevice* pstCci = _sns_get_cci_(pvObj);
    FH_UINT32 tmp, mirror, flip;

    mirror = (sensor_en_stat >> 1) & 0x1;
    flip = sensor_en_stat & 0x1;
    tmp =  Sensor_Read(pstCci, 0x3221);
    tmp &=(~((0x03<<1)|(0x03<<5)));
	tmp = tmp | (flip << 6) | (flip << 5) | (mirror << 2) | (mirror << 1); 

    Sensor_Write(pstCci, 0x3221, tmp);
    return 0;
}
/***********************not used for now start**********************************/
static int SetSensorAwbGain(void *pvObj, FH_UINT32 *awb_gain)
{
    (void)pvObj;
    (void)awb_gain;
    return 0;
}
static int GetSensorAwbGain(void *pvObj, FH_UINT32 *awb_gain)
{
    (void)pvObj;
    (void)awb_gain;
    return 0;
}
static int SetSensorIris(void *pvObj, FH_UINT32 iris)
{
    (void)pvObj;
    (void)iris;
    return 0;
}
/***********************not used for now end************************************/

#define SENSOR_SC3336P_MIPI "sc3336p_mipi"
struct isp_sensor_if sc3336p_mipi_sensor_if = {
    .name = SENSOR_SC3336P_MIPI,
    .get_vi_attr = GetSensorViAttr,
    .set_flipmirror = SetSensorFlipMirror,
    .get_flipmirror = GetSensorFlipMirror,
    .set_iris = SetSensorIris,
    .init = Sensor_Init,
    .reset = SensorReset,
    .deinit = Sensor_DeInit,
    .set_sns_fmt = SetSensorFmt,
    .kick = Sensor_Kick,
    .set_sns_reg = SetSensorReg,
    .set_exposure_ratio = SetExposureRatio,
    .get_exposure_ratio = GetExposureRatio,
    .get_sensor_attribute = GetSensorAttribute,
    .set_lane_num_max = SetLaneNumMax,
    .get_sns_reg = GetSensorReg,
    .get_awb_gain = GetSensorAwbGain,
    .set_awb_gain = SetSensorAwbGain,
    .data = 0,
    .common_if = SensorCommonIf,
    .get_sns_ae_default = GetAEDefault,
    .get_sns_ae_info = GetAEInfo,
    .set_sns_intt = SetIntt,
    .calc_sns_valid_intt = CalcSnsValidIntt,
    .set_sns_gain = SetGain,
    .calc_sns_valid_gain = CalcSnsValidGain,
    .set_sns_frame_height = SetSnsFrameH,
    .get_sensor_mirror_flip_bayer_format = GetMirrorFlipBayerFormat,
    .get_user_awb_gain = GetUserSensorAwbGain,
    .get_user_ltm_curve = GetSensorLtmCurve,
    .is_sensor_connect = Sensor_Isconnect,
};

struct isp_sensor_if *NAME(Sensor_Create)()
{
    SensorCreateInfo info = {
        .pstSnsIf = &sc3336p_mipi_sensor_if,
        .pstSnsCtrlPara = &stSnsDefaultCtrlPara,
        .pVer = SENSOR_GIT_VERSION,
    };

    return Sensor_Create_Common(&info);
}

void NAME(Sensor_Destroy)(struct isp_sensor_if *s_if)
{
    Sensor_Destroy_Common(s_if);
}

struct isp_sensor_if *(Sensor_Create_SC3336P)(void)
{
    SensorCreateInfo info = {
        .pstSnsIf = &sc3336p_mipi_sensor_if,
        .pstSnsCtrlPara = &stSnsDefaultCtrlPara,
        .pVer = SENSOR_GIT_VERSION,
    };
    return Sensor_Create_Common(&info);
}