/*
 * sc4336_mipi.c
 *
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "clock_ops.h"
#include "isp_sensor_if.h"
#include "sensor_ops.h"

#include "isp_enum.h"
#include "mipi_api.h"
#include "sc4336_mipi_param.h"
#include "sc4336_mipi_wdr_param.h"
#include "sensor.h"
#include "sc4336_mipi.h"
#include "sensor_std.h"

#define _NAME(n, s) n##_##s
#define NAME(n)     _NAME(n, sc4336_mipi)
static SNS_CTRL_PARA_S stSnsDefaultCtrlPara = {
                                       .bInited = 0,
                                       .u32CurrExposureRatio = 256, // 默认初始化为16倍
                                       .u32MinRstLine = 5,
                                       .u32MinAgain = 0x40,
                                   };

static int SetIntt(void *pvObj, FH_UINT32 intt, FH_UINT8 fNo);
static int GetIntt(void *pvObj);
static int SetGain(void *pvObj, FH_UINT32 again, FH_UINT8 fNo);
static int GetGain(void *pvObj);
static int SetExposureRatio(void *pvObj, FH_UINT32 exposure_ratio);
static int GetExposureRatio(void *pvObj, FH_UINT32 *exposure_ratio);
/********************sensor flow related start****************************/
static void SensorReset(void *pvObj)
{
    (void)pvObj;
    return;
}

static int Sensor_Init(void *pvObj, Sensor_Init_t *stInitConf)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    pstCtrlPara->u8CciDeviceId = stInitConf->u8CciDeviceId;
    pstCtrlPara->u8CsiDeviceId = stInitConf->u8CsiDeviceId;
    SensorDevice_Init(pstCci, (0x60 >> 1), 2, pstCtrlPara->u8CciDeviceId);
    SensorReset(pvObj);
    pstCtrlPara->bInited = 1;
    return 0;
}

static int Sensor_Isconnect(void *pvObj, FH_UINT8 deviceId)
{
    (void)pvObj;
    (void)deviceId;
    return 1;
#if 0
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    unsigned int sensor_chip_id = 0;
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    pstCtrlPara->u8CciDeviceId = deviceId;
    SensorDevice_Init(pstCci, (0x60 >> 1), 2, deviceId);

    sensor_chip_id = Sensor_Read(pstCci, 0x3108);
    sensor_chip_id |= Sensor_Read(pstCci, 0x3107) << 8;

    printf("sensor_chip_id=%x\n",sensor_chip_id);

    SensorDevice_Close(pstCci);

    if (sensor_chip_id == 0xcd2e)
        return 1;
    else {
        return 0;
    }
#endif
}

static int Sensor_DeInit(void *pvObj)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);

    SensorReset(pvObj);

    return SensorDevice_Close(pstCci);
}

static void SetSensorMipiCfg(void *pvObj)
{
    // printf("------SetSensorMipiCfg-------\n");
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    struct mipi_conf config;

    config.frq_range = pstCtrlPara->pstCurrConf->u8MipiClkRate;
    config.raw_type = RAW10;
    config.lane_num = pstCtrlPara->pstCurrConf->u8MipiLaneNum;
    config.mipi_id = pstCtrlPara->u8CsiDeviceId;
    // printf("config.frq_range=%d\n",config.frq_range);
    // printf("config.lane_num=%d\n",config.lane_num);
    if (pstCtrlPara->u32WdrFlag == 1) {
        config.sensor_mode = NOT_SONY_WDR_USE_WDR;
        config.lf_vc_id = 0;
        config.sf_vc_id = 1;
    } else {
        config.sensor_mode = NON_WDR;
        config.lf_vc_id = 0;
        config.sf_vc_id = 0;
    }
    mipi_init(&config);
    return;
}

static int SetLaneNumMax(void *pvObj, FH_UINT32 lane_num)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    pstCtrlPara->u32MaxLaneNum = lane_num;
    return 0;
}

static SNS_CONFIG_S *getSensorConfType(void *pvObj)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    pstCtrlPara->u32CurrConfSize = sizeof(stSnsConf0) / sizeof(SNS_CONFIG_S);
    return stSnsConf0;
}

static int getSensorConf(void *pvObj)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SNS_CONFIG_S *pstConfGrp = getSensorConfType(pvObj);
    FH_UINT32 i;

    for (i = 0; i < pstCtrlPara->u32CurrConfSize; i++) {
        if (pstCtrlPara->u32SnsFmt == pstConfGrp[i].u32SnsFmt) {
            if (pstCtrlPara->u32MaxLaneNum >= pstConfGrp[i].u8MipiLaneNum) {
                set_clk_rate(SNS_CLK_NAME(pstCtrlPara->u8CsiDeviceId), pstConfGrp[i].u32SnsClk);

                pstCtrlPara->pstCurrConf = &pstConfGrp[i];

                return FH_RET_SENSOR_OK;
            }
        }
    }

    return FH_RET_SENSOR_NO_SUPPORT_FORMAT;
}

static int GetSensorViAttr(void *pvObj, ISP_VI_ATTR_S *stViAttr)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    FH_SINT32 s32Ret;

    if (stViAttr == NULL)
        return FH_RET_SENSOR_NULL_POINTER;

    if (pstCtrlPara->pstCurrConf == NULL) {
        s32Ret = getSensorConf(pvObj);

        if (s32Ret != FH_RET_SENSOR_OK)
            return s32Ret;
    }

    stViAttr->u16WndWidth = pstCtrlPara->pstCurrConf->stSnsAttr->u16WndWidth;
    stViAttr->u16WndHeight = pstCtrlPara->pstCurrConf->stSnsAttr->u16WndHeight;
    stViAttr->u16InputWidth = pstCtrlPara->pstCurrConf->stSnsAttr->u16InputWidth;
    stViAttr->u16InputHeight = pstCtrlPara->pstCurrConf->stSnsAttr->u16InputHeight;
    stViAttr->u16PicWidth = pstCtrlPara->pstCurrConf->stSnsAttr->u16PicWidth;
    stViAttr->u16PicHeight = pstCtrlPara->pstCurrConf->stSnsAttr->u16PicHeight;
    stViAttr->u16OffsetX = pstCtrlPara->pstCurrConf->stSnsAttr->u16OffsetX;
    stViAttr->u16OffsetY = pstCtrlPara->pstCurrConf->stSnsAttr->u16OffsetY;
    stViAttr->enBayerType = pstCtrlPara->pstCurrConf->stSnsAttr->enBayerType;
    stViAttr->u16FrameRate = pstCtrlPara->pstCurrConf->stSnsAttr->u16FrameRate;
    pstCtrlPara->u32CurrHsf = pstCtrlPara->pstCurrConf->stSnsAttr->u16FrameRate * stViAttr->u16WndHeight;

    if (pstCtrlPara->pstCurrConf->stSnsAttr->u16FrameRate == 12)
        pstCtrlPara->u32CurrHsf = 25 * stViAttr->u16WndHeight / 2;
    else
        pstCtrlPara->u32CurrHsf = pstCtrlPara->pstCurrConf->stSnsAttr->u16FrameRate * stViAttr->u16WndHeight;

    return 0;
}

static int SetSensorFmt(void *pvObj, int format)
{
    printf("SetSensorFmt format:%d\n", format);
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    SensorDevice_Close(pstCci);
    pstCtrlPara->u32SnsFmt = format;
    pstCtrlPara->u32WdrFlag = (format >> 16) & 1;

    SensorDevice_Init(pstCci, (0x60 >> 1), 2, pstCtrlPara->u8CciDeviceId);
    SensorReset(pvObj);

    if (pstCtrlPara->bInited) {
        FH_SINT32 s32Ret;
        FH_UINT32 i;

        s32Ret = getSensorConf(pvObj);

        if (s32Ret)
            return s32Ret;

        // 先初始化好mipi再初始化sensor
        SetSensorMipiCfg(pvObj);

        for (i = 0; i < pstCtrlPara->pstCurrConf->u32ConfSize; i++) {
            Sensor_Write(pstCci, pstCtrlPara->pstCurrConf->pu16CciConf[2 * i],
                         pstCtrlPara->pstCurrConf->pu16CciConf[2 * i + 1]);
        }

    } else {
        printf("SetSensorFmt wrong\n");
        return -1;
    }

    pstCtrlPara->u32Vcycle = (Sensor_Read(pstCci, 0x320e) << 8) | (Sensor_Read(pstCci, 0x320f));
    pstCtrlPara->u32FullLineStd = pstCtrlPara->u32Vcycle; // 2*pstCtrlPara->u32Vcycle;

    if (pstCtrlPara->u32WdrFlag == 0) {
        pstCtrlPara->u32MinRstLine = 5;
    } else {
        pstCtrlPara->u32MinRstLine = pstCtrlPara->u32Vcycle / 7 + 5;
    }
    pstCtrlPara->u32MaxIntt = pstCtrlPara->u32Vcycle - pstCtrlPara->u32MinRstLine;
    SetIntt(pvObj, pstCtrlPara->u32MaxIntt, 0);
    SetExposureRatio(pvObj, pstCtrlPara->u32CurrExposureRatio);
    SetGain(pvObj, pstCtrlPara->u32MinAgain, 0);
    pstCtrlPara->u32CurrSnsGain = pstCtrlPara->u32MinAgain;

    return 0;
}
/********************sensor flow related end*************************************/

/************************AE&WDR Related start************************************/
static int GetAEInfo(void *pvObj, Sensor_AE_INFO_S *stSnsAEInfo)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    stSnsAEInfo->currIntt = GetIntt(pvObj);
    stSnsAEInfo->currAGain = GetGain(pvObj);
    stSnsAEInfo->currHsf = pstCtrlPara->u32CurrHsf;
    stSnsAEInfo->currFrameH = pstCtrlPara->u32Vcycle;
    return 0;
}

static int GetAEDefault(void *pvObj, Sensor_AE_Default_S *sensAEDefault)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    sensAEDefault->minIntt = 1;
    sensAEDefault->maxIntt = pstCtrlPara->u32MaxIntt;
    sensAEDefault->minAGain = pstCtrlPara->u32MinAgain;
    sensAEDefault->maxAGain = 0x3ff7;
    sensAEDefault->fullLineStd = pstCtrlPara->u32FullLineStd;
    sensAEDefault->MinRstLine = pstCtrlPara->u32MinRstLine;

    return 0;
}

// GetGain仅在ae refresh时会重新获取,所以尽量直接从sensor端获取,保证intt的正确性
static int GetGain(void *pvObj)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    // sensor gain因为计算不方便，暂就用变量
    return pstCtrlPara->u32CurrSnsGain;
}

static int CalcSnsValidGain(void *pvObj, FH_UINT32 *u32Gain)
{
    (void)pvObj;
    (void)u32Gain;
    return 0;
}

static int SetGain(void *pvObj, FH_UINT32 again, FH_UINT8 fNo)
{
    (void)fNo;
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SensorDevice* pstCci = _sns_get_cci_(pvObj);
    FH_UINT8 gain_reg_0x3e09 = 0x00;
    FH_UINT8 gain_reg_0x3e06 = 0x00;
    FH_UINT8 gain_reg_0x3e07 = 0x80;
    if (fNo == 0) {
        pstCtrlPara->u32CurrSnsGain = again;
        again = again << 4;
        again = again * 1000 / 1024;

        if (again < 2000) {
            gain_reg_0x3e09 = 0x00;
            gain_reg_0x3e06 = 0x00;
            gain_reg_0x3e07 = again * 128 / 1000;
        } else if (again < 4000) {
            gain_reg_0x3e09 = 0x08;
            gain_reg_0x3e06 = 0x00;
            gain_reg_0x3e07 = again * 128 / 2000;
        } else if (again < 8000) {
            gain_reg_0x3e09 = 0x09;
            gain_reg_0x3e06 = 0x00;
            gain_reg_0x3e07 = again * 128 / 4000;
        } else if (again < 16000) {
            gain_reg_0x3e09 = 0x0b;
            gain_reg_0x3e06 = 0x00;
            gain_reg_0x3e07 = again * 128 / 8000;
        } else if (again < 32000) {
            gain_reg_0x3e09 = 0x0f;
            gain_reg_0x3e06 = 0x00;
            gain_reg_0x3e07 = again * 128 / 16000;
        } else if (again < 32000 * 2) {
            gain_reg_0x3e09 = 0x1f;
            gain_reg_0x3e06 = 0x00;
            gain_reg_0x3e07 = again * 128 / 32000;
        } else if (again < 32000 * 4) // open dgain begin  max digital gain 4X
        {
            gain_reg_0x3e09 = 0x1f;
            gain_reg_0x3e06 = 0x01;
            gain_reg_0x3e07 = again * 128 / 32000 / 2;
        } else if (again < 32000 * 8) {
            gain_reg_0x3e09 = 0x1f;
            gain_reg_0x3e06 = 0x03;
            gain_reg_0x3e07 = again * 128 / 32000 / 4;
        } else if (again < 32000 * 15) {
            gain_reg_0x3e09 = 0x1f;
            gain_reg_0x3e06 = 0x07;
            gain_reg_0x3e07 = again * 128 / 32000 / 8;
        } else {
            gain_reg_0x3e09 = 0x1f;
            gain_reg_0x3e06 = 0x07;
            gain_reg_0x3e07 = 0xf0;
        }
        Sensor_Write(pstCci, 0x3e09, gain_reg_0x3e09);
        Sensor_Write(pstCci, 0x3e06, gain_reg_0x3e06);
        Sensor_Write(pstCci, 0x3e07, gain_reg_0x3e07);
    }

    return 0;
}

// GetIntt仅在ae refresh时会重新获取,所以尽量直接从sensor端获取,保证intt的正确性
static int GetIntt(void *pvObj)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    return pstCtrlPara->u32CurrSnsIntt;
}

static int CalcSnsValidIntt(void *pvObj, FH_UINT32 *u32Intt)
{
    (void)pvObj;
    (void)u32Intt;
    return 0;
}
// static int Set_short_exp_intt(void *pvObj, FH_UINT32 intt)
// {
//     (void)pvObj;
//     (void)intt;
//     return 0;
// }

static int SetIntt(void *pvObj, FH_UINT32 intt, FH_UINT8 fNo)
{
    (void)fNo;
    FH_UINT8 exposure_3e00 = 0x00;
    FH_UINT8 exposure_3e01 = 0x00;
    FH_UINT8 exposure_3e02 = 0x10;
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    if (intt < 2) {
        intt = 2;
    }

    pstCtrlPara->u32CurrSnsIntt = intt;
    intt = intt * 2;
    exposure_3e00 = (intt >> 12) & 0xf;
    exposure_3e01 = (intt >> 4) & 0xff;
    exposure_3e02 = (intt & 0xf) << 4;

    Sensor_Write(pstCci, 0x3e02, exposure_3e02);
    Sensor_Write(pstCci, 0x3e01, exposure_3e01);
    Sensor_Write(pstCci, 0x3e00, exposure_3e00);

    if (pstCtrlPara->u32WdrFlag) {
        // FH_UINT32 short_exp_intt;
        // short_exp_intt = MAX(1, ((MIN(intt, pstCtrlPara->u32MaxIntt) << 4) + (pstCtrlPara->u32CurrExposureRatio >>
        // 1)) / pstCtrlPara->u32CurrExposureRatio);
        // Set_short_exp_intt(pvObj, short_exp_intt);
    }
    return 0;
}

static int SetSnsFrameH(void *pvObj, FH_UINT32 frameH)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    Sensor_Write(pstCci, 0x320e, (frameH >> 8) & 0xff);
    Sensor_Write(pstCci, 0x320f, frameH & 0xff);
    // printf("enter case\n");
    pstCtrlPara->u32Vcycle = frameH;
    // printf("act frame%d\n",frameH);
    return 0;
}

static int SetExposureRatio(void *pvObj, FH_UINT32 exposure_ratio)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    pstCtrlPara->u32CurrExposureRatio = exposure_ratio;
    return 0;
}

static int GetExposureRatio(void *pvObj, FH_UINT32 *exposure_ratio)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    *exposure_ratio = (pstCtrlPara->u32CurrSnsIntt << 4) / MAX(1, pstCtrlPara->u32CurrSnsShortIntt);
    return 0;
}
/************************AE&WDR Related End************************************/

/************************sensor attr related start*****************************/
static int GetSensorAttribute(void *pvObj, char *name, FH_UINT32 *value)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    if (strcmp(name, "WDR") == 0) {
        *value = pstCtrlPara->u32WdrFlag;
        return 0;
    }
    return -1;
}
/************************sensor attr related end*******************************/

/***********************fastboot related start*********************************/
static int Sensor_Kick(void *pvObj)
{
    (void)pvObj;
    return 0;
}
/***********************fastboot related end**********************************/

/************************sensor func related start****************************/
static int SetSensorReg(void *pvObj, unsigned short addr, unsigned short data)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    Sensor_Write(pstCci, addr, data);
    return 0;
}

static int GetSensorReg(void *pvObj, unsigned short addr, unsigned short *data)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    *data = Sensor_Read(pstCci, addr);
    return 0;
}

static int GetSensorFlipMirror(void *pvObj, FH_UINT32 *sensor_en_stat)
{
    FH_UINT8 mirror, flip;
    SensorDevice *pstCci = _sns_get_cci_(pvObj);

    flip = ((Sensor_Read(pstCci, 0x3220) >> 6) & 0x1);
    mirror = !((Sensor_Read(pstCci, 0x3221) >> 2) & 0x1);
    *sensor_en_stat = (mirror << 1) | flip;

    printf("%s:%d support\n", __func__, __LINE__);
    return 0;
}

static int SensorCommonIf(void *pvObj, FH_UINT32 cmd, ISP_SENSOR_COMMON_CMD_DATA0* data0, ISP_SENSOR_COMMON_CMD_DATA1* data1)
{
    (void)pvObj;
    (void)cmd;
    (void)data0;
    (void)data1;
    int ret = -1;
#if 0
    switch (cmd)
    {
        case CMD_INIT:
            ret = 0;
            break;
        case CMD_MAP_INTT_GAIN:
            {
                MAP_INTT_GAIN_CFG *map_intt_gain;
                FH_UINT32 intt_new= 0, curr_frame_height= 0, des_frame_height= 0;
                FH_UINT32 curr_format_param = 0;
                FH_UINT32 des_format_param = 0;

                map_intt_gain = (MAP_INTT_GAIN_CFG*)data0;
                if(map_intt_gain == NULL)
                    return ret;
                switch (map_intt_gain->currFormat)
                {
                    case FORMAT_720P60:
                        curr_format_param = 60;
                        curr_frame_height = FRAM_H_720P60_2L;
                        break;
                    default:
                        ret = -1;
                }

                switch (map_intt_gain->desFormat)
                {
                    case FORMAT_400WP15:
                        des_format_param = 15;
                        des_frame_height = FRAM_H_400WP15_2L;
                        break;
                    case FORMAT_400WP25:
                        des_format_param = 25;
                        des_frame_height = FRAM_H_400WP25_2L;
                        break;
                    case FORMAT_400WP20:
                        des_format_param = 20;
                        des_frame_height = FRAM_H_400WP20_2L;
                        break;
                    default:
                        ret = -1;
                }
                intt_new = (des_format_param * des_frame_height * (FH_UINT64)(map_intt_gain->currIntt) * (FH_UINT64)(map_intt_gain->currTotalGain))/(curr_format_param * curr_frame_height);

                if(intt_new < (16 * 64))
                {
                    map_intt_gain->desIntt = MAX(1, intt_new >> 6);
                    map_intt_gain->desTotalGain = MAX(sns_cfg.minAGain, intt_new / (map_intt_gain->desIntt));
                }
                else
                {
                    intt_new = intt_new >> 6;
                    map_intt_gain->desIntt = MIN(pstCtrlPara->u32MaxIntt, intt_new);
                    map_intt_gain->desTotalGain = MAX(sns_cfg.minAGain, (intt_new<<6)/ map_intt_gain->desIntt);
                }
                ret = 0;
                break;
            }
        case CMD_GET_BLC_VALUE:
            {
                BLC_PARAM_CFG   *blc_param_cfg;
                blc_param_cfg = (BLC_PARAM_CFG*)data0;
                blc_param_cfg->blc_value = 0x10;
                ret = 0;
                break;
            }
        case CMD_GET_GPIO_PARAM:
            {

            //rt_kprintf("houliming CMD_GET_GPIO_PARAM test \n");
            
                GPIO_PARAM_CFG  *gpio_param_cfg;
                gpio_param_cfg = (GPIO_PARAM_CFG*)data0;
                gpio_param_cfg->gpio_enable = 1;
                gpio_param_cfg->gpio_time = (0x708 << 16) | 0x4b0;
                ret = 0;
                break;
            }
        default:
            ret = -1;
    }
#endif
    printf("%s:%d, not support\n", __func__, __LINE__);
    return ret;
}
/***********************sensor func related end**********************************/

/***********************adv api related start************************************/
static FH_UINT32 *GetMirrorFlipBayerFormat(void *pvObj)
{
    (void)pvObj;
    return 0;
}
static FH_UINT32 *GetUserSensorAwbGain(void *pvObj, FH_UINT32 idx)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    if (pstCtrlPara->u32WdrFlag)
        return (AwbGainWdr)[idx];
    else
        return (AwbGain)[idx];
}
static FH_UINT32 *GetSensorLtmCurve(void *pvObj, FH_UINT32 idx)
{ // printf("------GetSensorLtmCurve-------\n");
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    if (!pstCtrlPara->u32WdrFlag) {
        switch (idx) {
        case 0:
            return (LtmMapping00);
            break;
        case 1:
            return (LtmMapping01);
            break;
        case 2:
            return (LtmMapping02);
            break;
        case 3:
            return (LtmMapping03);
            break;
        case 4:
            return (LtmMapping04);
            break;
        case 5:
            return (LtmMapping05);
            break;
        case 6:
            return (LtmMapping06);
            break;
        case 7:
            return (LtmMapping07);
            break;
        case 8:
            return (LtmMapping08);
            break;
        case 9:
            return (LtmMapping09);
            break;
        case 10:
            return (LtmMapping10);
        }
        return (LtmMapping00);
    }
    return 0;
}
/***********************adv api related end*************************************/

static int SetSensorFlipMirror(void *pvObj, FH_UINT32 sensor_en_stat)
{
    SensorDevice* pstCci = _sns_get_cci_(pvObj);
    FH_UINT8 tmp1;
    FH_UINT8 flip, mirror;
    Sensor_Write(pstCci, 0x0100, 0x00); //解决FLIP功能导致的画面割裂问题,关闭sensor输出
    flip = sensor_en_stat & 0x1;
    mirror = (((sensor_en_stat >> 1)) & 0x1);

    tmp1 = Sensor_Read(pstCci, 0x3221) & 0x99;
    tmp1 = tmp1 | (flip << 6) | (flip << 5) | (mirror << 2) | (mirror << 1);

    Sensor_Write(pstCci, 0x3221, tmp1);
    Sensor_Write(pstCci, 0x0100, 0x01); //解决FLIP功能导致的画面割裂问题，恢复sensor输出
    return 0;
}
/***********************not used for now start**********************************/
static int SetSensorAwbGain(void *pvObj, FH_UINT32 *awb_gain)
{
    (void)pvObj;
    (void)awb_gain;
    return 0;
}
static int GetSensorAwbGain(void *pvObj, FH_UINT32 *awb_gain)
{
    (void)pvObj;
    (void)awb_gain;
    return 0;
}
static int SetSensorIris(void *pvObj, FH_UINT32 iris)
{
    (void)pvObj;
    (void)iris;
    return 0;
}
/***********************not used for now end************************************/

#define SENSOR_SC4336_MIPI "sc4336_mipi"
struct isp_sensor_if sc4336_mipi_sensor_if = {
    .name = SENSOR_SC4336_MIPI,
    .get_vi_attr = GetSensorViAttr,
    .set_flipmirror = SetSensorFlipMirror,
    .get_flipmirror = GetSensorFlipMirror,
    .set_iris = SetSensorIris,
    .init = Sensor_Init,
    .reset = SensorReset,
    .deinit = Sensor_DeInit,
    .set_sns_fmt = SetSensorFmt,
    .kick = Sensor_Kick,
    .set_sns_reg = SetSensorReg,
    .set_exposure_ratio = SetExposureRatio,
    .get_exposure_ratio = GetExposureRatio,
    .get_sensor_attribute = GetSensorAttribute,
    .set_lane_num_max = SetLaneNumMax,
    .get_sns_reg = GetSensorReg,
    .get_awb_gain = GetSensorAwbGain,
    .set_awb_gain = SetSensorAwbGain,
    .data = 0,
    .common_if = SensorCommonIf,
    .get_sns_ae_default = GetAEDefault,
    .get_sns_ae_info = GetAEInfo,
    .set_sns_intt = SetIntt,
    .calc_sns_valid_intt = CalcSnsValidIntt,
    .set_sns_gain = SetGain,
    .calc_sns_valid_gain = CalcSnsValidGain,
    .set_sns_frame_height = SetSnsFrameH,
    .get_sensor_mirror_flip_bayer_format = GetMirrorFlipBayerFormat,
    .get_user_awb_gain = GetUserSensorAwbGain,
    .get_user_ltm_curve = GetSensorLtmCurve,
    .is_sensor_connect = Sensor_Isconnect,
};

struct isp_sensor_if *NAME(Sensor_Create)()
{
    SensorCreateInfo info = {
        .pstSnsIf = &sc4336_mipi_sensor_if,
        .pstSnsCtrlPara = &stSnsDefaultCtrlPara,
        .pVer = SENSOR_GIT_VERSION,
    };

    return Sensor_Create_Common(&info);
}

void NAME(Sensor_Destroy)(struct isp_sensor_if *s_if)
{
    Sensor_Destroy_Common(s_if);
}

struct isp_sensor_if *(Sensor_Create_SC4336)(void)
{
    SensorCreateInfo info = {
        .pstSnsIf = &sc4336_mipi_sensor_if,
        .pstSnsCtrlPara = &stSnsDefaultCtrlPara,
        .pVer = SENSOR_GIT_VERSION,
    };
    return Sensor_Create_Common(&info);
}
