/*
 * sc4336p_mipi.c
 *
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
// #include "isp_func_def.h"

#include "isp_sensor_if.h"
#include "sensor_ops.h"
#include "clock_ops.h"

#include "isp_enum.h"
#include "mipi_api.h"
#include "sensor.h"
// #include "sensor_ops.c"
#include "sc4336p_mipi.h"
#include "sensor_std.h"

#define _NAME(n, s) n##_##s
#define NAME(n)     _NAME(n, sc4336p_mipi)

static SNS_CTRL_PARA_S stSnsDefaultCtrlPara = {
    .bInited = 0,
    .u32CurrExposureRatio = 256, // 默认初始化为16倍
    .u32MinRstLine = 5,
    .u32MinAgain = 0x40,
};
// static FH_UINT32 curr_sensor_gain=0x40;
static int SetIntt(void *pvObj, FH_UINT32 intt, FH_UINT8 fNo);
static int GetIntt(void *pvObj);
static int SetGain(void *pvObj, FH_UINT32 again, FH_UINT8 fNo);
static int GetGain(void *pvObj);
static int SetExposureRatio(void *pvObj, FH_UINT32 exposure_ratio);
static int GetExposureRatio(void *pvObj, FH_UINT32 *exposure_ratio);
/********************sensor flow related start****************************/
static void SensorReset(void *pvObj)
{
    (void)pvObj;
    return;
}
static FH_UINT8 sc4336p_i2c_addr = 0x60;

static int Sensor_Isconnect(void *pvObj, FH_UINT8 u8DeviceId)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    FH_UINT32 u32SnsChipId;
    pstCtrlPara->u8CciDeviceId = u8DeviceId;

    FH_UINT8 i2c_addr[2] = {
        0x60,
        0x64,
    };
    int i;
    for (i = 0; i < 2; i++) {
        SensorDevice_Init(pstCci, (i2c_addr[i] >> 1), 2, pstCtrlPara->u8CciDeviceId);

        Sensor_Write(pstCci, 0x440d, 0x10);
        Sensor_Write(pstCci, 0x4400, 0x11);
        usleep(12000);
        u32SnsChipId = Sensor_Read(pstCci, 0x3108);
        u32SnsChipId |= Sensor_Read(pstCci, 0x3107) << 8;

        SensorDevice_Close(pstCci);
        printf("u32SnsChipId 0x%x\n", u32SnsChipId);
        if (u32SnsChipId == 0x9c42) {
            printf("SC4336P i2c_addr=%x,u32SnsChipId=%x\n", i2c_addr[i], u32SnsChipId);
            sc4336p_i2c_addr = i2c_addr[i];
            return 1;
        } else {
            continue;
        }
    }
    return 0;
}
static int Sensor_Init(void *pvObj, Sensor_Init_t *stInitConf)
{
    // printf("============flag\n");
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    pstCtrlPara->u8CciDeviceId = stInitConf->u8CciDeviceId;
    pstCtrlPara->u8CsiDeviceId = stInitConf->u8CsiDeviceId;
#ifdef XGM
    pstCtrlPara->bGrpSync = stInitConf->bGrpSync;
#endif
    Sensor_Isconnect(pvObj, pstCtrlPara->u8CciDeviceId);
    SensorDevice_Init(pstCci, (sc4336p_i2c_addr >> 1), 2, pstCtrlPara->u8CciDeviceId);
    SensorReset(pvObj);
    pstCtrlPara->bInited = 1;
    return 0;
}

static int Sensor_DeInit(void *pvObj)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    SensorReset(pvObj);

    return SensorDevice_Close(pstCci);
}

static int SetLaneNumMax(void *pvObj, FH_UINT32 lane_num)
{
    (void)pvObj;
    (void)lane_num;
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
#ifdef FPGA_ENV
    pstCtrlPara->u32MaxLaneNum = 4;
#else
    pstCtrlPara->u32MaxLaneNum = 4;
#endif
    return 0;
}

static SNS_CONFIG_S *getSensorConfType(void *pvObj)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    pstCtrlPara->u32CurrConfSize = sizeof(stSnsConf0) / sizeof(SNS_CONFIG_S);
    return stSnsConf0;
}

static int getSensorConf(void *pvObj)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SNS_CONFIG_S *pstConfGrp = getSensorConfType(pvObj);
    int i;

    for (i = 0; i < (int)pstCtrlPara->u32CurrConfSize; i++) {
        if (pstCtrlPara->u32SnsFmt == pstConfGrp[i].u32SnsFmt) {
            if (pstCtrlPara->u32MaxLaneNum >= pstConfGrp[i].u8MipiLaneNum) {
                set_clk_rate(SNS_CLK_NAME(pstCtrlPara->u8CsiDeviceId), pstConfGrp[i].u32SnsClk);

                pstCtrlPara->pstCurrConf = &pstConfGrp[i];

                return FH_RET_SENSOR_OK;
            }
        }
    }

    return FH_RET_SENSOR_NO_SUPPORT_FORMAT;
}

static void SetSensorMipiCfg(void *pvObj)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    struct mipi_conf config;
    config.frq_range = pstCtrlPara->pstCurrConf->u8MipiClkRate;
    config.raw_type = RAW10;
    config.lane_num = pstCtrlPara->pstCurrConf->u8MipiLaneNum;
    config.mipi_id = pstCtrlPara->u8CsiDeviceId;

    if (pstCtrlPara->u32WdrFlag == 1) {
        config.sensor_mode = NOT_SONY_WDR_USE_WDR;
        config.lf_vc_id = 0;
        config.sf_vc_id = 1;
    } else {
        config.sensor_mode = NON_WDR; // NOT_SONY_WDR_USE_WDR;
        config.lf_vc_id = 0;
        config.sf_vc_id = 0;
    }
    mipi_init(&config);
    return;
}

static int GetSensorViAttr(void *pvObj, ISP_VI_ATTR_S *stViAttr)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    FH_SINT32 s32Ret;

    if (stViAttr == NULL)
        return FH_RET_SENSOR_NULL_POINTER;

    if (pstCtrlPara->pstCurrConf == NULL) {
        s32Ret = getSensorConf(pvObj);

        if (s32Ret != FH_RET_SENSOR_OK)
            return s32Ret;
    }

    stViAttr->u16WndWidth = pstCtrlPara->pstCurrConf->stSnsAttr->u16WndWidth;
    stViAttr->u16WndHeight = pstCtrlPara->pstCurrConf->stSnsAttr->u16WndHeight;
    stViAttr->u16InputWidth = pstCtrlPara->pstCurrConf->stSnsAttr->u16InputWidth;
    stViAttr->u16InputHeight = pstCtrlPara->pstCurrConf->stSnsAttr->u16InputHeight;
    stViAttr->u16PicWidth = pstCtrlPara->pstCurrConf->stSnsAttr->u16PicWidth;
    stViAttr->u16PicHeight = pstCtrlPara->pstCurrConf->stSnsAttr->u16PicHeight;
    stViAttr->u16OffsetX = pstCtrlPara->pstCurrConf->stSnsAttr->u16OffsetX;
    stViAttr->u16OffsetY = pstCtrlPara->pstCurrConf->stSnsAttr->u16OffsetY;
    stViAttr->enBayerType = pstCtrlPara->pstCurrConf->stSnsAttr->enBayerType;
    pstCtrlPara->u32CurrHsf = pstCtrlPara->pstCurrConf->stSnsAttr->u16FrameRate * stViAttr->u16WndHeight;

    if (pstCtrlPara->pstCurrConf->stSnsAttr->u16FrameRate == 12)
        pstCtrlPara->u32CurrHsf = 25 * stViAttr->u16WndHeight / 2;
    else
        pstCtrlPara->u32CurrHsf = pstCtrlPara->pstCurrConf->stSnsAttr->u16FrameRate * stViAttr->u16WndHeight;

    return 0;
}

static int SetSensorFmt(void *pvObj, FH_SINT32 s32Format)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SensorDevice *pstCci = _sns_get_cci_(pvObj);

    SensorDevice_Close(pstCci);
    pstCtrlPara->u32SnsFmt = s32Format;
    pstCtrlPara->u32WdrFlag = (s32Format >> 16) & 1;

    SensorDevice_Init(pstCci, (sc4336p_i2c_addr >> 1), 2, pstCtrlPara->u8CciDeviceId);
    SensorReset(pvObj);

    if (pstCtrlPara->bInited) {
        FH_SINT32 i, s32Ret;

        s32Ret = getSensorConf(pvObj);

        if (s32Ret)
            return s32Ret;

        SetSensorMipiCfg(pvObj);

        for (i = 0; i < (int)pstCtrlPara->pstCurrConf->u32ConfSize; i++) {
            Sensor_Write(pstCci, pstCtrlPara->pstCurrConf->pu16CciConf[2 * i],
                         pstCtrlPara->pstCurrConf->pu16CciConf[2 * i + 1]);
        }

    } else {
        return -1;
    }

    // pstCtrlPara->u32Vcycle = Sensor_Read(pstCci, 0x320e) << 8;
    // pstCtrlPara->u32Vcycle |= Sensor_Read(pstCci, 0x320f);
    pstCtrlPara->u32Vcycle = pstCtrlPara->pstCurrConf->stSnsAttr->u16WndHeight;
    pstCtrlPara->u32FullLineStd = pstCtrlPara->u32Vcycle;

    if (pstCtrlPara->u32WdrFlag == 0) {
        pstCtrlPara->u32MinRstLine = 5;
    } else {
        pstCtrlPara->u32MinRstLine = 5;
    }
    pstCtrlPara->u32MaxIntt = pstCtrlPara->u32Vcycle - pstCtrlPara->u32MinRstLine;
    usleep(5000);
    SetIntt(pvObj, pstCtrlPara->u32MaxIntt, 0);
    SetExposureRatio(pvObj, pstCtrlPara->u32CurrExposureRatio);
    SetGain(pvObj, pstCtrlPara->u32MinAgain, 0);
    pstCtrlPara->u32CurrSnsGain = pstCtrlPara->u32MinAgain;

    return 0;
}
/********************sensor flow related end*************************************/

/************************AE&WDR Related start************************************/

static int GetAEInfo(void *pvObj, Sensor_AE_INFO_S *stSnsAEInfo)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    stSnsAEInfo->currIntt = GetIntt(pvObj);
    stSnsAEInfo->currAGain = GetGain(pvObj);
    stSnsAEInfo->currHsf = pstCtrlPara->u32CurrHsf;
    stSnsAEInfo->currFrameH = pstCtrlPara->u32Vcycle;
    return 0;
}

static int GetAEDefault(void *pvObj, Sensor_AE_Default_S *sensAEDefault)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    sensAEDefault->minIntt = 1;
    sensAEDefault->maxIntt = pstCtrlPara->u32MaxIntt;
    sensAEDefault->minAGain = pstCtrlPara->u32MinAgain;
    sensAEDefault->maxAGain = 0x7dff;
    sensAEDefault->fullLineStd = pstCtrlPara->u32FullLineStd;
    sensAEDefault->MinRstLine = pstCtrlPara->u32MinRstLine;

    return 0;
}

// GetGain仅在ae refresh时会重新获取,所以尽量直接从sensor端获取,保证intt的正确性
static int GetGain(void *pvObj)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    return pstCtrlPara->u32CurrSnsGain;
}

static int CalcSnsValidGain(void *pvObj, FH_UINT32 *u32Gain)
{
    (void)pvObj;
    (void)u32Gain;
    return 0;
}

static int SetGain(void *pvObj, FH_UINT32 again, FH_UINT8 fNo)
{
    (void)fNo;
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SensorDevice *pstCci = _sns_get_cci_(pvObj);

    pstCtrlPara->u32CurrSnsGain = again;

    unsigned char gain_0x3e06 = 0x00;
    unsigned char gain_0x3e07 = 0x80;
    unsigned char gain_0x3e09 = 0x00;
    // UINT32 gainShift = 0;
    if (again < 2 * 64) // again 1x dgain 1x-2x
    {
        gain_0x3e09 = 0x00;
        gain_0x3e06 = 0x00;
        gain_0x3e07 = again * 2;
    } else if (again < 2 * 64 * 2) // again 2x dgain 1x-2x
    {
        gain_0x3e09 = 0x08;
        gain_0x3e06 = 0x00;
        gain_0x3e07 = again * 2 / 2;
    } else if (again < 2 * 64 * 4) // again 4x dgain 1x-2x
    {
        gain_0x3e09 = 0x09;
        gain_0x3e06 = 0x00;
        gain_0x3e07 = again / 2;
    } else if (again < 2 * 64 * 8) // again 8x dgain 1x-2x
    {
        gain_0x3e09 = 0x0b;
        gain_0x3e06 = 0x00;
        gain_0x3e07 = again / 2 / 2;
    } else if (again < 2 * 64 * 16) // again 16x dgain 1x-2x
    {
        gain_0x3e09 = 0x0f;
        gain_0x3e06 = 0x00;
        gain_0x3e07 = again / 2 / 4;
    } else if (again < 2 * 64 * 32) // again 32x dgain 1x-2x
    {
        gain_0x3e09 = 0x1f;
        gain_0x3e06 = 0x00;
        gain_0x3e07 = again / 2 / 8;
    }

    else // again 32x dgain 4x
    {
        gain_0x3e09 = 0x1f;
        gain_0x3e06 = 0x03;
        gain_0x3e07 = 0x80;
    }
    // high frm GAIN
    Sensor_Write(pstCci, 0x3e06, gain_0x3e06);
    Sensor_Write(pstCci, 0x3e07, gain_0x3e07);
    Sensor_Write(pstCci, 0x3e09, gain_0x3e09);

    return 0;
}

// GetIntt仅在ae refresh时会重新获取,所以尽量直接从sensor端获取,保证intt的正确性
static int GetIntt(void *pvObj)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    return pstCtrlPara->u32CurrSnsIntt;
}

static int CalcSnsValidIntt(void *pvObj, FH_UINT32 *u32Intt)
{
    (void)pvObj;
    (void)u32Intt;
    return 0;
}

// static int Set_short_exp_intt(void* pvObj, FH_UINT32 s_intt)
// {
//     SNS_CTRL_PARA_S*    pstCtrlPara = _sns_get_ctrl_para(pvObj);
//     SensorDevice*       pstCci      = _sns_get_cci_(pvObj);
//     pstCtrlPara->u32CurrSnsShortIntt = s_intt;

//     return 0;
// }

static int SetIntt(void *pvObj, FH_UINT32 intt, FH_UINT8 fNo)
{
    (void)fNo;
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SensorDevice *pstCci = _sns_get_cci_(pvObj);

    pstCtrlPara->u32CurrSnsIntt = intt;

    if (intt < 1) {
        intt = 1; // to avoid 0
    }
    FH_UINT8 exposure_3e00 = 0x00;
    FH_UINT8 exposure_3e01 = 0x00;
    FH_UINT8 exposure_3e02 = 0x10;

    exposure_3e00 = (intt >> 12) & 0xf;
    exposure_3e01 = (intt >> 4) & 0xff;
    exposure_3e02 = (intt & 0xf) << 4;

    Sensor_Write(pstCci, 0x3e02, exposure_3e02);
    Sensor_Write(pstCci, 0x3e01, exposure_3e01);
    Sensor_Write(pstCci, 0x3e00, exposure_3e00);
    return 0;
}

static int SetSnsFrameH(void *pvObj, FH_UINT32 frameH)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    if (frameH > 0) {
        Sensor_Write(pstCci, 0x320e, (frameH >> 8) & 0xff);
        Sensor_Write(pstCci, 0x320f, frameH & 0xff);
        pstCtrlPara->u32Vcycle = frameH;
        // SetIntt(pvObj, frameH-5,0);
    }

    return 0;
}

static int SetExposureRatio(void *pvObj, FH_UINT32 exposure_ratio)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    pstCtrlPara->u32CurrExposureRatio = exposure_ratio;
    return 0;
}

static int GetExposureRatio(void *pvObj, FH_UINT32 *exposure_ratio)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    *exposure_ratio = (pstCtrlPara->u32CurrSnsIntt << 4) / MAX(1, pstCtrlPara->u32CurrSnsShortIntt);
    return 0;
}
/************************AE&WDR Related End************************************/

/************************sensor attr related start*****************************/
static int GetSensorAttribute(void *pvObj, char *name, FH_UINT32 *value)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    if (strcmp(name, "WDR") == 0) {
        *value = pstCtrlPara->u32WdrFlag;
        return 0;
    }

    return -1;
}
/************************sensor attr related end*******************************/

/***********************fastboot related start*********************************/
static int Sensor_Kick(void *pvObj)
{
    SNS_CTRL_PARA_S *pstCtrlPara = _sns_get_ctrl_para(pvObj);

    if (pstCtrlPara->bInited) {
        // enable sensor output
    }
    return 0;
}

/************************sensor func related start****************************/
static int SetSensorReg(void *pvObj, unsigned short addr, unsigned short data)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    Sensor_Write(pstCci, addr, data);
    return 0;
}

static int GetSensorReg(void *pvObj, unsigned short addr, unsigned short *data)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    *data = Sensor_Read(pstCci, addr);
    return 0;
}

static int SetSensorFlipMirror(void *pvObj, FH_UINT32 sensor_en_stat)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);

    FH_UINT8 tmp;
    FH_UINT8 flip, mirror;
    Sensor_Write(pstCci, 0x0100, 0x00); //解决FLIP功能导致的画面割裂问题,关闭sensor输出
    flip = sensor_en_stat & 0x1;
    mirror = (sensor_en_stat >> 1) & 0x1;
    tmp = (mirror << 1) | (mirror << 2) | (flip << 5) | (flip << 6);
    // tmp1 = Sensor_Read(pstCci, 0x3820);
    // tmp2 = Sensor_Read(pstCci, 0x3821);

    // tmp1 = (tmp1 & 0xfb) | (flip << 2);
    // tmp2 = (tmp2 & 0xfb) | (mirror << 2);

    Sensor_Write(pstCci, 0x3221, tmp);

    Sensor_Write(pstCci, 0x0100, 0x01); //解决FLIP功能导致的画面割裂问题，恢复sensor输出
    return 0;
}

static int GetSensorFlipMirror(void *pvObj, FH_UINT32 *sensor_en_stat)
{
    SensorDevice *pstCci = _sns_get_cci_(pvObj);
    FH_UINT8 mirror, flip, tmp;
    tmp = Sensor_Read(pstCci, 0x3221);
    mirror = (tmp >> 1) & 0x1;
    flip = (tmp >> 5) & 0x01;
    *sensor_en_stat = (mirror << 1) | flip;

    return 0;
}

static int SensorCommonIf(void *pvObj, FH_UINT32 cmd, ISP_SENSOR_COMMON_CMD_DATA0 *data0,
                          ISP_SENSOR_COMMON_CMD_DATA1 *data1)
{
    (void)pvObj;
    (void)cmd;
    (void)data0;
    (void)data1;
    int ret = -1;
    return ret;
}
/***********************sensor func related end**********************************/

/***********************adv api related start************************************/
static FH_UINT32 *GetMirrorFlipBayerFormat(void *pvObj)
{
    (void)pvObj;
    return 0;
}

static FH_UINT32 *GetUserSensorAwbGain(void *pvObj, FH_UINT32 idx)
{
    (void)pvObj;
    (void)idx;
    return 0;
}
static FH_UINT32 *GetSensorLtmCurve(void *pvObj, FH_UINT32 idx)
{
    (void)pvObj;
    (void)idx;
    return 0;
}
/***********************adv api related end*************************************/

/***********************not used for now start**********************************/
static int SetSensorAwbGain(void *pvObj, FH_UINT32 *awb_gain)
{
    (void)pvObj;
    (void)awb_gain;
    return 0;
}
static int GetSensorAwbGain(void *pvObj, FH_UINT32 *awb_gain)
{
    (void)pvObj;
    (void)awb_gain;
    return 0;
}
static int SetSensorIris(void *pvObj, FH_UINT32 iris)
{
    (void)pvObj;
    (void)iris;
    return 0;
}
/***********************not used for now end************************************/

#define SENSOR_SC4336P_MIPI "sc4336p_mipi"
#ifdef OS_RTT
struct isp_sensor_if sc4336p_mipi_sensor_if __attribute__((section(".sensor_drv"))) = {
#else
struct isp_sensor_if sc4336p_mipi_sensor_if = {
#endif
    .name = SENSOR_SC4336P_MIPI,
    .get_vi_attr = GetSensorViAttr,
    .set_flipmirror = SetSensorFlipMirror,
    .get_flipmirror = GetSensorFlipMirror,
    .set_iris = SetSensorIris,
    .init = Sensor_Init,
    .reset = SensorReset,
    .deinit = Sensor_DeInit,
    .set_sns_fmt = SetSensorFmt,
    .kick = Sensor_Kick,
    .set_sns_reg = SetSensorReg,
    .set_exposure_ratio = SetExposureRatio,
    .get_exposure_ratio = GetExposureRatio,
    .get_sensor_attribute = GetSensorAttribute,
    .set_lane_num_max = SetLaneNumMax,
    .get_sns_reg = GetSensorReg,
    .get_awb_gain = GetSensorAwbGain,
    .set_awb_gain = SetSensorAwbGain,
    .data = 0,
    .common_if = SensorCommonIf,
    .get_sns_ae_default = GetAEDefault,
    .get_sns_ae_info = GetAEInfo,
    .set_sns_intt = SetIntt,
    .calc_sns_valid_intt = CalcSnsValidIntt,
    .set_sns_gain = SetGain,
    .calc_sns_valid_gain = CalcSnsValidGain,
    .set_sns_frame_height = SetSnsFrameH,
    .get_sensor_mirror_flip_bayer_format = GetMirrorFlipBayerFormat,
    .get_user_awb_gain = GetUserSensorAwbGain,
    .get_user_ltm_curve = GetSensorLtmCurve,
    .is_sensor_connect = Sensor_Isconnect,
};

#ifdef OS_RTT
#else
struct isp_sensor_if *NAME(Sensor_Create)()
{
    struct isp_sensor_if *s_if;
    SNS_PARA_S *pstPara;

    s_if = (struct isp_sensor_if *)malloc(sizeof(struct isp_sensor_if));
    memcpy(s_if, &sc4336p_mipi_sensor_if, sizeof(struct isp_sensor_if));

    pstPara = (SNS_PARA_S *)malloc(sizeof(SNS_PARA_S));
    memcpy(&pstPara->stCtrlPara, &stSnsDefaultCtrlPara, sizeof(SNS_CTRL_PARA_S));
    memset(&pstPara->stCci, 0, sizeof(SensorDevice));

    s_if->para = pstPara;

    return s_if;
}

void NAME(Sensor_Destroy)(struct isp_sensor_if *s_if)
{
    free(s_if->para);
    s_if->para = 0;
    free(s_if);
    s_if = 0;
}
#endif

struct isp_sensor_if *(Sensor_Create_SC4336P)(void)
{
    SensorCreateInfo info = {
        .pstSnsIf = &sc4336p_mipi_sensor_if,
        .pstSnsCtrlPara = &stSnsDefaultCtrlPara,
        .pVer = SENSOR_GIT_VERSION,
    };
    return Sensor_Create_Common(&info);
}