#include "FHASD.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/prctl.h>
#include <unistd.h>
#define FRAME_SIZE_MAX (1280)
#define FRAME_SIZE_8K  (160)

typedef struct slist_head {
    struct slist_head *next;
    char *p_buf;
    int buf_used;
} SLIST_HEAD;

typedef struct {
    pthread_mutex_t lock; // 读写数据的锁
    int cry_result;       // 存储哭声检测结果
    int send_cnt;
    int det_cnt;
    int send_failed;
    int pthread_start;
    int pthread_running;
    SLIST_HEAD *p_list;
    void *st;
} TY_DEV_CRY_HANDLE_S;

TY_DEV_CRY_HANDLE_S g_cry_handle = {0};

TY_DEV_CRY_HANDLE_S *ty_dev_get_cry_handle(void)
{
    return &g_cry_handle;
}

// 申请节点
static SLIST_HEAD *__slist_node_init(char *p_data, int len)
{
    SLIST_HEAD *p_node = (SLIST_HEAD *)malloc(sizeof(SLIST_HEAD));
    if (p_node == NULL) {
        printf("malloc failed, line:%d\n", __LINE__);
        return NULL;
    }

    if (len < 0 || len > (FRAME_SIZE_MAX)) {
        free(p_node);
        printf("len is wrong, line:%d, len:%d\n", __LINE__, len);
        return NULL;
    }

    p_node->next = NULL;
    p_node->p_buf = (char *)malloc(len);
    if (p_node->p_buf == NULL) {
        free(p_node);
        printf("malloc failed, line:%d\n", __LINE__);
        return NULL;
    }

    memcpy(p_node->p_buf, p_data, len);
    p_node->buf_used = len;
    return p_node;
}

// 尾部添加
static SLIST_HEAD *__slist_add_tail(SLIST_HEAD *list, SLIST_HEAD *n)
{
    SLIST_HEAD *pos = NULL;
    SLIST_HEAD *last = list;
    if (list == NULL) {
        return n;
    }

    for (pos = list->next; pos; pos = pos->next) {
        last = pos;
    }
    last->next = n;
    return list;
}

// 删除头部节点
static SLIST_HEAD *__slist_del_head(SLIST_HEAD *list)
{
    SLIST_HEAD *pos = NULL;
    SLIST_HEAD *head = list;

    if (head == NULL) {
        return NULL;
    }
    pos = list->next;
    free(head->p_buf);
    free(head);
    return pos;
}

// 释放所有节点
static int __slist_all_free(SLIST_HEAD *list)
{
    while (list) {
        list = __slist_del_head(list);
    }
    return 0;
}

void *__thread_cry_detect(void *args)
{
    (void)args;
    SLIST_HEAD *p_node = NULL;
    TY_DEV_CRY_HANDLE_S *pHadl = ty_dev_get_cry_handle();
    prctl(PR_SET_NAME, "dev_cry_detect");
    pHadl->pthread_running = 1;
    int out = 0;
    int bcd_cnt = 0;

    while (pHadl->pthread_start) {
        pthread_mutex_lock(&pHadl->lock);
        p_node = pHadl->p_list;
        if (p_node == NULL) {
            pthread_mutex_unlock(&pHadl->lock);
            usleep(20 * 1000);
            continue;
        }
        pHadl->det_cnt++;
        // sambles：输入音频buffer长度，16K采样率一般为160，8K采样率一般为80,  ******* thread_cry_detect:Crying is
        // detected,len:320， 100fps
        pthread_mutex_unlock(&pHadl->lock);
        FHASD_Process(pHadl->st, (short *)p_node->p_buf, p_node->buf_used / 2);
        out = 0;
        FHASD_Get(pHadl->st, FHASD_GET_BABYCRY_DETECTOR_RESULT, &out);
        pthread_mutex_lock(&pHadl->lock);
        pHadl->p_list = __slist_del_head(pHadl->p_list);
        if (out == 1) {
            bcd_cnt++;
        } else {
            if (bcd_cnt > 0) {
                bcd_cnt--;
            }
        }
        if (bcd_cnt >= 400) { // 4s
            bcd_cnt = 0;
            pHadl->cry_result = 1;
            // ret 1,检测到哭声，0，未检测到
            printf("******** thread_cry_detect:Crying is detected,len:%d, out:%d\n", p_node->buf_used, out);
            usleep(100 * 1000);
        } else {
            pHadl->cry_result = 0;
        }

        pthread_mutex_unlock(&pHadl->lock);
        usleep(5 * 1000);
    }

    pHadl->pthread_running = 0;
    return NULL;
}

int ty_audio_cry_init(int sample, int level)
{
    (void)level;
    int ret = -1;
    TY_DEV_CRY_HANDLE_S *pHadl = ty_dev_get_cry_handle();
    printf("cry init sample:%d, ver:FHASD20231120\n", sample);

    if (sample != 16000 && sample != 8000) {
        printf("sample is %d, not supported\n", sample);
        return -1;
    }

    // 初始化
    ret = FHASD_Init(&pHadl->st, sample);
    if (ret != 0) {
        printf("FHCD_Init failed!: ret=%d.\n", ret);
        return -1;
    }
    /*
    调参:(一般使用FHASD_Init即可，初始化中有默认参数设置)
    实际使用时可以根据测试结果，
    调整检测门限，门限设置建议0.6到1，
    调整检测时长，检测时长建议哭声检测30，检测时长玻璃破碎20，值不得超过50
    调整检测间隔, 表示检测结果多久更新一次
    */
    float silence_time = 0;
    // 检测间隔，1表示检测结果每1s更新一次，更进一步的实际使用:因为婴儿啼哭往往持续几秒时间，可以统计若干秒内出现婴儿啼哭的次数降低误检
    // 0 返回实时结果
    int baby_hist_len = 30; // 检测时长,30表示计算过去2.4s内的检测概率均值
    float baby_thr = 0.85; // 检测门限，将检测概率均值和门限比较如果超过门限则认为是婴儿啼哭
    /* 设置婴儿啼哭检测门限和检测时长 */
    FHASD_Set(pHadl->st, FHASD_SET_BABYCRY_DETECTOR_THR, &baby_thr);
    FHASD_Set(pHadl->st, FHASD_SET_BABYCRY_DETECTOR_HIST_LEN, &baby_hist_len);
    /* 设置检测到一次婴儿啼哭后<silence_time>s内不再检测 */
    FHASD_Set(pHadl->st, FHASD_SET_BABYCRY_DETECTOR_SILENCE_TIME, &silence_time);

    ret = pthread_mutex_init(&pHadl->lock, NULL);
    if (ret != 0) {
        printf("pthread_mutex_init failed %d\n", ret);
        return -1;
    }

    pHadl->pthread_start = 1;

    pthread_t cry_thread;
    pthread_attr_t cry_attr;
    pthread_attr_init(&cry_attr);
    pthread_attr_setdetachstate(&cry_attr, PTHREAD_CREATE_DETACHED);
    pthread_attr_setstacksize(&cry_attr, 512 * 1024);
    if (pthread_create(&cry_thread, &cry_attr, __thread_cry_detect, NULL)) {
        printf("Error[cry]: __thread_cry_detect failed\n");
        pthread_attr_destroy(&cry_attr);
        return -1;
    }
    pthread_attr_destroy(&cry_attr);
    return 0;
}

// 给哭声检测线程送入音频数据
int ty_audio_cry_send_data(char *pdata, int len)
{
    TY_DEV_CRY_HANDLE_S *pHadl = ty_dev_get_cry_handle();
    SLIST_HEAD *p_node = NULL;
    // static int num = 0;
    // static int max_cnt = 0;

    pthread_mutex_lock(&pHadl->lock);
    if (pHadl->pthread_running == 1 && pdata != NULL) {
        // 入队
        p_node = __slist_node_init(pdata, len); // 8k len 为640
        if (p_node == NULL) {
            printf("__slist_node_init failed, line:%d\n", __LINE__);
            pthread_mutex_unlock(&pHadl->lock);
            return -1;
        }
        pHadl->p_list = __slist_add_tail(pHadl->p_list, p_node);
        pHadl->send_cnt++;
    }
    pthread_mutex_unlock(&pHadl->lock);
    // if (num % 50 == 0) {
    //     printf("cry_send_data send:%d, det:%d, max_cnt:%d\n", pHadl->send_cnt, pHadl->det_cnt, max_cnt);
    // }
    // num++;
    // if (pHadl->send_cnt - pHadl->det_cnt > max_cnt) {
    //     max_cnt = pHadl->send_cnt - pHadl->det_cnt;
    // }

    return 0;
}

int ty_audio_cry_get_result(int *val)
{
    TY_DEV_CRY_HANDLE_S *pHadl = ty_dev_get_cry_handle();
    if (pHadl->pthread_running == 0 || pHadl->pthread_start == 0) {
        return -1;
    }

    *val = pHadl->cry_result;
    return 0;
}

int ty_audio_cry_uninit(void)
{
    printf("ty_audio_cry_uninit\n");
    TY_DEV_CRY_HANDLE_S *pHadl = ty_dev_get_cry_handle();
    pHadl->pthread_start = 0;

    while (pHadl->pthread_running == 1) {
        usleep(10 * 1000);
    }

    // 哭声侦测内存释放
    int ret = FHASD_DeInit(pHadl->st);
    if (ret != 0) {
        printf("FHCD DeInit failed! ret=%d.\n", ret);
        return -1;
    }
    pHadl->st = NULL;
    __slist_all_free(pHadl->p_list);
    return 0;
}
