#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "tkl_detect_db.h"

/*
1分贝         刚能听到的声音
15分贝以下     感觉安静
30分贝        耳语的音量大小
40分贝        冰箱的嗡嗡声
60分贝        正常交谈的声音
70分贝        相当于走在闹市区
85分贝        汽车穿梭的马路上
95分贝        摩托车启动声音
100分贝       装修电钻的声音
110分贝       卡拉OK、大声播放MP3 的声音
120分贝       飞机起飞时的声音
150分贝       燃放烟花爆竹的声音
*/

/**
 * 获取所有振幅之平均值,计算db (振幅最大值 2^16-1 = 65535 最大值是 96.32db)
 * 16 bit == 2字节 == short int
 * 无符号16bit：96.32=20*lg(65535);
 */
static int ty_media_pcm_cal_decibel(char *pData, int len)
{
    double v = 0;
    short int value = 0;
    int ndb = 0, i = 0;

    for (i = 0; i < len; i += 2) {
        *(char *)&value = *(pData + i);
        *((char *)&value + 1) = *(pData + i + 1);
        v += abs(value);
    }

    v = v / (len / 2);
    if (v != 0) {
        ndb = (int)(20.0 * log10((double)v / 65535));
        ndb += 96;
    }

    //printf("db_value: %d\n", ndb);

    return ndb;
}

int ty_media_pcm_db_set_data(TY_MEDIA_PCM_HANDLE_T *phdl, char *pdata, int len)
{
    int db = -1;
    int copylen = 0;

    if (phdl->pcmLen < phdl->statisLen) {
        int dvalue = phdl->statisLen - phdl->pcmLen;
        copylen = (dvalue > len) ? len : dvalue;

        memcpy(phdl->pcmBuf + phdl->pcmLen, pdata, copylen);
        phdl->pcmLen += copylen;
    }

    if (phdl->pcmLen == phdl->statisLen) {
        db = ty_media_pcm_cal_decibel(phdl->pcmBuf, phdl->statisLen);
        if (copylen < len) {
            memcpy(phdl->pcmBuf, pdata + copylen, len - copylen);
            phdl->pcmLen = len - copylen;
        } else {
            phdl->pcmLen = 0;
        }
    }
    return db;
}

int ty_media_pcm_db_init(TY_MEDIA_PCM_HANDLE_T *phdl, TY_MEDIA_PCM_PARAM_T *pparam)
{
    printf("begin___\n");

    if (phdl == NULL || pparam == NULL) {
        return -1;
    }

    memset(phdl, 0, sizeof(TY_MEDIA_PCM_HANDLE_T));

    if (pparam->sampleCnt <= 0) {
        pparam->sampleCnt = pparam->sampleRate;
    }

    phdl->statisLen = pparam->sampleCnt * (pparam->bitRate / 8) * pparam->channel;
    phdl->pcmBuf = (char *)malloc(phdl->statisLen);
    if (phdl->pcmBuf == NULL) {
        printf("malloc memory fail\n");
        return -1;
    }

    printf("end___\n");

    return 0;
}

int ty_media_pcm_db_uninit(TY_MEDIA_PCM_HANDLE_T *phdl)
{
    printf("begin___\n");

    if (phdl == NULL) {
        return -1;
    }

    if (phdl->pcmBuf != NULL) {
        free(phdl->pcmBuf);
        phdl->pcmBuf = NULL;
    }

    memset(phdl, 0, sizeof(TY_MEDIA_PCM_HANDLE_T));

    printf("end___\n");

    return 0;
}
