#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "tkl_video_in.h"
#include "tkl_video_enc.h"
#include "tkl_fh8636_video.h"
#include "tkl_fh8636_osd.h"
#include "tkl_fh8636_sensor.h"
#include <sys/prctl.h>

// 原厂sdk头文件
#include "isp_api.h"
#include "FHAdv_Isp_mpi_v3.h"
#include "fh_system_mpi.h"
#include "fh_vpu_mpi.h"
#include "fh_venc_mpi.h"

OPERATE_RET tkl_vi_isp_set_antiflicker(TKL_VI_CHN_E chn, TKL_ISP_FLICKER_TYPE_E eAntiFlicker);
static OPERATE_RET tkl_vi_isp_set_day_figure_style(TKL_VI_CHN_E chn, TKL_ISP_FIG_STYLE_CONFIG_T *p_figure_style);
static OPERATE_RET tkl_vi_isp_set_night_figure_style(TKL_VI_CHN_E chn, TKL_ISP_FIG_STYLE_CONFIG_T *p_figure_style);
static TKL_ISP_DN_SWITCH_CONFIG_T FH8636_dn_config;
static TKL_ISP_DN_SWITCH_CALC_DATA_T FH8636_dn_calc_data;
static TKL_ISP_DN_RETURN_T dn_return;

OPERATE_RET tkl_vi_dnswitch_set_config(TKL_ISP_DN_SWITCH_CONFIG_T *pdnswitch_config);
OPERATE_RET tkl_vi_dnswitch_illumin_calculate(INT32_T *illumin_result);

//--------------------------------------------------------//
//-----------1.ISP基础逻辑,每个接口都必须实现------------//
//--------------------------------------------------------//

#define THRD_ISP_VIDEO "isp_video"
#define SENSOR_ID      0
//#define TY_ISP_DEBUG 1

// static void fh8636_vpu_reconfig(void)
// {
//     FH_VPU_SIZE vpu_size;
//     ISP_VI_ATTR_S isp_vi;

//     FH_VPSS_Reset();
//     API_ISP_Pause(u32IspDevId);
//     API_ISP_Resume();

//     FH_VPSS_GetViAttr(&vpu_size);
//     API_ISP_GetViAttr(&isp_vi);
//     if (vpu_size.vi_size.u32Width != isp_vi.u16PicWidth || vpu_size.vi_size.u32Height != isp_vi.u16PicHeight) {
//         vpu_size.vi_size.u32Width = isp_vi.u16PicWidth;
//         vpu_size.vi_size.u32Height = isp_vi.u16PicHeight;
//         FH_VPSS_SetViAttr(&vpu_size);
//     }

//     API_ISP_SensorKick();
//     printf("vpu reconfig!\n");

//     return;
// }

static void *fh8636_isp_proc(void *arg)
{
    (void)arg;
    FH_SINT32 ret;
    // FH_SINT32 err_cnt = 0;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    int channel = 0;

    prctl(PR_SET_NAME, THRD_ISP_VIDEO);
    while (!pHand->isp_thread_stop) {
        // ISP策略处理
        ret = API_ISP_Run(channel);
        if (ret) {
            // err_cnt++;
            // // 检测幅面信息是否符合预期
            // ret = API_ISP_DetectPicSize();
            // if (ret && err_cnt > 5) {
            //     fh8636_vpu_reconfig();
            //     err_cnt = 0;
            // }
            usleep(10000);
        }
    }

    pHand->isp_thread_running = 0;
    return NULL;
}

static FH_SINT32 fh8636_isp_start(void)
{
    FH_UINT32 ret = -1;
    pthread_t isp_thread = {0};
    pthread_attr_t attr = {0};
    struct sched_param param = {0};
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();

    if (pHand->isp_thread_running) {
        printf("isp has been running!\n");
        return 0;
    }

    pHand->isp_thread_running = 1;
    pHand->isp_thread_stop = 0;
    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
    pthread_attr_setstacksize(&attr, 10 * 1024);
    pthread_attr_setschedparam(&attr, &param);

    ret = pthread_create(&isp_thread, &attr, fh8636_isp_proc, NULL);
    if (FH_SUCCESS != ret) {
        printf("create ISP thread failed!\n");
        pHand->isp_thread_running = 0;
    }
    pthread_attr_destroy(&attr);

    return ret;
}

static FH_SINT32 fh8636_load_isp_param(FH_CHAR *file_path)
{
    FH_SINT32 ret;
    FILE *fp = NULL;
    FH_CHAR *buf = NULL;
    int u32IspDevId = 0;
    ISP_PARAM_CONFIG isp_param_config = {0};

    fp = fopen(file_path, "rb");
    if (fp == NULL) {
        printf("fopen %s error\n", file_path);
        return FH_FAILURE;
    }

    ret = API_ISP_GetBinAddr(u32IspDevId, &isp_param_config);
    if (ret != 0) {
        printf("API_ISP_GetBinAddr ERROR!!!ret(%d - %x)\n", ret, ret);
        return FH_FAILURE;
    }

    buf = (char *)malloc(isp_param_config.u32BinSize);
    if (buf == NULL) {
        printf("malloc error\n");
        return -1;
    }
    fread((void *)buf, 1, isp_param_config.u32BinSize, fp);
    fclose(fp);
    ret = API_ISP_LoadIspParam(u32IspDevId, buf);
    if (ret != 0) {
        printf("API_ISP_LoadIspParam ERROR!!!ret(%d - %x)\n", ret, ret);
        return FH_FAILURE;
    }

    free(buf);
    return FH_SUCCESS;
}

static struct isp_sensor_if* __sensor_create(void) {
    TY_FH8636_VI_HANDLE_T *pViHand = tkl_vi_get_handle();
    if (strcmp(pViHand->config.isp.name, "sc401ai") == 0 || 
        strcmp(pViHand->config.isp.name, "SC401AI") == 0){
        return Sensor_Create_SC401AI();
    } else if (strcmp(pViHand->config.isp.name, "imx415") == 0 || 
            strcmp(pViHand->config.isp.name, "IMX415") == 0){
        return Sensor_Create_IMX415();   
    } else if (strcmp(pViHand->config.isp.name, "ovos04c10") == 0 || 
            strcmp(pViHand->config.isp.name, "OVOS04C10") == 0){
        return Sensor_Create_OVOS04C10();
    } else if (strcmp(pViHand->config.isp.name, "sc4336") == 0 || 
            strcmp(pViHand->config.isp.name, "SC4336") == 0){
        return Sensor_Create_SC4336();
    } else if (strcmp(pViHand->config.isp.name, "sc4336p") == 0 || 
            strcmp(pViHand->config.isp.name, "SC4336P") == 0){
        return Sensor_Create_SC4336P();
    }else if (strcmp(pViHand->config.isp.name, "sc3336p") == 0 || 
            strcmp(pViHand->config.isp.name, "SC3336P") == 0){
        return Sensor_Create_SC3336P();
    } else {
        printf("not support sensor:%s\n", pViHand->config.isp.name);
    }    
    return NULL;
}

FH_SINT32 fh8636_isp_init(void)
{
    FH_SINT32 ret = -1;
    //FH_UINT32 param_addr, param_size;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    TY_FH8636_VI_HANDLE_T *pViHand = tkl_vi_get_handle();
    Sensor_Init_t initConf = {0};
    AE_DEFAULT_CFG ae_default_s;
    int u32IspDevId = 0;
    // ISP内部使用的memory分配与初始化
    ISP_MEM_INIT stMemInit = {0};
    stMemInit.stPicConf.u32Width = pHand->viParam.viSize.vi_size.u32Width;
    stMemInit.stPicConf.u32Height = pHand->viParam.viSize.vi_size.u32Height;
    stMemInit.enIspOutMode = ISP_OUT_TO_VPU;
    printf("fh8636_isp_init isp w h:%d %d\n",  stMemInit.stPicConf.u32Width,  stMemInit.stPicConf.u32Height);
    ret = API_ISP_MemInit(u32IspDevId, &stMemInit);
    TY_CHECK_RETURN(ret, ret);

    // start sensor
    pHand->viParam.pSensorinfo = __sensor_create();
    if (pHand->viParam.pSensorinfo) {
        ret = FHAdv_Isp_SensorInit(u32IspDevId, pHand->viParam.pSensorinfo);
        TY_CHECK_RETURN(ret, ret);
    } else {
        printf("sensor_create failed\n");
        return -1;
    }

    // 拷贝sensor的回调函数到目标地址
    ret = API_ISP_SensorRegCb(u32IspDevId, SENSOR_ID, pHand->viParam.pSensorinfo);
    TY_CHECK_RETURN(ret, ret);

    // sensor预初始化，并未配置sensor寄存器
    initConf.u8CsiDeviceId = u32IspDevId;
    initConf.u8CciDeviceId = 0;
    initConf.bGrpSync = FH_FALSE; // 不需要拼接

    ret = API_ISP_SensorInit(u32IspDevId, &initConf);
    TY_CHECK_RETURN(ret, ret);

    // 初始化sensor
    ret = API_ISP_SetSensorFmt(u32IspDevId, pHand->viParam.ispFormat);
    TY_CHECK_RETURN(ret, ret);

    // 启动sensor输出，有的sensor需要用到
    ret = API_ISP_SensorKick(u32IspDevId);
    if (FH_FAILURE == ret) {
        printf("API_ISP_SensorKick's return is not necessary!\n");
    }

    // 初始化ISP硬件寄存器，并启动ISP
    ret = API_ISP_Init(u32IspDevId);
    TY_CHECK_RETURN(ret, ret);

    // 加载指定参数到DRV寄存器
    printf("ISP_PARAM_HEX:%s %s\n", pViHand->config.isp.name, pViHand->config.isp.conf);
    ret = fh8636_load_isp_param(pViHand->config.isp.conf);
    TY_CHECK_RETURN(ret, ret);

    ret = API_ISP_GetAeDefaultCfg(u32IspDevId, &ae_default_s);
    if (ret < 0) {
        printf("API_ISP_GetAeDefaultCfg failed\n");
    }
    pViHand->ref_luma[0] = ae_default_s.stAeLumaCfg.u08LumaRef;
    ret = FHAdv_Isp_Init(u32IspDevId);
    if (FH_FAILURE == ret) {
        printf("FHAdv_Isp_Init's return is not necessary!\n");
    }

    // start isp线程
    ret = fh8636_isp_start();
    TY_CHECK_RETURN(ret, ret);

    return FH_SUCCESS;
}

void fh8636_isp_deinit(void)
{
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    int u32IspDevId = 0;
    if (pHand->isp_thread_running) {
        pHand->isp_thread_stop = 1;
        while (pHand->isp_thread_running) {
            usleep(20 * 1000);
        }

        // ISP线程退出，清理状态
        API_ISP_Exit(u32IspDevId);
    }

    return;
}

/***********************************************************
*  函数: tkl_vi_isp_init
*  功能：isp初始化函数
*  参数: IN TKL_VI_CHN_E chn 设备号
         IN TKL_ISP_PARAM_INFO_T *pisp_info ISP可配置参数结构体,包括ADN/WDR/AWB/denoise等模块
*
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_init(TKL_VI_CHN_E chn, TKL_ISP_PARAM_INFO_T *pisp_info)
{
    (void)chn;
    INT32_T ret = 0;

    ret = tkl_vi_dnswitch_set_config(&pisp_info->isp_dn_switch_config);
    if (ret < 0) {
        printf("tkl_vi_isp_load_user_isp_param failed\n");
    }

    printf("********tkl_vi_isp_init********\n");
    return 0;
}

/***********************************************************
 *  函数: tkl_vi_isp_load_isp_file
 *  功能：载入各个平台的图像参数,通常为bin或者ini文件
 *  参数: IN TKL_ISP_DN_MODE_E irmode,当前日夜模式,0-AUTO模式,1-白天,2-夜晚。入参auto会强制转白天,不支持auto
 *
 *  返回值: <0 失败,=0 成功
 ***********************************************************/
OPERATE_RET tkl_vi_isp_load_isp_file(TKL_ISP_DN_MODE_E irmode)
{
    TY_FH8636_VI_HANDLE_T *pViHand = tkl_vi_get_handle();
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    int u32IspDevId = 0;

    /*校验irmode合理性 */
    if ((irmode < 0) || (irmode > 2)) {
        printf("ty_isp_load_isp_file error unsupported irmode =%d,expect [0,2]!\n", irmode);
        return -1;
    }
    if (irmode == AUTO_MODE) {
        irmode = DAY_MODE;
    }

    //白天强抗闪，夜晚弱抗闪，产品新需求20230620
    FH8636_dn_config.cur_irmode = irmode;

    FH_SINT32 ret = 0;
    AE_DEFAULT_CFG ae_default_s;

    ret = API_ISP_Pause(u32IspDevId);
    if (ret != 0) {
        printf("API_ISP_Pause ERROR!!!ret = 0x%x\n", ret);
        return FH_FAILURE;
    }

    if (irmode == 1) {
        // 加载指定参数到DRV寄存器
        if(pHand->isp_wdr_mode == 1){
            printf("ISP_PARAM_HEX: %s\n", pViHand->config.isp.conf2);
            ret = fh8636_load_isp_param(pViHand->config.isp.conf2);
            ret = API_ISP_GetAeDefaultCfg(u32IspDevId, &ae_default_s);
            if (ret < 0) {
                printf("API_ISP_GetAeDefaultCfg failed\n");
            }
            pViHand->ref_luma[2] = ae_default_s.stAeLumaCfg.u08LumaRef;
        }else{
            printf("ISP_PARAM_HEX: %s\n", pViHand->config.isp.conf);
            ret = fh8636_load_isp_param(pViHand->config.isp.conf);
        }
        TY_CHECK_RETURN(ret, -1);
        ret = FHAdv_Isp_Init(u32IspDevId);
        TY_CHECK_RETURN(ret, -1);
        ret = tkl_vi_isp_set_day_figure_style(pViHand->config.chn, &pViHand->config.isp.isp_fig_style_day);
        TY_CHECK_RETURN(ret, -1);
        // 加载用户设置的抗闪参数
        ret = tkl_vi_isp_set_antiflicker(0, pHand->isp_anti_flicker);
        if (ret != 0) {
            printf("tkl_vi_isp_set_day_antiflicker ERROR!!!ret = 0x%x\n", ret);
            return FH_FAILURE;
        }
    } else if (irmode == 2) {
        printf("ISP_PARAM_HEX 1: %s\n", pViHand->config.isp.conf1);
        ret = fh8636_load_isp_param(pViHand->config.isp.conf1);
        TY_CHECK_RETURN(ret, -1);

        // 记录下夜间参考亮度
        ret = API_ISP_GetAeDefaultCfg(u32IspDevId, &ae_default_s);
        if (ret < 0) {
            printf("API_ISP_GetAeDefaultCfg failed\n");
        }
        pViHand->ref_luma[1] = ae_default_s.stAeLumaCfg.u08LumaRef;
        ret = FHAdv_Isp_Init(u32IspDevId);
        TY_CHECK_RETURN(ret, -1);
        ret = tkl_vi_isp_set_night_figure_style(pViHand->config.chn, &pViHand->config.isp.isp_fig_style_night);
        TY_CHECK_RETURN(ret, -1);
        // 加载用户设置的抗闪参数
        ret = tkl_vi_isp_set_antiflicker(0, pHand->isp_anti_flicker);
        if (ret != 0) {
            printf("tkl_vi_isp_set_night_antiflicker ERROR!!!ret = 0x%x\n", ret);
            return FH_FAILURE;
        }
    }
    if (ret != 0) {
        printf("fh8636_load_isp_param!!!ret = 0x%x\n", ret);
        return FH_FAILURE;
    }

    ret = API_ISP_Resume(u32IspDevId);
    if (ret != 0) {
        printf("API_ISP_Resume ERROR!!!ret = 0x%x\n", ret);
        return FH_FAILURE;
    }

#ifdef FH_LSC_CONFIG
    ret = ty_isp_set_day_shading(irmode);
    if (ret != 0) {
        printf("ty_isp_set_day_shading failed!\n");
    }
#endif
    return ret;
}

/***********************************************************
*  函数: tkl_vi_isp_load_user_isp_param
*
功能：载入用户自定义的图像配置,比如画面风格、白平衡/WDR/去噪参数。此函数配置的图像配置都是静态配置,不需要在IPC运行过程中实时计算。
*  参数: IN TKL_VI_CHN_E chn 设备号
         IN TKL_ISP_PARAM_INFO_T *pisp_info,init过程需要使用的isp所有可配置参数的索引结构。
*
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_load_user_isp_param(TKL_VI_CHN_E chn, TKL_ISP_PARAM_INFO_T *pisp_info)
{
    (void)chn;
    //  INT32_T ret = 0;
    INT32_T irmode = pisp_info->irmode;
    /*校验irmode合理性 */
    if ((irmode < 0) || (irmode > 2)) {
        printf("tkl_vi_isp_load_user_isp_param error unsupported irmode =%d,expect [0,2]!\n", irmode);
        return -1;
    }
    if (irmode == AUTO_MODE)
        irmode = DAY_MODE;
    //  if( irmode == 1)
    //      ret = tkl_vi_isp_set_day_figure_style(&pisp_info->isp_fig_style_config);
    //  else
    //      ret = tkl_vi_isp_set_night_figure_style(&pisp_info->isp_fig_style_config);
    //  if(ret < 0)
    //      printf("tkl_vi_isp_set_day_night_figure_style failed\n");
    //
    //  ret = tkl_vi_isp_set_awb_config(&pisp_info->isp_awb_config);
    //  if(ret < 0)
    //      printf("tkl_vi_isp_set_awb_config failed\n");
    //
    //  ret = tkl_vi_isp_set_wdr_config(&pisp_info->isp_wdr_config);
    //  if(ret < 0)
    //      printf("tkl_vi_isp_set_wdr_config failed\n");
    //
    //  ret = tkl_vi_isp_set_denoise_config(&pisp_info->isp_denoise_config);
    //  if(ret < 0)
    //      printf("tkl_vi_isp_set_denoise_config failed\n");

    return 0;
}

/***********************************************************
*  函数: tkl_vi_isp_load_dynamic_isp_param
*
功能：载入动态的图像配置,比如动态shading、自研宽动态算法等。此函数配置的图像配置都是动态配置,需要在IPC运行过程中实时计算。
*  参数: IN TKL_VI_CHN_E chn 设备号
         IN TKL_ISP_PARAM_INFO_T *pisp_info,init过程需要使用的isp所有可配置参数的索引结构。
*
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_load_dynamic_isp_param(TKL_VI_CHN_E chn, TKL_ISP_PARAM_INFO_T *pisp_info)
{
    (void)chn;
    // INT32_T ret = 0;
    INT32_T irmode = pisp_info->irmode;
    /*校验irmode合理性 */
    if ((irmode < 0) || (irmode > 2)) {
        printf("tkl_vi_isp_load_dynamic_isp_param error unsupported irmode =%d,expect [0,2]!\n", irmode);
        return -1;
    }
    if (irmode == AUTO_MODE)
        irmode = DAY_MODE;

    //    if( irmode == 1)
    //        ret = tkl_vi_isp_set_day_shading(chn, &(pisp_info->isp_shading_config));
    //    else
    //        ret = tkl_vi_isp_set_night_shading(chn, &(pisp_info->isp_shading_config));
    //    if(ret < 0)
    //        printf("tkl_vi_isp_set_shading failed\n");

    return 0;
}

/***********************************************************
 *  函数: tkl_vi_isp_get_mirror_flip
 *  功能：获取当前画面翻转状态
 *  参数: IN TKL_VI_CHN_E chn 设备通道号
 *        [out] pmirror
 *        [out] pflip
 *  返回值: <0 失败,=0 成功
 ***********************************************************/
OPERATE_RET tkl_vi_isp_get_mirror_flip(TKL_VI_CHN_E chn, int *pmirror, int *pflip)
{
    (void)chn;
    int32_t ret = -1;
    int u32IspDevId = 0;
    ret = FHAdv_Isp_GetMirrorAndflip(u32IspDevId,(uint32_t *)pmirror, (uint32_t *)pflip);
    if (ret != 0) {
        printf("FHAdv_Isp_GetMirrorAndflip mirror flip: %d %d failed\n", *pmirror, *pflip);
        return -1;
    } else {
        printf("FHAdv_Isp_GetMirrorAndflip mirror flip: %d %d suc\n", *pmirror, *pflip);
    }
    return 0;
}

/***********************************************************
*  函数: tkl_vi_isp_set_mirror_flip
*  功能：设置画面翻转
*  参数: IN TKL_VI_CHN_E chn 设备号
         IN mirror和flip
*
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_set_mirror_flip(TKL_VI_CHN_E chn, int mirror, int flip)
{
    (void)chn;
    int32_t ret = -1;
    int u32IspDevId = 0;
    ret = FHAdv_Isp_SetMirrorAndflip(u32IspDevId, FH_GRP_ID, (uint32_t)mirror, (uint32_t)flip);
    if (ret != 0) {
        printf("FHAdv_Isp_SetMirrorAndflip mirror flip: %d %d failed\n", mirror, flip);
        return -1;
    } else {
        printf("FHAdv_Isp_SetMirrorAndflip mirror flip: %d %d suc\n", mirror, flip);
    }
    return 0;
}

/***********************************************************
*  函数: tkl_vi_isp_get_antiflicker
*  功能：获取当前抗频闪状态
*  参数: IN TKL_VI_CHN_E chn 设备号
         OUT TKL_ISP_FLICKER_TYPE_E *eAntiFlicker, 0-关闭,1-50hz,2-60hz,3-自动抗闪模式
*
*  返回值: <0 失败,=0/1/2 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_get_antiflicker(TKL_VI_CHN_E chn, TKL_ISP_FLICKER_TYPE_E *eAntiFlicker)
{
    (void)chn;
    int ret = -1;
    AE_DEFAULT_CFG ae_default_s;
    int u32IspDevId = 0;

    ret = API_ISP_GetAeDefaultCfg(u32IspDevId, &ae_default_s);
    if (ret < 0) {
        printf("API_ISP_GetAeDefaultCfg failed\n");
    }

    if (!ae_default_s.stAeEnableCfg.bAntiFlickerEn) {
        return 0;
    }
    //富瀚抗闪模式分为强抗闪和亚抗闪，且可以适配不同的抗闪频率
    //由于tuya app抗闪配置未区分强抗闪和亚抗闪，这里只上报2种状态，不再上报4种。
    if (!ae_default_s.stAeAntiflickerCfg.bAntiFlickerMode) {
        if (ae_default_s.stAeAntiflickerCfg.u16Frequency == 50) {
            *eAntiFlicker = FLICKER_TYPE_50HZ;
        } else {
            *eAntiFlicker = FLICKER_TYPE_60HZ;
        }
    } else {
        if (ae_default_s.stAeAntiflickerCfg.u16Frequency == 60) {
            *eAntiFlicker = FLICKER_TYPE_50HZ;
        } else {
            *eAntiFlicker = FLICKER_TYPE_60HZ;
        }
    }
    return 0;
}

/***********************************************************
*  函数: tkl_vi_isp_set_antiflicker
*  功能：设置当前抗频闪配置
*  参数: IN TKL_VI_CHN_E chn 设备号
         IN TKL_ISP_FLICKER_TYPE_E eAntiFlicker,0-关闭,1-50hz,2-60hz,3-自动抗闪模式
*
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_set_antiflicker(TKL_VI_CHN_E chn, TKL_ISP_FLICKER_TYPE_E eAntiFlicker)
{
    (void)chn;
    int ret = -1;
    AE_DEFAULT_CFG ae_default_s;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    int u32IspDevId = 0;

    ret = API_ISP_GetAeDefaultCfg(u32IspDevId, &ae_default_s);
    if (ret < 0) {
        printf("API_ISP_GetAeDefaultCfg failed\n");
    }

    switch (eAntiFlicker) {
    case FLICKER_TYPE_DISABLE:
        ae_default_s.stAeEnableCfg.bAntiFlickerEn = 0;
        printf("close antiflicker func\n");
        break;
    case FLICKER_TYPE_50HZ:
        ae_default_s.stAeEnableCfg.bAntiFlickerEn = 1;
        ae_default_s.stAeAntiflickerCfg.bAntiFlickerMode = 0;
        ae_default_s.stAeAntiflickerCfg.u16Frequency = 50;
        break;
    case FLICKER_TYPE_60HZ:
        ae_default_s.stAeEnableCfg.bAntiFlickerEn = 1;
        ae_default_s.stAeAntiflickerCfg.bAntiFlickerMode = 0;
        ae_default_s.stAeAntiflickerCfg.u16Frequency = 60;
        break;
        //由于tuya app抗闪配置未区分强抗闪和亚抗闪，这里只配置为亚抗闪模式防止过曝
        //      case 3:
        //          ae_default_s.stAeEnableCfg.bAntiFlickerEn = 1;
        //          ae_default_s.stAeAntiflickerCfg.bAntiFlickerMode = 1;
        //          ae_default_s.stAeAntiflickerCfg.u16Frequency = 50;
        //          break;
        //      case 4:
        //          ae_default_s.stAeEnableCfg.bAntiFlickerEn = 1;
        //          ae_default_s.stAeAntiflickerCfg.bAntiFlickerMode = 1;
        //          ae_default_s.stAeAntiflickerCfg.u16Frequency = 60;
        //          break;
    default:
        printf("ty_isp_set_antiflicker unsupport value = %d error,expect [0,4]!\n", eAntiFlicker);
        return -1;
    }

    //白天强抗闪，夜晚弱抗闪，产品新需求20230620
    if(FH8636_dn_config.cur_irmode == DAY_MODE)
        ae_default_s.stAeAntiflickerCfg.bAntiFlickerMode = 1;

    printf("tkl_vi_isp_set_antiflicker en:%d, mode:%d, freq:%d\n", ae_default_s.stAeEnableCfg.bAntiFlickerEn, ae_default_s.stAeAntiflickerCfg.bAntiFlickerMode, ae_default_s.stAeAntiflickerCfg.u16Frequency);
    ret = API_ISP_SetAeDefaultCfg(u32IspDevId, &ae_default_s);
    if (ret < 0) {
        printf("API_ISP_SetAeDefaultCfg failed\n");
    }

    pHand->isp_anti_flicker = eAntiFlicker;

    return 0;
}

/***********************************************************
*  函数: tkl_vi_isp_get_gamma
*  功能：获取当前gamma
*  参数: IN TKL_VI_CHN_E chn 设备号
         OUT unsigned short *gamma,获取到的当前设备gamma曲线数组
         OUT INT32_T *gamma_length,gamma数组的长度,君正ssc333是129
*
*  返回值: <0 失败,>=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_get_gamma(TKL_VI_CHN_E chn, PUINT16_T gamma, PINT32_T gamma_length)
{
    (void)chn;
    (void)gamma;
    (void)gamma_length;
    // INT32_T ret = -1;
    //*gamma_length = 256;
    //暂时不需要读写gamma,接口留空
    return -1;
}

/***********************************************************
*  函数: tkl_vi_isp_set_gamma
*  功能：设置当前gamma
*  参数: IN TKL_VI_CHN_E chn 设备号
         IN unsigned short *gamma,需要设置的gamma曲线数组
         IN INT32_T *gamma_length,gamma数组的长度
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_set_gamma(TKL_VI_CHN_E chn, PUINT16_T gamma, PINT32_T gamma_length)
{
    (void)chn;
    (void)gamma;
    (void)gamma_length;
    //暂时不需要读写gamma,接口留空
    return -1;
}

/***********************************************************
*  函数: tkl_vi_isp_set_color_to_gray
*  功能：设置画面彩转灰、灰转彩
*  参数: IN TKL_VI_CHN_E chn 设备号
         IN int gray_flag,当前画面的黑白模式,0-画面是彩色模式,1-画面是黑白模式,不支持其他值
*
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_set_color_to_gray(TKL_VI_CHN_E chn, int gray_flag)
{
    (void)chn;
    int ret = -1;
    int u32IspDevId = 0;
    ISP_SAT_CFG pstSatCfg = {0};
    ISP_AE_INFO pstAeInfo = {0};
    /*校验irmode合理性 */
    if ((gray_flag < 0) || (gray_flag > 1)) {
        printf("ty_isp_load_user_isp_param error unsupported gray_flag =%d,expect [0,1]!\n", gray_flag);
        return -1;
    }

    ret = API_ISP_GetSaturation(u32IspDevId, &pstSatCfg);
    ret |= API_ISP_GetAeInfo(u32IspDevId, &pstAeInfo);
    if (ret < 0) {
        printf("API_ISP_GetSaturation ERROR!!!ret = 0x%x\n", ret);
        return -1;
    }

    if (gray_flag == 0) {
        pstSatCfg.bGainMappingEn = 1;
        pstSatCfg.u08Sat = 64;
    } else if (gray_flag == 1) {
        pstAeInfo.u32SensorGain = 0x40;
        pstSatCfg.bGainMappingEn = 0;
        pstSatCfg.u08Sat = 0;
    }

    ret = API_ISP_SetAeInfo(u32IspDevId, &pstAeInfo);
    ret |= API_ISP_SetSaturation(u32IspDevId, &pstSatCfg);
    if (ret < 0) {
        printf("API_ISP_SetSaturation ERROR!!!ret = 0x%x\n", ret);
        return -1;
    }

    return 0;
}

/***********************************************************
 *  函数: tkl_vi_dnswitch_set_config
 *  功能：设置日夜切换的配置参数。
 *  参数: IN TKL_ISP_DN_SWITCH_CONFIG_T *pdnswitch_config 日夜切换的配置参数
 *
 *  返回值: <0 失败,=0 成功
 ***********************************************************/
OPERATE_RET tkl_vi_dnswitch_set_config(TKL_ISP_DN_SWITCH_CONFIG_T *pdnswitch_config)
{
    INT32_T i = 0;
    INT32_T irmode = 0;
    /*校验pdnswitch_config合理性,校验cur_irmode合理性 */
    if (NULL == pdnswitch_config) {
        printf("pdnswitch_config is NULL!\n");
        return -1;
    }

    /*校验irmode合理性 */
    irmode = pdnswitch_config->cur_irmode;
    if ((irmode < 0) || (irmode > 2)) {
        printf("tkl_vi_dnswitch_set_config error unsupported irmode =%d,expect [0,2]!\n", irmode);
        return -1;
    }
    if (irmode == AUTO_MODE)
        irmode = DAY_MODE;

    FH8636_dn_config = *pdnswitch_config;
    for (i = 0; i < 6; i++) {
        FH8636_dn_calc_data.coffR[i] = FH8636_dn_config.coffR[i];
        FH8636_dn_calc_data.coffG[i] = FH8636_dn_config.coffG[i];
        FH8636_dn_calc_data.coffB[i] = FH8636_dn_config.coffB[i];
    }

    return 0;
}

/***********************************************************
*  函数: tkl_vi_dnswitch_get_illumin
*  功能：获取日夜切换算法的照度值计算结果,给日夜切换组件使用。
*  参数: IN TKL_VI_CHN_E chn 设备号
         IN TKL_ISP_DN_MODE_E irmode,当前日夜模式,0-AUTO模式,1-白天,2-夜晚。入参auto会强制转白天,不支持auto
         OUT INT32_T *illumin_result,日夜切换算法的计算结果, 默认是整数,夜视强反光场景会返回result=1并伴有串口提示。
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_dnswitch_get_illumin(TKL_VI_CHN_E chn, TKL_ISP_DN_MODE_E irmode, INT32_T *illumin_result)
{
    (void)chn;
    int ret = -1;
    ISP_AE_STATUS FH8636_ae_params;
    ISP_AWB_STATUS FH8636_awb_params;
	GLOBE_STAT FH8636_global_stat[256];
    int u32IspDevId = 0;
	int Rtable_sum = 0;
    int GRtable_sum = 0;
    int GBtable_sum = 0;
    int Btable_sum = 0;

    ret = API_ISP_GetAeStatus(u32IspDevId, &FH8636_ae_params);
    if (ret < 0) {
        printf("API_ISP_GetAeStatus failed\n");
        return -1;
    }

    ret = API_ISP_GetAwbStatus(u32IspDevId, &FH8636_awb_params);
    if (ret < 0) {
        printf("API_ISP_GetAwbStatus failed\n");
        return -1;
   }

	ret = API_ISP_GetGlobeStat(u32IspDevId, FH8636_global_stat);
    if (ret < 0) {
        printf("API_ISP_GetGlobeStat failed\n");
        return -1;
    }

    /*校验cur_irmode合理性 */
    if ((irmode < 0) || (irmode > 5)) {
        printf("ty_dnswitch_get_illumin error unsupported irmode =%d,expect [0,5]!\n", irmode);
        return -1;
    }
    if (irmode == AUTO_MODE)
        FH8636_dn_config.cur_irmode = DAY_MODE;
    else
        FH8636_dn_config.cur_irmode = irmode;

    /*获取AE曝光信息*/

    FH8636_dn_calc_data.ExpTime = FH8636_ae_params.u32CurrIntt; //当前曝光时间，单位固定us
    FH8636_dn_calc_data.AGain = (float)FH8636_ae_params.u16CurrAgain;
    FH8636_dn_calc_data.ISPDGain = ((float)FH8636_ae_params.u16CurrDgain);
    FH8636_dn_calc_data.ISO = (FH8636_dn_calc_data.AGain) * (FH8636_dn_calc_data.ISPDGain);
    FH8636_dn_calc_data.Exposure = ((float)FH8636_dn_calc_data.ExpTime) * ((float)FH8636_dn_calc_data.AGain) *
                                   ((float)FH8636_dn_calc_data.ISPDGain) /
                                   (1000 * 64 * 64); // IspDgain = ISPgain*Dgain, 这里要除以1000*64*64
    /*获取AWB曝光信息*/
    for(int i=0;i<256;i++){
        Rtable_sum  += FH8636_global_stat[i].r.sum;
        GRtable_sum += FH8636_global_stat[i].gr.sum;
        GBtable_sum += FH8636_global_stat[i].gb.sum;
        Btable_sum  += FH8636_global_stat[i].b.sum;
    }
    FH8636_dn_calc_data.GlobalR = (unsigned short)(Rtable_sum/24576);
    FH8636_dn_calc_data.GlobalG = (unsigned short)((GRtable_sum+GBtable_sum)/49152);
    FH8636_dn_calc_data.GlobalB = (unsigned short)(Btable_sum/24576);
 //   FH8636_dn_calc_data.GlobalR = (unsigned short)(FH8636_awb_params.u16RAvg) * 50;
 //   FH8636_dn_calc_data.GlobalG = (unsigned short)(FH8636_awb_params.u16GAvg) * 50;
 //   FH8636_dn_calc_data.GlobalB = (unsigned short)(FH8636_awb_params.u16BAvg) * 50;

    /*根据当前日夜模式,计算照度值(*illumin_result) */

    if (FH8636_dn_config.cur_irmode == DAY_MODE || FH8636_dn_config.cur_irmode == COLOR_DAY_MODE ||
        FH8636_dn_config.cur_irmode == COLOR_NIGHT_MODE) {
        FH8636_dn_calc_data.RGBComponent =
            (0.3 * FH8636_dn_calc_data.GlobalR + 0.6 * FH8636_dn_calc_data.GlobalG + 0.1 * FH8636_dn_calc_data.GlobalB);
        FH8636_dn_calc_data.Illumination = FH8636_dn_calc_data.RGBComponent * 10 /
                                           (FH8636_dn_calc_data.Exposure == 0 ? 1 : FH8636_dn_calc_data.Exposure);
        *illumin_result = (int)FH8636_dn_calc_data.Illumination;
    }

    else if (FH8636_dn_config.cur_irmode == NIGHT_MODE) {
        ret = tkl_vi_dnswitch_illumin_calculate(illumin_result);
        if (ret < 0) {
            printf("tkl_vi_dnswitch_illumin_calculate failed\n");
        }
    } else {
        printf("ty_dnswitch_get_illumin unsupported irmode =%d\n", FH8636_dn_config.cur_irmode);
        return -1;
    }

#ifdef TY_ISP_DEBUG
	printf("***********************************************************\n");
	printf("***********************新的日夜切换接口测试****************\n");
	printf("当前日夜模式 = %d\n", FH8636_dn_config.cur_irmode);
	printf("FH8636_global_stat.r.sum = %d\n", FH8636_global_stat[0].r.sum);
	printf("FH8636_global_stat.r.cnt = %d\n", FH8636_global_stat[0].r.cnt);
	printf("FH8636_global_stat.r.max = %d\n", FH8636_global_stat[0].r.max);
	printf("FH8636_global_stat.r.min = %d\n", FH8636_global_stat[0].r.min);
	printf("FH8636_global_stat.gr.sum = %d\n", FH8636_global_stat[0].gr.sum);
	printf("FH8636_global_stat.gr.cnt = %d\n", FH8636_global_stat[0].gr.cnt);
	printf("FH8636_global_stat.gr.max = %d\n", FH8636_global_stat[0].gr.max);
	printf("FH8636_global_stat.gr.min = %d\n", FH8636_global_stat[0].gr.min);
	printf("FH8636_global_stat.gb.sum = %d\n", FH8636_global_stat[0].gb.sum);
	printf("FH8636_global_stat.gb.cnt = %d\n", FH8636_global_stat[0].gb.cnt);
	printf("FH8636_global_stat.gb.max = %d\n", FH8636_global_stat[0].gb.max);
	printf("FH8636_global_stat.gb.min = %d\n", FH8636_global_stat[0].gb.min);
	printf("FH8636_global_stat.b.sum = %d\n", FH8636_global_stat[0].b.sum);
	printf("FH8636_global_stat.b.cnt = %d\n", FH8636_global_stat[0].b.cnt);
	printf("FH8636_global_stat.b.max = %d\n", FH8636_global_stat[0].b.max);
	printf("FH8636_global_stat.b.min = %d\n", FH8636_global_stat[0].b.min); 


    if (FH8636_dn_config.cur_irmode == DAY_MODE) //白天
    {
        printf("***********************************************************\n");
        printf("***********************************************************\n");
        printf("当前日夜模式 = %d\n", FH8636_dn_config.cur_irmode);
        printf("FH8636_dn_calc_data.ExpTime = %d\n", FH8636_dn_calc_data.ExpTime);
        printf("FH8636_dn_calc_data.AGain = %d\n", FH8636_dn_calc_data.AGain);
        printf("FH8636_dn_calc_data.ISPDGain = %d\n", FH8636_dn_calc_data.ISPDGain);
        printf("FH8636_dn_calc_data.ISO = %d\n", FH8636_dn_calc_data.ISO);
        printf("FH8636_dn_calc_data.Exposure = %16.2f\n", FH8636_dn_calc_data.Exposure);
        printf("FH8636_dn_calc_data.RGBComponent =  %16.2f\n", FH8636_dn_calc_data.RGBComponent);
        printf("FH8636_dn_calc_data.GlobalR = %d\n", FH8636_dn_calc_data.GlobalR);
        printf("FH8636_dn_calc_data.GlobalG = %d\n", FH8636_dn_calc_data.GlobalG);
        printf("FH8636_dn_calc_data.GlobalB = %d\n", FH8636_dn_calc_data.GlobalB);
        printf("__ty_dnswitch_get_illumin  ComponentR= %16.2f\n",
               0.3 * (FH8636_dn_calc_data.GlobalR / FH8636_dn_calc_data.Exposure));
        printf("__ty_dnswitch_get_illumin  ComponentG= %16.2f\n",
               0.6 * (FH8636_dn_calc_data.GlobalG / FH8636_dn_calc_data.Exposure));
        printf("__ty_dnswitch_get_illumin  ComponentB= %16.2f\n",
               0.1 * (FH8636_dn_calc_data.GlobalB / FH8636_dn_calc_data.Exposure));
        printf("当前照度值ILLUMIN = %d\n", *illumin_result);
    } else if (FH8636_dn_config.cur_irmode == NIGHT_MODE) //夜晚
    {
        printf("***********************************************************\n");
        printf("***********************************************************\n");
        printf("当前日夜模式 = %d\n", FH8636_dn_config.cur_irmode);
        printf("FH8636_dn_calc_data.ExpTime = %d\n", FH8636_dn_calc_data.ExpTime);
        printf("FH8636_dn_calc_data.AGain = %d\n", FH8636_dn_calc_data.AGain);
        printf("FH8636_dn_calc_data.ISPDGain = %d\n", FH8636_dn_calc_data.ISPDGain);
        printf("FH8636_dn_calc_data.ISO = %d\n", FH8636_dn_calc_data.ISO);
        printf("FH8636_dn_calc_data.Exposure =  %16.2f\n", FH8636_dn_calc_data.Exposure);
        printf("FH8636_dn_calc_data.RGBComponent =  %16.2f\n", FH8636_dn_calc_data.RGBComponent);
        printf("FH8636_dn_calc_data.GlobalR = %d\n", FH8636_dn_calc_data.GlobalR);
        printf("FH8636_dn_calc_data.GlobalG = %d\n", FH8636_dn_calc_data.GlobalG);
        printf("FH8636_dn_calc_data.GlobalB = %d\n", FH8636_dn_calc_data.GlobalB);
        printf("__ty_dnswitch_get_illumin  coffR= %16.2f\n", dn_return.R_Return);
        printf("__ty_dnswitch_get_illumin  coffG= %16.2f\n", dn_return.G_Return);
        printf("__ty_dnswitch_get_illumin  coffB= %16.2f\n", dn_return.B_Return);
        printf("__ty_dnswitch_get_illumin  ComponentR= %16.2f\n",
               0.3 * (FH8636_dn_calc_data.GlobalR * dn_return.R_Return / FH8636_dn_calc_data.Exposure));
        printf("__ty_dnswitch_get_illumin  ComponentG= %16.2f\n",
               0.6 * (FH8636_dn_calc_data.GlobalG * dn_return.G_Return / FH8636_dn_calc_data.Exposure));
        printf("__ty_dnswitch_get_illumin  ComponentB= %16.2f\n",
               0.1 * (FH8636_dn_calc_data.GlobalB * dn_return.B_Return / FH8636_dn_calc_data.Exposure));
        printf("当前照度值ILLUMIN = %d\n", *illumin_result);
    } else // auto或者其他状态
    {
        printf("***********************************************************\n");
        printf("***********************************************************\n");
        printf("当前日夜模式 = %d\n", FH8636_dn_config.cur_irmode);
        printf("FH8636_dn_calc_data.ExpTime = %d\n", FH8636_dn_calc_data.ExpTime);
        printf("FH8636_dn_calc_data.AGain = %d\n", FH8636_dn_calc_data.AGain);
        printf("FH8636_dn_calc_data.ISPDGain = %d\n", FH8636_dn_calc_data.ISPDGain);
        printf("FH8636_dn_calc_data.ISO = %d\n", FH8636_dn_calc_data.ISO);
        printf("FH8636_dn_calc_data.Exposure =  %16.2f\n", FH8636_dn_calc_data.Exposure);
        printf("FH8636_dn_calc_data.RGBComponent =  %16.2f\n", FH8636_dn_calc_data.RGBComponent);
        printf("FH8636_dn_calc_data.GlobalR = %d\n", FH8636_dn_calc_data.GlobalR);
        printf("FH8636_dn_calc_data.GlobalG = %d\n", FH8636_dn_calc_data.GlobalG);
        printf("FH8636_dn_calc_data.GlobalB = %d\n", FH8636_dn_calc_data.GlobalB);

        printf("__ty_dnswitch_get_illumin  coffR= %16.2f\n", dn_return.R_Return);
        printf("__ty_dnswitch_get_illumin  coffG= %16.2f\n", dn_return.G_Return);
        printf("__ty_dnswitch_get_illumin  coffB= %16.2f\n", dn_return.B_Return);
        printf("__ty_dnswitch_get_illumin  ComponentR_DAY= %16.2f\n",
               0.3 * (FH8636_dn_calc_data.GlobalR / FH8636_dn_calc_data.Exposure));
        printf("__ty_dnswitch_get_illumin  ComponentG_DAY= %16.2f\n",
               0.6 * (FH8636_dn_calc_data.GlobalG / FH8636_dn_calc_data.Exposure));
        printf("__ty_dnswitch_get_illumin  ComponentB_DAY= %16.2f\n",
               0.1 * (FH8636_dn_calc_data.GlobalB / FH8636_dn_calc_data.Exposure));
        printf("__ty_dnswitch_get_illumin  ComponentR_NIGHT= %16.2f\n",
               0.3 * (FH8636_dn_calc_data.GlobalR * dn_return.R_Return / FH8636_dn_calc_data.Exposure));
        printf("__ty_dnswitch_get_illumin  ComponentG_NIGHT= %16.2f\n",
               0.6 * (FH8636_dn_calc_data.GlobalG * dn_return.G_Return / FH8636_dn_calc_data.Exposure));
        printf("__ty_dnswitch_get_illumin  ComponentB_NIGHT= %16.2f\n",
               0.1 * (FH8636_dn_calc_data.GlobalB * dn_return.B_Return / FH8636_dn_calc_data.Exposure));
        printf("当前照度值ILLUMIN = %d\n", *illumin_result);
    }
#endif

    return 0;
}

/***********************************************************
 *  函数: tkl_vi_dnswitch_illumin_calculate
 *  功能：illumin_result的计算过程。
 *  参数: OUT INT32_T *illumin_result 日夜切换算法的计算结果,* 默认是整数,夜视强反光场景会返回result=1并伴有串口提示。
 *  返回值: <0 失败,=0 成功
 ***********************************************************/
OPERATE_RET tkl_vi_dnswitch_illumin_calculate(INT32_T *illumin_result)
{
    dn_return.R_Return = 0;
    dn_return.G_Return = 0;
    dn_return.B_Return = 0;

    dn_return.R_Return =
        (FH8636_dn_calc_data.coffR[0] +
         FH8636_dn_calc_data.coffR[1] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalR) +
         FH8636_dn_calc_data.coffR[2] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalB) +
         FH8636_dn_calc_data.coffR[3] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalR) *
             (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalR) +
         FH8636_dn_calc_data.coffR[4] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalB) *
             (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalB) +
         FH8636_dn_calc_data.coffR[5] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalR) *
             (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalB));
    dn_return.G_Return =
        (FH8636_dn_calc_data.coffG[0] +
         FH8636_dn_calc_data.coffG[1] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalR) +
         FH8636_dn_calc_data.coffG[2] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalB) +
         FH8636_dn_calc_data.coffG[3] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalR) *
             (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalR) +
         FH8636_dn_calc_data.coffG[4] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalB) *
             (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalB) +
         FH8636_dn_calc_data.coffG[5] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalR) *
             (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalB));
    dn_return.B_Return =
        (FH8636_dn_calc_data.coffB[0] +
         FH8636_dn_calc_data.coffB[1] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalR) +
         FH8636_dn_calc_data.coffB[2] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalB) +
         FH8636_dn_calc_data.coffB[3] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalR) *
             (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalR) +
         FH8636_dn_calc_data.coffB[4] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalB) *
             (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalB) +
         FH8636_dn_calc_data.coffB[5] * (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalR) *
             (FH8636_dn_calc_data.GlobalG * 1.0 / FH8636_dn_calc_data.GlobalB));

    /* 系数修正 */
    if (dn_return.R_Return < 0) {
        dn_return.R_Return = 0.1 - dn_return.R_Return;
    }
    if (dn_return.R_Return > FH8636_dn_config.offsetR[1]) {
        dn_return.R_Return = (dn_return.R_Return - FH8636_dn_config.offsetR[1]) * FH8636_dn_config.offsetR[0] +
                             FH8636_dn_config.offsetR[1];
    }
    if (dn_return.G_Return < 0) {
        dn_return.G_Return = 0.1 - dn_return.G_Return;
    }
    if (dn_return.G_Return > FH8636_dn_config.offsetG[1]) {
        dn_return.G_Return = (dn_return.G_Return - FH8636_dn_config.offsetG[1]) * FH8636_dn_config.offsetG[0] +
                             FH8636_dn_config.offsetG[1];
    }
    if (dn_return.B_Return < 0) {
        dn_return.B_Return = 0.1 - dn_return.B_Return;
    }
    if (dn_return.B_Return > FH8636_dn_config.offsetB[1]) {
        dn_return.B_Return = (dn_return.B_Return - FH8636_dn_config.offsetB[1]) * FH8636_dn_config.offsetB[0] +
                             FH8636_dn_config.offsetB[1];
    }

    /* 强反光判断 */
    if (((dn_return.R_Return < 0.07) || (dn_return.G_Return < 0.08) || (dn_return.B_Return < 0.08)) &&
        (FH8636_dn_calc_data.ExpTime < 10000)) {
        FH8636_dn_calc_data.OverExpFlag = 1;
        *illumin_result = 1;
        printf("into night mode with a strong reflection scene!\n");
    } else {
        FH8636_dn_calc_data.OverExpFlag = 0;
        FH8636_dn_calc_data.RGBComponent = (0.3 * dn_return.R_Return * FH8636_dn_calc_data.GlobalR +
                                            0.6 * dn_return.G_Return * FH8636_dn_calc_data.GlobalG +
                                            0.1 * dn_return.B_Return * FH8636_dn_calc_data.GlobalB);
        FH8636_dn_calc_data.Illumination = FH8636_dn_calc_data.RGBComponent * 10 /
                                           ((FH8636_dn_calc_data.Exposure == 0 ? 1 : FH8636_dn_calc_data.Exposure));
        *illumin_result = (int)FH8636_dn_calc_data.Illumination;
    }

    return 0;
}

/***********************************************************
 *  函数: tkl_vi_isp_set_smartir_config
 *  功能：
 *  参数:
 *
 *  返回值: <0 失败,=0 成功
 ***********************************************************/
OPERATE_RET tkl_vi_isp_set_smartir_config()
{
    //君正暂时不需要smartir功能,此接口留空
    return 0;
}

/***********************************************************
*  函数: tkl_vi_isp_show_fw_version
*  功能：显示当前设备的isp参数版本号,每次上电启动时串口输出。
*  参数: ISP_FW_VERSION,宏定义,当前设备的ISP参数版本号
         ISP_SENSOR_NAME,宏定义,当前设备的sensor名称
*
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_show_fw_version(PCHAR_T version, PCHAR_T name)
{
    if (name != NULL) {
        printf("Sensor name:%s\n", name);
    }

    if (version != NULL) {
        printf("ISP Firmware version:%s\n", version);
    }

    return 0;
}
#if 0
/***********************************************************
*  函数: tkl_vi_isp_get_debug_log
*  功能：ISP的图像参数调试接口,提测固件不带,仅供调试和修bug
*  参数: 先留空,根据实际需求来添加数据类型
*      
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_get_debug_log()
{
    //此接口留空
    return 0;
}

/***********************************************************
*  函数: tkl_vi_isp_uninit
*  功能：isp反初始化
*  参数: 无
*      
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_uninit()
{
    //此接口留空
    return 0;
}
#endif
//--------------------------------------------------------//
//------2.用户自定义的图像参数,可以只实现部分接口--------//
//--------------------------------------------------------//

/***********************************************************
 *  函数: tkl_vi_isp_set_day_figure_style
 *  功能：
 *  参数:
 *
 *  返回值: <0 失败,=0 成功
 ***********************************************************/
static OPERATE_RET tkl_vi_isp_set_day_figure_style(TKL_VI_CHN_E chn, TKL_ISP_FIG_STYLE_CONFIG_T *p_figure_style)
{
    
    (void)chn;
    // 富瀚接口支持亮度、饱和度、对比度，不支持锐度和gamma，后者要自己适配到fh isp库中.
    // ISP_BRIGHTNESS_CFG pstBrightCfg = {0};
    ISP_SAT_CFG pstSatCfg = {0};
    ISP_CONTRAST_CFG pstContrastCfg = {0};
    int ret = -1;
    int u32IspDevId = 0;
    //int i = 0;
    // 富瀚接口的传参范围不是[0,100]，而且有初始值，需要做换算。
    int Ref_Saturation = 76; // FH_UINT8 u08Sat的参考值，饱和度调整因子 U3.5 | [0-0xff]
    int Ref_Contrast = 64;   // FH_UINT8 u08Crt的参考值，手动模式下对比度 U2.6 | [0-0xff]
    int Result_Saturation = 0;
    int Result_Contrast = 0;
    int sharpness = 0;
    AE_DEFAULT_CFG ae_default_s;
    TY_FH8636_VI_HANDLE_T *pViHand = tkl_vi_get_handle();
    int brightness = 0;
    // int sharp_mode = 0;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();

    ret = API_ISP_GetAeDefaultCfg(u32IspDevId, &ae_default_s);
    if (ret < 0) {
        printf("API_ISP_GetAeDefaultCfg failed\n");
    }

    ret = API_ISP_GetSaturation(u32IspDevId, &pstSatCfg);
    TY_CHECK_RETURN(ret, -1);
    ret = API_ISP_GetContrastCfg(u32IspDevId, &pstContrastCfg);
    TY_CHECK_RETURN(ret, -1);

    //    printf ("*********isp style log day******* \r\n");
    //    printf ("p_figure_style->brightness = %d \r\n", p_figure_style->brightness);
    //    printf ("p_figure_style->contrast = %d \r\n", p_figure_style->contrast);
    //    printf ("p_figure_style->gamma = %d \r\n", p_figure_style->gamma);
    //    printf ("p_figure_style->saturation = %d \r\n", p_figure_style->saturation);
    //    printf ("p_figure_style->sharpness = %d \r\n", p_figure_style->sharpness);
    //    printf ("pstBrightCfg.bYcEn = %d \r\n", pstBrightCfg.bYcEn);
    //  printf ("pstBrightCfg.bGainMappingEn = %d \r\n", pstBrightCfg.bGainMappingEn);
    //    printf ("pstBrightCfg.s08Brt = %d \r\n", pstBrightCfg.s08Brt);
    //  for(i=0;i<12;i++)
    //      printf ("pstBrightCfg.s08BrtMap[%d] = %d \r\n", i, pstBrightCfg.s08BrtMap[i]);
    //    printf ("pstSatCfg.bSatEn = %d \r\n", pstSatCfg.bSatEn);
    //    printf ("pstSatCfg.bGainMappingEn = %d \r\n", pstSatCfg.bGainMappingEn);
    //    printf ("pstSatCfg.u08Sat = %d \r\n", pstSatCfg.u08Sat);
    //    printf ("pstContrastCfg.bYcEn = %d \r\n", pstContrastCfg.bYcEn);
    //    printf ("pstContrastCfg.bGainMappingEn = %d \r\n", pstContrastCfg.bGainMappingEn);
    //    printf ("pstContrastCfg.u08Crt = %d \r\n", pstContrastCfg.u08Crt);

    Result_Saturation = Ref_Saturation + (p_figure_style->saturation - 50); //[0,100] to [0~256]，考虑到实测效果缩减到[0~128]
    if (Result_Saturation >= 128)
        Result_Saturation = 128;
    else if (Result_Saturation <= 0) 
        Result_Saturation = 0;

    Result_Contrast = Ref_Contrast + (p_figure_style->contrast - 50); //[0,100] to [0~256]，考虑到实测效果缩减到[0~128]
    if (Result_Contrast >= 128) 
        Result_Contrast = 128;
    else if (Result_Contrast <= 0)
        Result_Contrast = 0;

    pstSatCfg.u08Sat = Result_Saturation;
    pstContrastCfg.u08Crt = Result_Contrast;

    sharpness = p_figure_style->sharpness * 255 / 100;
    sharpness = (sharpness > 255) ? 255 : sharpness;   //限制到0-255

    //    printf ("after pstBrightCfg.s08Brt = %d \r\n", pstBrightCfg.s08Brt);
    //  printf ("after pstSatCfg.u08Sat = %d \r\n", pstSatCfg.u08Sat);
    //    printf ("after pstContrastCfg.u08Crt = %d \r\n", pstContrastCfg.u08Crt);
    if(pHand->isp_wdr_mode == 1){
        brightness = pViHand->ref_luma[2] + p_figure_style->brightness - 50;
    } else {
        brightness = pViHand->ref_luma[0] + p_figure_style->brightness - 50;
    }
    brightness = (brightness  > 255) ? 255 : brightness ;   //限制到0-255
    brightness = (brightness  < 0) ? 0 : brightness ;       //限制到0-255
    ae_default_s.stAeLumaCfg.u08LumaRef = (FH_UINT8)(brightness);

    ret = API_ISP_SetAeDefaultCfg(u32IspDevId, &ae_default_s);
    if (ret < 0) {
        printf("API_ISP_SetAeDefaultCfg failed\n");
    }
    // ret = API_ISP_SetSaturation(u32IspDevId, &pstSatCfg);
    // TY_CHECK_RETURN(ret, -1);
    // ret = API_ISP_SetContrastCfg(u32IspDevId, &pstContrastCfg);
    // TY_CHECK_RETURN(ret, -1);
    ret = FHAdv_Isp_SetSharpeness(0, 1, sharpness);
    TY_CHECK_RETURN(ret, -1);
    
    return 0;
}

/***********************************************************
 *  函数: tkl_vi_isp_set_night_figure_style
 *  功能：
 *  参数:
 *
 *  返回值: <0 失败,=0 成功
 ***********************************************************/
static OPERATE_RET tkl_vi_isp_set_night_figure_style(TKL_VI_CHN_E chn, TKL_ISP_FIG_STYLE_CONFIG_T *p_figure_style)
{
    (void)chn;
    // 富瀚接口支持亮度、饱和度、对比度，不支持锐度和gamma，后者要自己适配到fh isp库中.
    // ISP_BRIGHTNESS_CFG pstBrightCfg = {0};
    ISP_SAT_CFG pstSatCfg = {0};
    ISP_CONTRAST_CFG pstContrastCfg = {0};
    int ret = -1;
    AE_DEFAULT_CFG ae_default_s;
    TY_FH8636_VI_HANDLE_T *pViHand = tkl_vi_get_handle();
    int u32IspDevId = 0;

    // 富瀚接口的传参范围不是[0,100]，而且有初始值，需要做换算。
    int Ref_Bright = 0; // FH_SINT8 s08Brt的参考值，手动模式下亮度S8(-128到127) | [-128~127]
    // int Ref_Saturation = 76; // FH_UINT8 u08Sat的参考值，饱和度调整因子 U3.5 | [0-0xff]
    int Ref_Contrast = 64; // FH_UINT8 u08Crt的参考值，手动模式下对比度 U2.6 | [0-0xff]
    int Result_Bright = 0;
    int Result_Saturation = 0;
    int Result_Contrast = 0;
    int sharpness = 0;
    int brightness = 0;
    // int sharp_mode = 0;

    ret = API_ISP_GetAeDefaultCfg(u32IspDevId, &ae_default_s);
    if (ret < 0) {
        printf("API_ISP_GetAeDefaultCfg failed\n");
    }
    // ret = API_ISP_GetBrightnessCfg(&pstBrightCfg);
    // TY_CHECK_RETURN(ret, -1);
    ret = API_ISP_GetSaturation(u32IspDevId, &pstSatCfg);
    TY_CHECK_RETURN(ret, -1);
    ret = API_ISP_GetContrastCfg(u32IspDevId, &pstContrastCfg);
    TY_CHECK_RETURN(ret, -1);

    Result_Bright = Ref_Bright + (p_figure_style->brightness - 50) * 2; //[0,100] to [-128,127]，考虑到实测效果缩减到[-100~100]
    if (Result_Bright >= 100)
        Result_Bright = 100;
    else if (Result_Bright <= -100)
        Result_Bright = -100;

    Result_Contrast = Ref_Contrast + (p_figure_style->contrast - 50); //[0,100] to [0~256]，考虑到实测效果缩减到[0~128]
    if (Result_Contrast >= 128)
        Result_Contrast = 128;
    else if (Result_Contrast <= 0)
        Result_Contrast = 0;

    pstSatCfg.u08Sat = Result_Saturation;
    pstContrastCfg.u08Crt = Result_Contrast;

    sharpness = p_figure_style->sharpness * 255 / 100;
    sharpness = (sharpness > 255) ? 255 : sharpness;   //限制到0-255

    brightness = pViHand->ref_luma[1] + p_figure_style->brightness - 50;
    brightness = (brightness  > 255) ? 255 : brightness ;   //限制到0-255
    brightness = (brightness  < 0) ? 0 : brightness ;       //限制到0-255
    ae_default_s.stAeLumaCfg.u08LumaRef = (FH_UINT8)(brightness);

    ret = API_ISP_SetAeDefaultCfg(u32IspDevId, &ae_default_s);
    if (ret < 0) {
        printf("API_ISP_SetAeDefaultCfg failed\n");
    }

    // ret = API_ISP_SetContrastCfg(u32IspDevId, &pstContrastCfg);
    // TY_CHECK_RETURN(ret, -1);
    ret = FHAdv_Isp_SetSharpeness(u32IspDevId, 1, sharpness);
    TY_CHECK_RETURN(ret, -1);

    //  //夜视不设置saturation
    //  Result_Saturation = Ref_Saturation + (p_figure_style->saturation-50) ;  //[0,100] to
    //[0~256]，考虑到实测效果缩减到[0~128]     if (Result_Saturation >= 128)       Result_Saturation = 128;    else if(
    // Result_Saturation <= 0)      Result_Saturation = 0;
    //    ret = API_ISP_SetSaturation(&pstSatCfg);
    return 0;
}
#if 0
/***********************************************************
*  函数: tkl_vi_isp_set_awb_config
*  功能：
*  参数: 
*      
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_set_awb_config(TKL_VI_CHN_E chn, TKL_ISP_AWB_CONFIG_T *p_awb_config)
{
    //接口留空,待实现
    
    return 0;
}

/***********************************************************
*  函数: tkl_vi_isp_set_wdr_config
*  功能：
*  参数: 
*      
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_set_wdr_config(TKL_VI_CHN_E chn, TKL_ISP_WDR_CONFIG_T *p_wdr_config)
{
    //君正暂缺wdr强度调节接口,此接口留空。
    return 0;
}

/***********************************************************
*  函数: tkl_vi_isp_set_denoise_config
*  功能：
*  参数: 
*      
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_set_denoise_config(TKL_VI_CHN_E chn, TKL_ISP_DENOISE_CONFIG_T *p_denoise_config)
{
    
    return 0;
}

//--------------------------------------------------------//
//-----3.跟ISO和AE联动的动态参数,可以只实现部分接口------//
//--------------------------------------------------------//

/***********************************************************
*  函数: tkl_vi_isp_set_day_shading
*  功能：
*  参数: 
*      
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_set_day_shading(TKL_VI_CHN_E chn, TKL_ISP_SHADING_CONFIG_T *p_shading_config)
{
    //配置LSC模块，开启shading矫正
    int ret = 0;
    /*
    unsigned int moduleCfg = 0;
    ISP_LSC_CFG pstLscCfg = {0};

    pstLscCfg.bLscEn = 1;
    memcpy(pstLscCfg.u32Coff ,g_Tuya_ISP_FH8636_SHADING_CONFIG.u32ShadingConf, 299 * sizeof(unsigned int));

    ret = API_ISP_Get_HWmodule_cfg(&moduleCfg);
    if(ret != 0)
    {
        printf("API_ISP_Get_HWmodule_cfg failed!!! ret = 0x%x\n", ret);
    }
    
    if (irmode == 1)
    {
        moduleCfg = moduleCfg | 0x10;
    }
    else
    {
        moduleCfg = moduleCfg & 0xFFFFFFEF;
    }
    ret = API_ISP_Set_HWmodule_cfg(moduleCfg);
    if(ret != 0)
    {
        printf("API_ISP_Set_HWmodule_cfg failed!!! ret = 0x%x\n", ret);
    }

    ret = API_ISP_SetLscCfg(&pstLscCfg);
    if(ret != 0)
    {
        printf("API_ISP_SetLscCfg failed!!! ret = 0x%x\n", ret);
    }
    */
    return ret;
}

/***********************************************************
*  函数: tkl_vi_isp_set_night_shading
*  功能：
*  参数: 
*      
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_set_night_shading(TKL_VI_CHN_E chn, TKL_ISP_SHADING_CONFIG_T *p_shading_config)
{
    //夜视暂时不需要动态shading调节功能,此接口留空。
    return 0;
}

/******************************************************************************
*  函数: tkl_vi_isp_save_param
*  功能：保存ISP参数信息到/mnt/sdcard/isp0.log日志文件中
*  参数: INT32_T iRLuma 日夜切换算法计算出的画面实时照度值 
*  返回值: 无
******************************************************************************/
VOID tkl_vi_isp_save_param()
{
    return;
}
#endif

/******************************************************************************
*  函数: tkl_vi_isp_get_param
*  功能：从平台接口获取ISP pipeline各个模块参数,目前仅获取AE和AWB参数
*  参数: IN TKL_VI_CHN_E chn 设备号
         OUT TKL_ISP_LOG_INFO_T *pstISPParam isp日志功能打开后，需要显示的参数
*  返回值:    <0 失败,=0 成功
******************************************************************************/
OPERATE_RET tkl_vi_isp_get_param(TKL_VI_CHN_E chn, TKL_ISP_LOG_INFO_T *pstISPParam)
{
    (void)chn;
    (void)pstISPParam;
    int ret = -1;
    ISP_AE_STATUS FH8636_ae_params;
    ISP_AWB_STATUS FH8636_awb_params;
    int u32IspDevId = 0;

    ret = API_ISP_GetAeStatus(u32IspDevId, &FH8636_ae_params);
    if (ret < 0) {
        printf("API_ISP_GetAeStatus failed\n");
        return -1;
    }

    ret = API_ISP_GetAwbStatus(u32IspDevId, &FH8636_awb_params);
    if (ret < 0) {
        printf("API_ISP_GetAwbStatus failed\n");
        return -1;
    }

	pstISPParam->US = FH8636_ae_params.u32CurrIntt;
	pstISPParam->R = (unsigned short)(FH8636_awb_params.u16RAvg) * 50;
	pstISPParam->G = (unsigned short)(FH8636_awb_params.u16GAvg) * 50;	
	pstISPParam->B = (unsigned short)(FH8636_awb_params.u16BAvg) * 50;
	pstISPParam->TotalGain = FH8636_ae_params.u16CurrAgain * FH8636_ae_params.u16CurrDgain;
    //    /*获取ADN曝光信息*/
    //    pstISPParam->dayToNight = FH8636_dn_config.dayToNight;
    //    pstISPParam->nightToDay = FH8636_dn_config.nightToDay;
    //    pstISPParam->Illumination = FH8636_dn_calc_data.Illumination;

    return 0;
}

OPERATE_RET tkl_vi_isp_set_fig_style_config(TKL_VI_CHN_E chn, TKL_ISP_FIG_STYLE_MODE_E mode, TKL_ISP_FIG_STYLE_CONFIG_T *config)
{
    (void)chn;
    int ret = 0;
    TY_FH8636_VI_HANDLE_T *pViHand = tkl_vi_get_handle();

    if (config == NULL) {
        printf("tkl_vi_isp_set_fig_style_config invalid input config:%p\n", config);
        return -1;
    }

    if (mode == FIG_STYLE_DAY) {
        pViHand->config.isp.isp_fig_style_day = *config;
        if (FH8636_dn_config.cur_irmode == DAY_MODE) {
            ret = tkl_vi_isp_set_day_figure_style(pViHand->config.chn, &pViHand->config.isp.isp_fig_style_day);
            TY_CHECK_RETURN(ret, -1);
        }
    } else if(mode == FIG_STYLE_NIGHT) {
        pViHand->config.isp.isp_fig_style_night = *config;
        if (FH8636_dn_config.cur_irmode == NIGHT_MODE) {
            ret = tkl_vi_isp_set_night_figure_style(pViHand->config.chn, &pViHand->config.isp.isp_fig_style_night);
            TY_CHECK_RETURN(ret, -1);
        }
    } else {
        printf("tkl_vi_isp_set_fig_style_config invalid input mode:%d\n", mode);
        return -1;
    }

    return 0;
}

OPERATE_RET tkl_vi_isp_get_fig_style_config(TKL_VI_CHN_E chn, TKL_ISP_FIG_STYLE_MODE_E mode, TKL_ISP_FIG_STYLE_CONFIG_T *config)
{
    (void)chn;

    TY_FH8636_VI_HANDLE_T *pViHand = tkl_vi_get_handle();

    if (config == NULL) {
        printf("tkl_vi_isp_get_fig_style_config invalid input config:%p\n", config);
        return -1;
    }

    if (mode == FIG_STYLE_DAY) {
        *config = pViHand->config.isp.isp_fig_style_day;

    } else if(mode == FIG_STYLE_NIGHT) {
        *config = pViHand->config.isp.isp_fig_style_night;
    } else {
        printf("tkl_vi_isp_get_fig_style_config invalid input mode:%d\n", mode);
        return -1;
    }

    return 0;
}

/***********************************************************
*  函数: tkl_vi_isp_get_sensor_fps
*  功能：获取sensor的帧率（该帧率能影响sensor的最大曝光时间）
*  参数: IN TKL_VI_CHN_E chn 设备号
         OUT INT32_T *sensor_fps,当前sensor帧率的实际值（该帧率能影响sensor的最大曝光时间）
*      
*  返回值: <0 失败,=0 成功
***********************************************************/
OPERATE_RET tkl_vi_isp_get_sensor_fps(TKL_VI_CHN_E chn, INT32_T *sensor_fps)
{
    (void)chn;
    (void)sensor_fps;
    // OPERATE_RET ret = 0;
    // UINT32_T fps_num = 1;    //帧率公式的分子   
    // UINT32_T fps_den = 1;    //帧率公式的分母
    
    // ret = IMP_ISP_Tuning_GetSensorFPS(&fps_num, &fps_den);
    // if(ret<0) {
    //  TYDEBUG("tkl_vi_isp_set_sensor_fps failed！\n");
    // }    

    // *sensor_fps = fps_num/fps_den;   
    return 0;
}

/***********************************************************
*  函数: tkl_vi_isp_set_sensor_fps
*  功能：设置sensor的帧率（该帧率能影响sensor的最大曝光时间）
*  参数: IN TKL_VI_CHN_E chn 设备号
         IN INT32_T sensor_fps,sensor帧率的配置值（该帧率能影响sensor的最大曝光时间）
*      
*  返回值: <0 失败,=0 成功
***********************************************************/

OPERATE_RET tkl_vi_isp_set_sensor_fps(TKL_VI_CHN_E chn, INT32_T sensor_fps)
{
    (void)chn;
    (void)sensor_fps;
    // OPERATE_RET ret = 0;
    // UINT32_T fps_den = 1;    //帧率公式的分母，默认为1
    
    // ret = IMP_ISP_Tuning_SetSensorFPS(sensor_fps, fps_den);
    // if(ret<0) {
    //  TYDEBUG("tkl_vi_isp_set_sensor_fps failed!\n"); 
    // }
    
    // int i = 0;
    // for (i = 0; i <= T31_VI_SUB_CHN; i++) {
    //  ret = ty_t31_set_venc_GOP(i, 2 * sensor_fps);
    //  if (ret != 0) {
    //      TYERROR("set venc GOP failed ret:%d\n", ret);
    //  }
    // }

    return 0;
}

/**
* @brief vi isp set WDR
* 
* @param[in] chn: vi chn
* @param[in] mode: WDR mode, 0-disable, 1-auto, 2-manual 
* @param[in] strength: WDR strength, [0, 100]
*
* @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
*/
OPERATE_RET tkl_vi_isp_set_wdr(TKL_VI_CHN_E chn, INT32_T mode, INT32_T strength)
{
    (void)chn;
    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    if(mode < 0 || mode >1){
        printf("tkl_vi_isp_set_wdr failed, mode[%d] not support.\n", mode);
        return OPRT_COM_ERROR;
    }
    printf("tkl_vi_isp_set_wdr mode, mode[%d],strength[%d].\n", mode, strength);
    pHand->isp_wdr_mode = mode;
    return OPRT_OK;
}

/**
* @brief vi isp get WDR params
* 
* @param[in] chn: vi chn
* @param[out] *mode: WDR mode, 0-disable, 1-auto, 2-manual 
* @param[out] *strength: WDR strength
*
* @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
* */
OPERATE_RET tkl_vi_isp_get_wdr(TKL_VI_CHN_E chn, INT32_T *mode, INT32_T *strength)
{
    (void)chn;
    (void)strength;
    if(mode){
        TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
        *mode = pHand->isp_wdr_mode;
        printf("tkl_vi_isp_get_wdr,mode[%d].\n", pHand->isp_wdr_mode);
        return OPRT_OK;
    }
    printf("tkl_vi_isp_get_wdr failed,mode null.\n");
    return OPRT_COM_ERROR;
}

