/* 适用芯片: FH865x系列 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <signal.h>
#include <sys/prctl.h>
#include <unistd.h>
#include <math.h>
#include <assert.h>
#include <pthread.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include "tkl_video_enc.h"
#include "types/bufCtrl.h"
#include "dsp/fh_system_mpi.h"
#include "dsp/fh_nna_mpi.h"
#include "dsp/fh_vpu_mpi.h"
#include "dsp/fh_venc_mpi.h"
#include "types/type_def.h"
#include "FH_typedef.h"
#include "tkl_fh8636_video.h"
#include "tkl_fh8636_hd.h"
#include "dsp/fh_common.h"
#include "dsp_ext/FHAdv_OSD_mpi.h"
#include "ty_target_track.h"

#define RGB_CHAN (3)
#define NN_CHAN  (0)
// #define TY_NNA_DRAW_BOX

typedef struct {
    pthread_mutex_t lock;            // 读写数据的锁
    pthread_mutex_t lock_start_stop; // 开启宠物，人形时，互斥锁
    int32_t g_stop_nna_detect;
    int32_t g_nna_running;
    uint32_t flag; // 记录宠物，人形开启状态，
                   //  0，都关闭，1<<TKL_MEDIA_DETECT_TYPE_E,
                   // 人形开启，1<<TKL_MEDIA_DETECT_TYPE_E, 宠物开启， 1<<2|1<<1，
                   // 两者都开启
    void *pnna_handle;
    TKL_VI_DETECT_RESULT_T hd_result; // 存储人形检测结果
    TKL_VI_DETECT_RESULT_T pd_result; // 宠物检测结果
    void *tracker_hdl;                // 目标追踪
    int track_id[MAX_TRACKERS_NUM];    // 记录曾经超过阈值的检测目标id
} TY_DEV_DETECT_HANDLE_T;

static TY_DEV_DETECT_HANDLE_T g_ty_detect_handle = {0};

/**
 * @fn tycam_t31_video_handle_s * __get_video_handle()
 * @获取视频句柄
 * @param[in] 无
 * @retval 视频参数句柄指针
 * @attention 无。
 */
TY_DEV_DETECT_HANDLE_T *ty_dev_get_detect_handle()
{
    return &g_ty_detect_handle;
}

static int32_t __nna_deinit(void *detHandle)
{
    int ret = 0;

    if (detHandle) {
        ret = FH_NNA_DET_Exit(detHandle);
        if (ret != RETURN_OK) {
            printf("Error[FH_NN]: FH_NNA_DET_Exit failed, ret = %x\n", ret);
        }
    }
    return ret;
}

static void *__init_fh_nn(TKL_VI_DETECT_CONFIG_T *p_config)
{
    uint32_t ret = -1;
    void *detHandle = NULL;
    FH_NN_INIT_PARAM_T nna_param = {0};
    TY_FH8636_VIDEO_HANDLE_T *pvenc_hand = __get_video_handle();

    nna_param.src_w_in = pvenc_hand->vpuParam[RGB_CHAN].chnAttr.vpu_chn_size.u32Width;
    nna_param.src_h_in = pvenc_hand->vpuParam[RGB_CHAN].chnAttr.vpu_chn_size.u32Height;
    // printf("nna_param w h :%d %d\n", nna_param.src_w_in, nna_param.src_h_in);
    nna_param.src_c_in = 4; // 位深
    nna_param.type = FN_DET_TYPE_DEF_03;

    nna_param.conf_thr = 0.6;
    nna_param.rotate = FN_ROT_0;

    nna_param.nbg_data = (FH_UINT8 *)p_config->pmodel;
    if (!nna_param.nbg_data) {
        return NULL;
    }

    ret = FH_NNA_DET_Init(&detHandle, 0, &nna_param);
    if (ret) {
        nna_param.type = FN_DET_TYPE_C2;
        ret = FH_NNA_DET_Init(&detHandle, 0, &nna_param);
        if (ret) {
            printf("FH_NNA_DET_Init failed with %x\n", ret);
            detHandle = NULL;
        }
    }
    // printf("FH_NNA_DET_Init ok !!! %p\n", detHandle);
    return detHandle;
}
#ifdef TY_NNA_DRAW_BOX
char nn_log_buf[16];
static FH_SINT32 __set_gbox(FH_UINT32 chan, FH_UINT32 enable, FH_UINT32 box_id, TKL_VI_RECT_T *p_box, int score)
{
    FH_SINT32 ret;
    chan = 1;
    FHT_OSD_Gbox_t gbox_cfg = {0};
    TY_FH8636_VIDEO_HANDLE_T *pvenc_hand = __get_video_handle();
    int main_w = pvenc_hand->viParam.viSize.vi_size.u32Width; // 0 的时候和vi 的宽高保持一致
    int main_h = pvenc_hand->viParam.viSize.vi_size.u32Height;
    main_w = pvenc_hand->vpuParam[chan].chnAttr.vpu_chn_size.u32Width;
    main_h = pvenc_hand->vpuParam[chan].chnAttr.vpu_chn_size.u32Height;
    FHT_RgbColor_t color = {255, 0, 0, 255};
    gbox_cfg.enable = enable; /* 使能gbox显示 */
    gbox_cfg.gboxId = box_id; /* gbox ID 为0 */
    gbox_cfg.max_frame_w = main_w;
    gbox_cfg.max_frame_h = main_h;
    gbox_cfg.gboxLineWidth = 8; /* gbox边框像素宽度为8*/
    gbox_cfg.rotate = 0;        /* 不旋转 */
    gbox_cfg.osdColor.fAlpha = color.fAlpha;
    gbox_cfg.osdColor.fRed = color.fRed;
    gbox_cfg.osdColor.fGreen = color.fGreen;
    gbox_cfg.osdColor.fBlue = color.fBlue;
    if (enable) {
        gbox_cfg.area.fTopLeftX = p_box->x * main_w;   /* gbox左上角起始点像素宽度位置 */
        gbox_cfg.area.fTopLeftY = p_box->y * main_h;   /* gbox左上角起始点像素高度位置 */
        gbox_cfg.area.fWidth = p_box->width * main_w;  /* gbox宽度 */
        gbox_cfg.area.fHeigh = p_box->height * main_h; /* gbox高度 */
    } else /*just work-around FHAdv_Osd_Ex_SetGbox,it will check fWidth and fHeight*/
    {
        gbox_cfg.area.fTopLeftX = 0;
        gbox_cfg.area.fTopLeftY = 0;
        gbox_cfg.area.fWidth = 16;
        gbox_cfg.area.fHeigh = 16;
    }
    ret = FHAdv_Osd_Ex_SetGbox(FH_GRP_ID, chan, &gbox_cfg);
    if (ret != FH_SUCCESS) {
        printf("FHAdv_Osd_Ex_SetGbox failed with %x, enable:%d, box_id:%d, chan:%d  main_w, main_h:%d %d\n", ret,
               enable, box_id, chan, main_w, main_h);
        return ret;
    }
    if (enable == 1) {
        sprintf(nn_log_buf, " %d-%d", box_id, score);
    }
    return 0;
}
static FH_SINT32 clear_draw_gbox(int id)
{
    FH_UINT32 ret = -1;
    TKL_VI_RECT_T box = {0};
    ret = __set_gbox(0, false, id, &box, 0);
    if (ret) {
        printf("Error: __set_gbox chn:0,ret =  %d\n", ret);
        return ret;
    }
    return 0;
}

static FH_SINT32 clear_draw_gbox_all()
{
    int i = 0;
    for (i = 0; i < TKL_VI_TARGET_MAX * 2; i++) {
        clear_draw_gbox(i);
    }
    return 0;
}

static void __draw_gbox(int id, TKL_VI_RECT_T *p_box, float score)
{
    __set_gbox(0, true, id, p_box, score * 100);
    return;
}

static FH_SINT32 __box_show(TY_DEV_DETECT_HANDLE_T *pdet_handl)
{
    int i = 0;
    for (i = 0; i < TKL_VI_TARGET_MAX; i++) {
        if (i < pdet_handl->pd_result.count) {
#if 0
            if (pdet_handl->pd_result.count > 1){
                printf("draw [%d]nn_detect type:pd, box:x y w h:%f,%f,%f,%f\n", i,
                    pdet_handl->pd_result.target[i].draw_rect.x, pdet_handl->pd_result.target[i].draw_rect.y, 
                    pdet_handl->pd_result.target[i].draw_rect.width, pdet_handl->pd_result.target[i].draw_rect.height);
            }
#endif
            __draw_gbox(i, &pdet_handl->pd_result.target[i].draw_rect, pdet_handl->pd_result.target[i].score);
        } else {
            clear_draw_gbox(i);
        }
    }

    for (i = 0; i < TKL_VI_TARGET_MAX; i++) {
        if (i < pdet_handl->hd_result.count) {
#if 0
            if (pdet_handl->hd_result.count > 1){
                printf("draw [%d]nn_detect type:hd, box:x y w h:%f,%f,%f,%f\n", i,
                    pdet_handl->hd_result.target[i].draw_rect.x, pdet_handl->hd_result.target[i].draw_rect.y, 
                    pdet_handl->hd_result.target[i].draw_rect.width, pdet_handl->hd_result.target[i].draw_rect.height);
            }
#endif
            __draw_gbox(i + TKL_VI_TARGET_MAX, &pdet_handl->hd_result.target[i].draw_rect,
                        pdet_handl->hd_result.target[i].score);
        } else {
            clear_draw_gbox(i + TKL_VI_TARGET_MAX);
        }
    }

    return 0;
}
#endif

static uint64_t _fh8636_get_time_ms()
{
    struct timespec tsnow = {0};
    uint64_t u64ms = 0;

    clock_gettime(CLOCK_MONOTONIC, &tsnow);
    u64ms = tsnow.tv_sec;
    u64ms *= 1000ULL;
    u64ms += tsnow.tv_nsec / (1000 * 1000);
    return u64ms;
}
#if 0
// 更新目标框，过滤无效id
static void repro_track_rects(objGroup_t *trackOut)
{
    int i = 0;

    for (i = 0; i < trackOut->num; i++) {
        if (trackOut->objlist[i].id < 0) {
            continue;
        }

        if (trackOut->objlist[i].pos.w < 0) {
            trackOut->objlist[i].pos.x += trackOut->objlist[i].pos.w;
            trackOut->objlist[i].pos.w = -trackOut->objlist[i].pos.w;
        }

        if (trackOut->objlist[i].pos.h < 0) {
            trackOut->objlist[i].pos.y += trackOut->objlist[i].pos.h;
            trackOut->objlist[i].pos.h = -trackOut->objlist[i].pos.h;
        }

        if (trackOut->objlist[i].pos.x >= 1 || (trackOut->objlist[i].pos.x + trackOut->objlist[i].pos.w) <= 0 ||
            trackOut->objlist[i].pos.y >= 1 || trackOut->objlist[i].pos.y + trackOut->objlist[i].pos.h <= 0) {
            trackOut->objlist[i].id = -1;
            continue;
        }

        if (trackOut->objlist[i].pos.x < 0) {
            trackOut->objlist[i].pos.x = 0;
        }

        if (trackOut->objlist[i].pos.y < 0) {
            trackOut->objlist[i].pos.y = 0;
        }

        if ((trackOut->objlist[i].pos.x + trackOut->objlist[i].pos.w) > 1) {
            trackOut->objlist[i].pos.w = 1 - trackOut->objlist[i].pos.x;
        }

        if ((trackOut->objlist[i].pos.y + trackOut->objlist[i].pos.h) > 1) {
            trackOut->objlist[i].pos.h = 1 - trackOut->objlist[i].pos.y;
        }
    }
}
#endif

static int ___is_exist_id(int id, int *id_arr, int cnt)
{
    int k = 0;
    for (k = 0; k < cnt; k++) {
        if (id_arr[k] == id) {
            return 1;
        }
    }
    return 0;
}

static void __save_id_by_conf(SORT_TRACKING_LIST *p_out)
{
    TY_DEV_DETECT_HANDLE_T *pdet_handl = ty_dev_get_detect_handle();
    int old_id[MAX_TRACKERS_NUM] = {0};
    memcpy(old_id, pdet_handl->track_id, sizeof(int) * MAX_TRACKERS_NUM);
    memset(pdet_handl->track_id, 0, sizeof(int) * MAX_TRACKERS_NUM);
    int j = 0;
    int i = 0;

    for (i = 0; i < p_out->num && (p_out->item[i].id > 0); i++) {
        if (p_out->item[i].priv.conf > 0.75) {
            pdet_handl->track_id[j] = p_out->item[i].id;
            j++;
        } else {
            if (___is_exist_id(p_out->item[i].id, old_id, MAX_TRACKERS_NUM) == 1) {
                pdet_handl->track_id[j] = p_out->item[i].id;
                j++;
            }
        }
    }
}

// 获取结果和目标id
static void __get_result_and_id(SORT_TRACKING_LIST *p_in, TKL_VI_DETECT_RESULT_T *p_result_hd,
                                TKL_VI_DETECT_RESULT_T *p_result_pd)
{
    int ret = -1;
    int objNum = 0;
    SORT_TRACKING_LIST out = {0};
    TY_DEV_DETECT_HANDLE_T *pdet_handl = ty_dev_get_detect_handle();

    if (pdet_handl->tracker_hdl == NULL) {
        return;
    }

    ret = sort_tracker_update(pdet_handl->tracker_hdl, p_in, &out);
    if (ret < 0) {
        printf("sort_tracker_update failed\n");
        return;
    }

    __save_id_by_conf(&out);   // 记录置信度超过阈值的id

    // repro_track_rects(&out);
    for (objNum = 0; objNum < out.num; objNum++) {
        // printf("i:%d o:%d i:%d id:%d [%f,%f] match:%d\n", p_in->num, out.num, objNum, out.item[objNum].id,
        // out.item[objNum].box.x, out.item[objNum].box.y, out.item[objNum].b_match);
        if (out.item[objNum].id > 0 && out.item[objNum].b_match == 1) {
            if (___is_exist_id(out.item[objNum].id, pdet_handl->track_id, MAX_TRACKERS_NUM) <= 0) {
                continue;
            }

            if (out.item[objNum].priv.type == TKL_MEDIA_DETECT_TYPE_PET) {
                if (p_result_pd->count < TKL_VI_TARGET_MAX) {
                    p_result_pd->target[p_result_pd->count].type = TKL_MEDIA_DETECT_TYPE_PET;
                    p_result_pd->target[p_result_pd->count].draw_rect.x = out.item[objNum].box.x;
                    p_result_pd->target[p_result_pd->count].draw_rect.y = out.item[objNum].box.y;
                    p_result_pd->target[p_result_pd->count].draw_rect.width = out.item[objNum].box.width;
                    p_result_pd->target[p_result_pd->count].draw_rect.height = out.item[objNum].box.height;
                    p_result_pd->target[p_result_pd->count].score = out.item[objNum].priv.conf;
                    p_result_pd->target[p_result_pd->count].id = out.item[objNum].id;
                    p_result_pd->count++;
                }
            } else {
                if (p_result_hd->count < TKL_VI_TARGET_MAX) {
                    p_result_hd->target[p_result_hd->count].type = TKL_MEDIA_DETECT_TYPE_HUMAN;
                    p_result_hd->target[p_result_hd->count].draw_rect.x = out.item[objNum].box.x;
                    p_result_hd->target[p_result_hd->count].draw_rect.y = out.item[objNum].box.y;
                    p_result_hd->target[p_result_hd->count].draw_rect.width = out.item[objNum].box.width;
                    p_result_hd->target[p_result_hd->count].draw_rect.height = out.item[objNum].box.height;
                    p_result_hd->target[p_result_hd->count].score = out.item[objNum].priv.conf;
                    p_result_hd->target[p_result_hd->count].id = out.item[objNum].id;
                    p_result_hd->count++;
                }
            }
        }
    }

    // if (p_in->num != p_result_hd->count + p_result_pd->count) {
    //  printf("p_in num:%d hd:%d pd:%d \n", p_in->num, p_result_hd->count, p_result_pd->count);
    //}
}

#if 0
static FH_SINT32 __box_show_objGroup(objGroup_t *p_group, TKL_MEDIA_DETECT_TYPE_E type)
{
    int i = 0;
    int cnt = p_group->num;
    if (type == TKL_MEDIA_DETECT_TYPE_HUMAN) {
        for (i = 0; i < TKL_VI_TARGET_MAX; i++) {
            if (i < cnt) {
                __draw_gbox(i + TKL_VI_TARGET_MAX, (TKL_VI_RECT_T *)&p_group->objlist[i].pos, 1.0);
                if (cnt > 1){
                    printf("draw [%d]nn_detect type:%d, box:x y w h:%f,%f,%f,%f\n", i,
                        type, p_group->objlist[i].pos.x,p_group->objlist[i].pos.y, 
                        p_group->objlist[i].pos.w, p_group->objlist[i].pos.h);
                }
            } else {
                clear_draw_gbox(i + TKL_VI_TARGET_MAX);
            }
        }
    } else {
        for (i = 0; i < TKL_VI_TARGET_MAX; i++) {
            if (i < cnt) {
                __draw_gbox(i, (TKL_VI_RECT_T *)&p_group->objlist[i].pos, 1.0);
            } else {
                clear_draw_gbox(i);
            }
        }
    }
    return 0;
}
#endif

/* 512 * 288 */
void *thread_nn_detect(void *args)
{
    (void)args;
    int32_t ret;
    void *detHandle = NULL;
    FH_DETECTION_T nn_out = {0};
    uint32_t i = 0;
    SORT_TRACKING_LIST track_in = {0};

    TY_DEV_DETECT_HANDLE_T *pdet_handl = ty_dev_get_detect_handle();
    prctl(PR_SET_NAME, "dev_nn_detect");
    printf("thread_nn_detect begin\n");
    detHandle = pdet_handl->pnna_handle;
    while (!pdet_handl->g_stop_nna_detect) {
        ret = FH_NNA_DET_Forward(detHandle, &nn_out);
        if (ret) {
            printf("Error[FH_NN]: FH_NNA_DET_Forward failed, ret = %x\n", ret);
            usleep(200000);
            continue;
        }

        track_in.num = 0;
        if (nn_out.boxNum > 0) {
            // printf("thread_nn_detect num:%d \n", nn_out.boxNum);
            for (i = 0; i < nn_out.boxNum; i++) {
#if 0
                printf("[%d]nn_detect type:%d, conf:%f, box:x y w h:%f,%f,%f,%f\n", i,
                    nn_out.detBBox[i].clsType,  nn_out.detBBox[i].conf,
                    nn_out.detBBox[i].bbox.x, nn_out.detBBox[i].bbox.y,
                    nn_out.detBBox[i].bbox.w, nn_out.detBBox[i].bbox.h);
#endif
                if (nn_out.detBBox[i].clsType == 1) {
                    if ((nn_out.detBBox[i].bbox.w < 0.1) || (nn_out.detBBox[i].bbox.h < 0.05) ||
                        (nn_out.detBBox[i].bbox.w > 0.9) || (nn_out.detBBox[i].bbox.h > 0.9)) {
                        continue;
                    }

                    if (track_in.num >= MAX_TRACKERS_NUM) {
                        continue;
                    }

                    track_in.item[track_in.num].box.x = nn_out.detBBox[i].bbox.x;
                    track_in.item[track_in.num].box.y = nn_out.detBBox[i].bbox.y;
                    track_in.item[track_in.num].box.width = nn_out.detBBox[i].bbox.w;
                    track_in.item[track_in.num].box.height = nn_out.detBBox[i].bbox.h;
                    track_in.item[track_in.num].priv.type = TKL_MEDIA_DETECT_TYPE_PET;
                    track_in.item[track_in.num].priv.conf = nn_out.detBBox[i].conf;
                    track_in.num++;
                } else {
                    if ((nn_out.detBBox[i].bbox.w < 0.02) || (nn_out.detBBox[i].bbox.h < 0.02) ||
                        (nn_out.detBBox[i].bbox.w > nn_out.detBBox[i].bbox.h && (nn_out.detBBox[i].bbox.w < 0.4)) ||
                        (nn_out.detBBox[i].bbox.w > 0.8 && nn_out.detBBox[i].conf < 0.92)) {
                        continue;
                    }

                    if (track_in.num >= MAX_TRACKERS_NUM) {
                        continue;
                    }

                    track_in.item[track_in.num].box.x = nn_out.detBBox[i].bbox.x;
                    track_in.item[track_in.num].box.y = nn_out.detBBox[i].bbox.y;
                    track_in.item[track_in.num].box.width = nn_out.detBBox[i].bbox.w;
                    track_in.item[track_in.num].box.height = nn_out.detBBox[i].bbox.h;
                    track_in.item[track_in.num].priv.type = TKL_MEDIA_DETECT_TYPE_HUMAN;
                    track_in.item[track_in.num].priv.conf = nn_out.detBBox[i].conf;
                    track_in.num++;
                }
            }
        }
        pthread_mutex_lock(&pdet_handl->lock);
        memset(&pdet_handl->pd_result, 0, sizeof(pdet_handl->pd_result));
        memset(&pdet_handl->hd_result, 0, sizeof(pdet_handl->hd_result));
        __get_result_and_id(&track_in, &pdet_handl->hd_result, &pdet_handl->pd_result);
#ifdef TY_NNA_DRAW_BOX
        //__box_show_objGroup(&hd_in, TKL_MEDIA_DETECT_TYPE_HUMAN);
        __box_show(pdet_handl);
#endif
        pdet_handl->pd_result.timestamp = _fh8636_get_time_ms();
        pdet_handl->hd_result.timestamp = pdet_handl->pd_result.timestamp;
        pthread_mutex_unlock(&pdet_handl->lock);
    }

#ifdef TY_NNA_DRAW_BOX
    clear_draw_gbox_all();
#endif
    pdet_handl->g_nna_running = 0;
    printf("thread_nn_detect end\n");
    // FH_SYS_UnBindbySrc(DRAW_BOX_CHAN);
    // FH_VPSS_CloseChn(DRAW_BOX_CHAN);

    pthread_mutex_lock(&pdet_handl->lock);
    memset(&pdet_handl->pd_result, 0, sizeof(pdet_handl->pd_result));
    memset(&pdet_handl->hd_result, 0, sizeof(pdet_handl->hd_result));
    pthread_mutex_unlock(&pdet_handl->lock);

    return NULL;
}

static int __start_smart_detect(TKL_MEDIA_DETECT_TYPE_E type)
{
    TY_DEV_DETECT_HANDLE_T *pdet_handl = ty_dev_get_detect_handle();
    pdet_handl->flag |= (1 << type);
    // printf("__start_smart_detect:type:%d, flag:%d\n", type, pdet_handl->flag);
    return 0;
}

static int __stop_smart_detect(TKL_MEDIA_DETECT_TYPE_E type)
{
    TY_DEV_DETECT_HANDLE_T *pdet_handl = ty_dev_get_detect_handle();
    pdet_handl->flag &= ~(1 << type);
    // printf("__stop_smart_detect:type:%d, flag:%d\n", type, pdet_handl->flag);
    return 0;
}

int ty_dev_start_smart_detect(int chn, TKL_MEDIA_DETECT_TYPE_E type)
{
    (void)chn;
    TY_DEV_DETECT_HANDLE_T *pdet_handl = ty_dev_get_detect_handle();
    TY_FH8636_VIDEO_HANDLE_T *pvenc_hand = __get_video_handle();
    pthread_t nn_thread;
    pthread_attr_t nn_attr;
    if (pvenc_hand->encParam[TKL_VENC_HD].enable == 0) {
        printf("[FH_NN] hd is not enabled\n");
        return -1;
    }

    if ((type != TKL_MEDIA_DETECT_TYPE_HUMAN) && (type != TKL_MEDIA_DETECT_TYPE_PET)) {
        printf("[FH_NN] %d is not support\n", type);
        return -1;
    }

    pthread_mutex_lock(&pdet_handl->lock_start_stop);
    void *detHandle = pdet_handl->pnna_handle;
    if (detHandle == NULL) {
        printf("[FH_NN] nna isnot init\n");
        pthread_mutex_unlock(&pdet_handl->lock_start_stop);
        return -1;
    }

    if (pdet_handl->g_nna_running) {
        printf("[FH_NN] nna is running\n");
        __start_smart_detect(type);
        pthread_mutex_unlock(&pdet_handl->lock_start_stop);
        return 0;
    }

    pdet_handl->g_stop_nna_detect = 0;

    pthread_attr_init(&nn_attr);
    pthread_attr_setdetachstate(&nn_attr, PTHREAD_CREATE_DETACHED);
    pthread_attr_setstacksize(&nn_attr, 512 * 1024);
    if (pthread_create(&nn_thread, &nn_attr, thread_nn_detect, NULL)) {
        printf("Error[FH_NN]: pthread_create failed\n");
        pthread_attr_destroy(&nn_attr);
        pthread_mutex_unlock(&pdet_handl->lock_start_stop);
        return -1;
    }
    pthread_attr_destroy(&nn_attr);

    pdet_handl->g_nna_running = 1;
    __start_smart_detect(type);
    pthread_mutex_unlock(&pdet_handl->lock_start_stop);

    printf("ty fh nn detect start\n");
    return 0;
}

/***********************************************************
 *  函数名: ty_dev_stop_smart_detect
 *  功能：停止智能检测
 *  参数:  chn 预留，暂未使用
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
int ty_dev_stop_smart_detect(int chn, TKL_MEDIA_DETECT_TYPE_E type)
{
    (void)chn;
    TY_DEV_DETECT_HANDLE_T *pdet_handl = ty_dev_get_detect_handle();

    if ((type != TKL_MEDIA_DETECT_TYPE_HUMAN) && (type != TKL_MEDIA_DETECT_TYPE_PET)) {
        printf("[FH_NN] %d is not support\n", type);
        return -1;
    }

    pthread_mutex_lock(&pdet_handl->lock_start_stop);
    __stop_smart_detect(type);
    if (pdet_handl->flag > 0) {
        printf("flag:%d, return\n", pdet_handl->flag);
        pthread_mutex_unlock(&pdet_handl->lock_start_stop);
        return 0;
    }
    pdet_handl->g_stop_nna_detect = 1;
    while (pdet_handl->g_nna_running) {
        usleep(10);
    }
    pthread_mutex_unlock(&pdet_handl->lock_start_stop);
    return 0;
}

/***********************************************************
 *  函数名: ty_dev_get_detect_result
 *  功能：hd智能检测结果返回
 *  参数:  chn  预留
           type 类型
 *       pResult 检测结果
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
int ty_dev_get_detect_result(int chn, TKL_MEDIA_DETECT_TYPE_E type, TKL_VI_DETECT_RESULT_T *pResult)
{
    (void)chn;
    TY_DEV_DETECT_HANDLE_T *pHadl = ty_dev_get_detect_handle();
    if ((type != TKL_MEDIA_DETECT_TYPE_HUMAN) && (type != TKL_MEDIA_DETECT_TYPE_PET)) {
        printf("[FH_NN] %d is not support\n", type);
        return -1;
    }

    if (pResult == NULL) {
        printf("[FH_NN] pResult is NULL\n");
        return -1;
    }

    if (pHadl->g_nna_running == 1) {
        pthread_mutex_lock(&pHadl->lock);
        if (type == TKL_MEDIA_DETECT_TYPE_PET) {
            memcpy(pResult, &pHadl->pd_result, sizeof(TKL_VI_DETECT_RESULT_T));
        } else if ((type == TKL_MEDIA_DETECT_TYPE_HUMAN)) {
            memcpy(pResult, &pHadl->hd_result, sizeof(TKL_VI_DETECT_RESULT_T));
        } else {
            printf("not support type: %d\n", type);
            pthread_mutex_unlock(&pHadl->lock);
            return -1;
        }
        pthread_mutex_unlock(&pHadl->lock);
    } else {
        memset(pResult, 0, sizeof(TKL_VI_DETECT_RESULT_T));
    }
    return 0;
}

int ty_dev_set_detect_param()
{
    return 0;
}

int ty_fh8636_hd_init()
{
    TY_DEV_DETECT_HANDLE_T *pHadl = ty_dev_get_detect_handle();
    // 初始化锁
    int ret = pthread_mutex_init(&pHadl->lock, NULL);
    if (ret != 0) {
        printf("pthread_mutex_init failed %d\n", ret);
        return -1;
    }

    ret = pthread_mutex_init(&pHadl->lock_start_stop, NULL);
    if (ret != 0) {
        printf("pthread_mutex_init failed %d\n", ret);
        return -1;
    }
    printf("ty_fh8636_hd_init suc\n");
    return 0;
}

// int ty_fh8636_hd_uninit()
// {
//     TY_DEV_DETECT_HANDLE_T *pHadl = ty_dev_get_detect_handle();
//     pthread_mutex_destroy(&pHadl->lock);
//     pthread_mutex_destroy(&pHadl->lock_start_stop);
//     printf("ty_fh8636_hd_uninit suc\n");
//     return 0;
// }

int ty_dev_detect_init(int chn, int type, TKL_VI_DETECT_CONFIG_T *p_config)
{
    (void)chn;
    int ret = -1;
    TY_DEV_DETECT_HANDLE_T *pdet_handl = ty_dev_get_detect_handle();

    if ((type != TKL_MEDIA_DETECT_TYPE_HUMAN) && (type != TKL_MEDIA_DETECT_TYPE_PET)) {
        printf("[FH_NN] %d is not support\n", type);
        return -1;
    }

    if (pdet_handl->pnna_handle != NULL) {
        // printf("[FH_NN]: fh_nn is init,type %d\n", type);
        return 0;
    }

    pdet_handl->pnna_handle = __init_fh_nn(p_config);
    if (pdet_handl->pnna_handle == NULL) {
        printf("Error[FH_NN]: __init_fh_nn failed\n");
        return -1;
    }

    FH_BIND_INFO src, dst;
    src.obj_id = FH_OBJ_VPU_VO;
    src.dev_id = FH_GRP_ID;
    src.chn_id = RGB_CHAN;

    dst.obj_id = FH_OBJ_NN;
    dst.dev_id = NN_CHAN;
    dst.chn_id = 0;
    ret = FH_SYS_Bind(src, dst);
    if (ret != 0) {
        printf("Error[FH_NN]: FH_SYS_BindVpu2Nn fail, ret = %x\n", ret);
        return -1;
    }
    if (pdet_handl->tracker_hdl == NULL) {
        SORT_PARAM trackCfg = {0};
        trackCfg.iou_thrs = 0.1; // 交并比
        trackCfg.min_hits = 2;   // 最小更新的次数
        trackCfg.max_age = 5;    // 保留目标
        int ret = sort_tracker_init(&pdet_handl->tracker_hdl, &trackCfg);
        if (ret != 0) {
            printf("Error sort_tracker_init  fail, ret = %d\n", ret);
            return -1;
        }
    }
    // printf("[FH_NN]: fh_nn init suc,type %d\n", type);
    return 0;
}

int ty_dev_detect_uninit(int chn, int type)
{
    (void)chn;
    TY_DEV_DETECT_HANDLE_T *pHadl = ty_dev_get_detect_handle();

    if ((type != TKL_MEDIA_DETECT_TYPE_HUMAN) && (type != TKL_MEDIA_DETECT_TYPE_PET)) {
        printf("[FH_NN] %d is not support\n", type);
        return -1;
    }

    if (pHadl->pnna_handle == NULL) {
        printf("[FH_NN]: fh_nn is uninit,type %d\n", type);
        return 0;
    }

    pthread_mutex_lock(&pHadl->lock_start_stop);
    pHadl->g_stop_nna_detect = 1;
    while (pHadl->g_nna_running) {
        usleep(10);
    }
    pHadl->flag = 0;
    pthread_mutex_unlock(&pHadl->lock_start_stop);

    __nna_deinit(pHadl->pnna_handle);
    pHadl->pnna_handle = NULL;
    FH_BIND_INFO dst = {0};
    dst.obj_id = FH_OBJ_NN;
    dst.dev_id = NN_CHAN;
    dst.chn_id = 0;
    FH_SYS_UnBindbyDst(dst);
    printf("[FH_NN]: fh_nn uninit suc,type %d\n", type);

    return 0;
}
