#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <pthread.h>
#include <time.h>

#include "tkl_video_enc.h"
#include "tkl_fh8636_osd.h"
#include "tkl_fh8636_video.h"

#include "isp/isp_enum.h"
#include "dsp/fh_system_mpi.h"
#include "dsp/fh_vpu_mpi.h"
#include "dsp/fh_venc_mpi.h"
#include "tkl_fh8636_font.h"
#include "FHAdv_OSD_mpi.h"

// vpu 通道
#define VPU_CHN0 0     // 主码流使用
#define VPU_CHN1 1     // 子码流，抓图使用
#define FH_INITIAL_UTC_TIME         946656000 //2000-01-01 00:00:00

typedef struct ty_fh8636_osd_handle_ {
    FH_BOOL init;
    FH_UINT32 graph_ctrl;
} TY_FH8636_OSD_HANDLE_T;

static TY_FH8636_OSD_HANDLE_T g_osd_handle = {0}; // osd 句柄
static FH_UINT8 g_user_tag_codes[] = {
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', ':', '-' 
};                                                                       /* 设置FHT_OSD_USER1对应的字符集合 */

/**
 * @fn TY_FH8636_OSD_HANDLE_T * ty_fh8636_get_osd_handle()
 * @获取OSD句柄
 * @param[in] 无
 * @retval 视频参数句柄指针
 * @attention 无。
 */
static TY_FH8636_OSD_HANDLE_T *ty_fh8636_get_osd_handle()
{
    return &g_osd_handle;
}

static int32_t fh8636_osd_init(void)
{
    int32_t ret;
    TY_FH8636_OSD_HANDLE_T *pOsdHand = ty_fh8636_get_osd_handle();

    memset(pOsdHand, 0x00, sizeof(TY_FH8636_OSD_HANDLE_T));
    pOsdHand->graph_ctrl |= FHT_OSD_GRAPH_CTRL_TOSD_AFTER_VP;
    pOsdHand->graph_ctrl |= FHT_OSD_GRAPH_CTRL_GBOX_AFTER_VP;
    pOsdHand->graph_ctrl |= FHT_OSD_GRAPH_CTRL_MASK_BEFORE_VP;

    /* 初始化OSD */
    ret = FHAdv_Osd_Init(FH_GRP_ID, FHT_OSD_DEBUG_LEVEL_ERROR, pOsdHand->graph_ctrl, 0, 0);
    TY_CHECK_RETURN(ret, ret);

    return FH_SUCCESS;
}

/* USER_TAG回调函数，作用为告诉ADVAPI库，自定义字符会用到的所有字符*/
static FH_UINT32 fh8636_osd_char_traverse_callback(FH_UINT32 tag_code, FH_UINT32 sequence)
{
    (void)tag_code;
    if (sequence >= sizeof(g_user_tag_codes))
        return 0; /*如果sequence大于等于集合字符数,表示枚举结束,返回0*/

    return (FH_UINT32)(g_user_tag_codes[sequence]); /* 返回字符集合中第sequence个字符的编码 */
}

/* 把8位的ascii字符转换为32位 */
static void fh8636_osd_stringformchartoint(FH_UINT32 *code, FH_CHAR *s, FH_UINT32 cnt)
{
    FH_UINT32 i;
    for (i = 0; i < cnt; i++) {
        code[i] = (FH_UINT32)((FH_UINT8)(s[i]));
    }
}

/* 自定义字符回调函数，自动刷新FHT_OSD_USER1对应的字符内容，显示实时码率，调用频率和时间刷新频率相同 */
static FH_UINT32 fh8636_osd_char_refresh_callback(FH_UINT32 tagCode, FHT_DATE_TIME *fh_time, FH_UINT32 *pCodeArray)
{
    FH_UINT32 cnt = 19;
    (void)tagCode;
    (void)fh_time;
    FH_CHAR data[FHT_OSD_MAX_CHARS_OF_TAG];
    FH_UINT64 currTime = 0;
    struct tm currDate = {0};
    struct tm *p_currDate = NULL;
    int ret = -1;
    TY_FH8636_VIDEO_HANDLE_T * pHandle = __get_video_handle();
    if (pHandle->cb == 0){
        time((time_t *)&currTime);
        p_currDate = localtime((time_t *)&currTime);
        if (p_currDate == NULL) {
            return -1;
        }
    } else {
        ret = pHandle->cb(&currDate);
        if (ret < 0) {
            printf("get struct tm wrong, ret = %d\n", ret);
            return -1;
        }
        p_currDate = &currDate;
    }

    memset(data, 0, FHT_OSD_MAX_CHARS_OF_TAG);
    if (tagCode == 0xe40d) {
        if(p_currDate->tm_year > 2000 - 1900) {    //只显示2000年后时间，tm_year为减去1900的年
            strftime(data, sizeof(data), "%Y-%m-%d %H:%M:%S", p_currDate);
        }
        #if 0
            extern char nn_log_buf[16];
            sprintf(data+strlen(data), "%s", nn_log_buf);
            cnt = 19+16;
        #endif
    } else {
        printf("unknown tag code %x\n", tagCode);
    }
    fh8636_osd_stringformchartoint(pCodeArray, data, cnt);
    /*
     * 注意: 1)
     * 每次调用该函数的返回值必须保持一致,并且不能大于FHT_OSD_MAX_CHARS_OF_TAG,
     *       2)
     * 最多向数组pCodeArray中写入FHT_OSD_MAX_CHARS_OF_TAG个FH_UINT32的值,否则会出现堆栈溢出等问题
     */
    return cnt;
}

static int32_t fh8636_set_channel_tosd(int32_t channel, int enable)
{
    int32_t ret;
    FHT_OSD_CONFIG_t osd_cfg = {0};
    FHT_OSD_TextLine_t text_line_cfg = {0};
    FHT_OSD_Layer_Config_t  pOsdLayerInfo = {0};
    FHT_OSD_Ex_TextLine_t pTextCfg = {0};
    TY_FH8636_VIDEO_HANDLE_T *pVideoHand = __get_video_handle();

    /* 设置字符大小,像素单位 */
    pOsdLayerInfo.osdSize     = pVideoHand->osdParam[channel].fontW;

    /* 设置字符颜色为白色 */
    pOsdLayerInfo.normalColor.fAlpha = 255;
    pOsdLayerInfo.normalColor.fRed   = 255;
    pOsdLayerInfo.normalColor.fGreen = 255;
    pOsdLayerInfo.normalColor.fBlue  = 255;

    /* 设置字符反色颜色为黑色 */
    pOsdLayerInfo.invertColor.fAlpha = 255;
    pOsdLayerInfo.invertColor.fRed   = 0;
    pOsdLayerInfo.invertColor.fGreen = 0;
    pOsdLayerInfo.invertColor.fBlue  = 0;

    /* 设置字符钩边颜色为黑色 */
    pOsdLayerInfo.edgeColor.fAlpha = 255;
    pOsdLayerInfo.edgeColor.fRed   = 0;
    pOsdLayerInfo.edgeColor.fGreen = 0;
    pOsdLayerInfo.edgeColor.fBlue  = 0;

    /* 不显示背景 */
    pOsdLayerInfo.bkgColor.fAlpha = 0;

    /* 不旋转 */
    osd_cfg.osdRotate = 0;

    /* 钩边像素 */
    if (0 == channel) {
       pOsdLayerInfo.edgePixel = 2;
    } else {
       pOsdLayerInfo.edgePixel = 1;
    }

    /* 反色控制 */
    pOsdLayerInfo.osdInvertEnable  = FH_OSD_INVERT_DISABLE; /*disable反色功能*/
    // pOsdLayerInfo.osdInvertEnable  = FH_OSD_INVERT_BY_CHAR; /*以字符为单位进行反色控制*/
    /*osd_cfg.osdInvertEnable  = FH_OSD_INVERT_BY_LINE;*/ /*以行块为单位进行反色控制*/
    pOsdLayerInfo.osdInvertThreshold.high_level = 180;
    pOsdLayerInfo.osdInvertThreshold.low_level  = 160;

    /* 使用时间符号设置自定义时间格式*/
    FH_CHAR text_data[128] = { 
        0xe4, 0x0d, /*FHT_OSD_USER1,此处用于显示日期时间*/
        0,
    };

    osd_cfg.pOsdLayerInfo = &pOsdLayerInfo;
    pOsdLayerInfo.layerFlag = FHT_OSD_LAYER_DEFAULT;
    pOsdLayerInfo.layerId = channel;

    /* 设置text行块个数 */
    osd_cfg.nOsdLayerNum     = 1;
    pOsdLayerInfo.layerStartX = pVideoHand->osdParam[channel].left;
    pOsdLayerInfo.layerStartY = pVideoHand->osdParam[channel].top;
    pOsdLayerInfo.layerMaxWidth = pVideoHand->osdParam[channel].fontW * 19 + 16;
    pOsdLayerInfo.layerMaxHeight = pVideoHand->osdParam[channel].fontH + 16;

    ret = FHAdv_Osd_Ex_SetText(FH_GRP_ID, channel, &osd_cfg);
    if (ret != FH_SUCCESS) {
        if (channel == 0) /*TOSD可能位于分通道前,这样channel 1可能没有,因此就不打印出错*/
        {
            printf("FHAdv_Osd_Ex_SetText failed with %d\n", ret);
        }
        return ret;
    }
    pTextCfg.LineNum = 1;
    pTextCfg.Reconfig = 1;
    pTextCfg.pLineCfg = &text_line_cfg;
    text_line_cfg.textInfo = text_data;
    text_line_cfg.textEnable = enable; /* 使能自定义text */
    text_line_cfg.timeOsdEnable = 0;   /* 去使能时间显示 */
    text_line_cfg.textLineWidth = pVideoHand->osdParam[channel].fontW * 19; /* 每行最多显示19个像素宽度为w的字符 */
    text_line_cfg.linePositionX = pOsdLayerInfo.layerStartX; /* 左上角起始点宽度像素位置 */
    text_line_cfg.linePositionY = pOsdLayerInfo.layerStartY;  /* 左上角起始点高度像素位置 */
    text_line_cfg.lineId = 0;
    text_line_cfg.enable = 1;

    ret = FHAdv_Osd_Ex_SetTextLine(FH_GRP_ID, channel, pOsdLayerInfo.layerId, &pTextCfg);
    if (ret != FH_SUCCESS) {
        if (channel == 0) {/*TOSD可能位于分通道前,这样channel 1可能没有,因此就不打印出错*/
            printf("FHAdv_Osd_Ex_SetTextLine failed with 0x%x\n", ret);
        }
        return ret;
    }

    return 0;
}

// 设置隐私遮挡
int32_t fh8636_set_mask(TKL_VENC_MASK_T *pmask)
{
    int32_t ret = -1;
    TY_FH8636_VIDEO_HANDLE_T *pVideoHand = __get_video_handle();
    int w = pVideoHand->vpuParam[0].chnAttr.vpu_chn_size.u32Width;
    int h = pVideoHand->vpuParam[0].chnAttr.vpu_chn_size.u32Height;

    if (pmask == NULL){
        printf("set mask failed, pmask is NULL\n");
        return -1;
    }

    if (pmask->x < 0 || pmask->x >= 100 || pmask->y < 0 || pmask->y >= 100 || pmask->width <= 0 || pmask->width > 100 ||
        pmask->height <= 0 || pmask->height > 100 || pmask->x + pmask->width > 100 || pmask->y + pmask->height > 100){
        printf("set mask failed, param is wrong: x y width height:%f %f %f %f\n", pmask->x, pmask->y, pmask->width, pmask->height);
        return -1;
    }

    if (pmask->id > 3){
        printf("set mask failed, param is wrong: id:%d\n", pmask->id);
        return -1;
    }

    if (pmask->enable != 0 &&  pmask->enable != 1){
        printf("set mask failed, param is wrong: enable:%d\n", pmask->enable);
        return -1;
    }

    FHT_OSD_Mask_t mask_cfg = {0};
    mask_cfg.enable = pmask->enable;
    mask_cfg.maskId = pmask->id;
    mask_cfg.max_frame_w = w;
    mask_cfg.max_frame_h = h;
    mask_cfg.area.fTopLeftX = pmask->x * w / 100;
    mask_cfg.area.fTopLeftY = pmask->y * h / 100;
    mask_cfg.area.fWidth = pmask->width * w / 100;
    mask_cfg.area.fHeigh = pmask->height * h / 100;

    FH_UINT8 *p = (FH_UINT8 *)&pmask->color;
    mask_cfg.osdColor.fBlue = p[0];
    mask_cfg.osdColor.fGreen = p[1];
    mask_cfg.osdColor.fRed =  p[2];
    mask_cfg.osdColor.fAlpha = p[3];

    printf("set mask[%d] enable:%d x y(%d, %d), w h(%d %d), color:0x%x\n", mask_cfg.maskId, mask_cfg.enable, mask_cfg.area.fTopLeftX,
     mask_cfg.area.fTopLeftY, mask_cfg.area.fWidth, mask_cfg.area.fHeigh, pmask->color);

    ret = FHAdv_Osd_SetMask(FH_GRP_ID, &mask_cfg);
    if (ret != FH_SUCCESS) {
        printf("FHAdv_Osd_Ex_SetMask failed with %d, enable:%d, mask_id:%d\n", ret, mask_cfg.enable,  mask_cfg.maskId);
        return ret;
    }
    return 0;
}

int fh8636_osd_enable(int value)
{
    TY_FH8636_VIDEO_HANDLE_T *pVideoHand = __get_video_handle();
    TY_FH8636_OSD_HANDLE_T *pOsdHand = ty_fh8636_get_osd_handle();
    if (pOsdHand->graph_ctrl & FHT_OSD_GRAPH_CTRL_TOSD_BEFORE_VP) {
        if (pVideoHand->osdParam[VPU_CHN0].enable) {
            fh8636_set_channel_tosd(VPU_CHN0, value);
        }
    } else {
        if (pVideoHand->osdParam[VPU_CHN0].enable) {
            fh8636_set_channel_tosd(VPU_CHN0, value);
        }

        if (pVideoHand->osdParam[VPU_CHN1].enable) {
            fh8636_set_channel_tosd(VPU_CHN1, value);
        }
    }
    return 0;
}

static FH_UINT8* __get_user_font_addr(FH_UINT32 code, FH_UINT32 *pWidth, FH_UINT32 *pHeight)
{
    FH_UINT32 i = 0;
    if (code < 0xff) {
        if (code == '\0') {
            code = ' ';
        }
        for(i = 0; i < sizeof(idxArray) - 1; i++) {
            if(code == idxArray[i]) {
                break;
            }
        }
        if(i == sizeof(idxArray) - 1) {
            printf("__get_user_font_addr not found:%x\n", code);
            return NULL;
        }
        *pWidth = 48;
        *pHeight = 96;
        return fontArray + 96 * 6 * i; 
    }
    printf("__get_user_font_addr code err:%x\n", code);
    return NULL;
}
static int32_t fh8636_osd_tosd(void)
{
    int32_t ret = -1;
#if 1
    /* 注册自定义字库回调函数，此处设置编码为GB2312, 其他编码方式使用方法相同 */
    ret = FHAdv_Osd_Font_RegisterCallback(__get_user_font_addr, FHTEN_OSD_GB2312);
    if (ret != 0) {
        printf("FHAdv_Osd_Font_RegisterCallback failed with %d\n", ret);
        return -1;
    }
#else
    FHT_OSD_FontLib_t font_lib = {0};

    /* 加载asc字库 */
    font_lib.pLibData = asc16;
    font_lib.libSize = sizeof(asc16);
    ret = FHAdv_Osd_LoadFontLib(FHEN_FONT_TYPE_ASC, &font_lib);
    TY_CHECK_RETURN(ret, ret);
#endif
    /* 注册FHT_OSD_USER1的回调函数，其他用户自定义字符和FHT_OSD_USER1设置方式相同
     */
    /* 第二个参数设置该自定义字符的刷新频率,这里为设置为每秒刷新一次*/
    /* 最多注册16个自定义字符 */
    /* 所有使用到的user TAG要先通过函数FHAdv_Osd_SetTagCallback注册好,否则会报错*/
    FHAdv_Osd_SetTagCallback(FH_GRP_ID, FHT_OSD_USER1 + 0, FHT_UPDATED_SECOND, fh8636_osd_char_traverse_callback,
                             fh8636_osd_char_refresh_callback);

    fh8636_osd_enable(true);

#if  0
    ret = FHAdv_Osd_UnLoadFontLib(FHEN_FONT_TYPE_ASC);
    TY_CHECK_RETURN(ret, ret);
#endif

    return FH_SUCCESS;
}

int ty_fh8636_osd_init(void)
{
    int32_t ret = -1;
    TY_FH8636_OSD_HANDLE_T *pOsdHand = ty_fh8636_get_osd_handle();

    if (pOsdHand->init) {
        printf("osd has been inited!\n");
        return 0;
    }

    ret = fh8636_osd_init();
    TY_CHECK_RETURN(ret, ret);

    ret = fh8636_osd_tosd();
    TY_CHECK_RETURN(ret, ret);

    pOsdHand->init = FH_TRUE;

    return FH_SUCCESS;
}

void ty_fh8636_osd_deinit(void)
{
    TY_FH8636_OSD_HANDLE_T *pOsdHand = ty_fh8636_get_osd_handle();

    if (!pOsdHand->init) {
        printf("osd has been deinited!\n");
        return;
    }

    FHAdv_Osd_UnInit(FH_GRP_ID);

    pOsdHand->init = FH_FALSE;

    return;
}
