#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/prctl.h>

#include "tkl_video_in.h"
#include "tkl_video_enc.h"
#include "tkl_fh8636_video.h"
#include "tkl_fh8636_hd.h"

//原厂sdk头文件
#include "isp_api.h"
// #include "FHAdv_Isp_mpi_v2.h"
#include "fh_system_mpi.h"
#include "fh_vpu_mpi.h"
#include "fh_venc_mpi.h"

static TY_FH8636_VI_HANDLE_T g_vi_handle;

TY_FH8636_VI_HANDLE_T *tkl_vi_get_handle()
{
    return &g_vi_handle;
}

/***********************************************************
 *  函数名: tkl_vi_init
 *  功能：初始化函数
 *  参数: Input，TKL_VI_CONFIG_S *pconfig初始化参数
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_vi_init(TKL_VI_CONFIG_T *pconfig, int count)
{
    int ret = -1;
    int chn = 0;

    if (pconfig == NULL || count < 1) {
        printf("parm is wrong\n");
        return OPRT_COM_ERROR;
    }

    TKL_ISP_PARAM_INFO_T isp_info = {0};
    memcpy(&isp_info.isp_dn_switch_config, &pconfig->isp.isp_dn_switch_config, sizeof(TKL_ISP_DN_SWITCH_CONFIG_T));
    ret = tkl_vi_isp_init(chn, &isp_info);
    if (ret < 0) {
        printf("tkl_vi_isp_init failed\n");
        return -1;
    }

    ret = tkl_vi_isp_show_fw_version(pconfig->isp.version, pconfig->isp.name);
    if (ret < 0) {
        printf("tkl_vi_isp_show_fw_version failed\n");
    }

    memcpy(&g_vi_handle.config, pconfig, sizeof(TKL_VI_CONFIG_T));

    TY_FH8636_VIDEO_HANDLE_T *pHand = __get_video_handle();
    pHand->viParam.viSize.vi_size.u32Width = pconfig->isp.width;
    pHand->viParam.viSize.vi_size.u32Height = pconfig->isp.height;
    if (pconfig->isp.width == 1920 && pconfig->isp.height == 1080) {
        pHand->viParam.ispFormat = FORMAT_1080P20;
    } else if (pconfig->isp.width == 2560 && pconfig->isp.height == 1440) {
        pHand->viParam.ispFormat = FORMAT_400WP20;
    } else if (pconfig->isp.width == 2304 && pconfig->isp.height == 1296) {
        pHand->viParam.ispFormat = FORMAT_2304X1296P20;
    } else {
        printf("not support width and height:%d %d\n", pconfig->isp.width, pconfig->isp.height);
        return -1;
    }

    printf("********tkl_vi_init********\n");

    return 0;
}

/***********************************************************
 *  函数名: tkl_vi_set_mirror_flip
 *  功能：设置镜像翻转
 *  参数: Input，chn
 *        Input，flag 镜像翻转值
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_vi_set_mirror_flip(TKL_VI_CHN_E chn, TKL_VI_MIRROR_FLIP_E flag)
{
    INT32_T ret = 0;
    int bFilp = 0;
    int bMirror = 0;
    int bSensorMirror = g_vi_handle.config.mirror;
    int bSensorFilp = g_vi_handle.config.filp;

    switch (flag) {
    case TKL_VI_MIRROR_FLIP_NONE:
        bFilp = bSensorFilp;
        bMirror = bSensorMirror;
        break;
    case TKL_VI_MIRROR:
        bFilp = bSensorFilp;
        bMirror = (1 ^ bSensorMirror);
        break;
    case TKL_VI_FLIP:
        bFilp = (1 ^ bSensorFilp);
        bMirror = bSensorMirror;
        break;
    case TKL_VI_MIRROR_FLIP:
        bFilp = (1 ^ bSensorFilp);
        bMirror = (1 ^ bSensorMirror);
        break;
    default:
        printf("invlaid mirrorflip settting:%d\n", flag);
        return -1;
    }

    ret = tkl_vi_isp_set_mirror_flip(chn, bMirror, bFilp);
    if (ret < 0) {
        printf("tkl_vi_isp_set_mirror_flip failed\n");
        return -1;
    }

    return 0;
}

/***********************************************************
 *  函数名: tkl_vi_get_mirror_flip
 *  功能：获取镜像翻转
 *  参数: Input，chn
 *        Output，flag 镜像翻转值
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_vi_get_mirror_flip(TKL_VI_CHN_E chn, TKL_VI_MIRROR_FLIP_E *flag)
{
    INT32_T ret = 0;
    int bFilp = 0;
    int bMirror = 0;
    ret = tkl_vi_isp_get_mirror_flip(chn, &bMirror, &bFilp);
    if (ret < 0) {
        printf("tkl_vi_isp_get_mirror_flip failed\n");
        return -1;
    }

    int cfg_mirror = g_vi_handle.config.mirror;
    int cfg_filp = g_vi_handle.config.filp;

    if (bMirror == cfg_mirror && bFilp == cfg_filp) {
        *flag = TKL_VI_MIRROR_FLIP_NONE;
    } else if (bMirror == (1 ^ cfg_mirror) && bFilp == cfg_filp) {
        *flag = TKL_VI_MIRROR;
    } else if (bMirror == cfg_mirror && bFilp == (1 ^ cfg_filp)) {
        *flag = TKL_VI_FLIP;
    } else {
        *flag = TKL_VI_MIRROR_FLIP;
    }

    return 0;
}

/***********************************************************
 *  函数名: ty_tkl_vi_uninit
 *  功能：去初始化
 *  参数: 无
 *
 *  返回值: <0 失败，=0 成功
 ***********************************************************/
OPERATE_RET tkl_vi_uninit(void)
{
    return 0;
}

/**
 * @brief  set sensor reg value
 *
 * @param[in] chn: vi chn
 * @param[in] pparam: reg info
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tkl_vi_sensor_reg_set(TKL_VI_CHN_E chn, TKL_VI_SENSOR_REG_CONFIG_T *parg)
{
    (void)chn;
    (void)parg;
    return -1;
}

/**
 * @brief  get sensor reg value
 *
 * @param[in] chn: vi chn
 * @param[in] pparam: reg info
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tkl_vi_sensor_reg_get(TKL_VI_CHN_E chn, TKL_VI_SENSOR_REG_CONFIG_T *parg)
{
    (void)chn;
    (void)parg;
    return -1;
}

/**
 * @brief detect start
 *
 * @param[in] chn: vi chn
 * @param[in] type: detect type
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tkl_vi_detect_start(TKL_VI_CHN_E chn, TKL_MEDIA_DETECT_TYPE_E type)
{
    printf("tkl_vi_detect_start %d %d\n", chn, type);
    return ty_dev_start_smart_detect(chn, type);
}

/**
 * @brief detect stop
 *
 * @param[in] chn: vi chn
 * @param[in] type: detect type
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tkl_vi_detect_stop(TKL_VI_CHN_E chn, TKL_MEDIA_DETECT_TYPE_E type)
{
    printf("tkl_vi_detect_stop %d %d\n", chn, type);
    return ty_dev_stop_smart_detect(chn, type);
}

/**
 * @brief get detection results
 *
 * @param[in] chn: vi chn
 * @param[in] type: detect type
 * @param[out] presult: detection results
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tkl_vi_detect_get_result(TKL_VI_CHN_E chn, TKL_MEDIA_DETECT_TYPE_E type, TKL_VI_DETECT_RESULT_T *presult)
{
    return ty_dev_get_detect_result(chn, type, presult);
}

/**
 * @brief set detection param
 *
 * @param[in] chn: vi chn
 * @param[in] type: detect type
 * @param[in] pparam: detection param
 *
 * @return OPRT_OK on success. Others on error, please refer to tkl_error_code.h
 */
OPERATE_RET tkl_vi_detect_set(TKL_VI_CHN_E chn, TKL_MEDIA_DETECT_TYPE_E type, TKL_VI_DETECT_PARAM_T *pparam)
{
    (void)chn;
    (void)type;
    (void)pparam;
    return 0;
}

OPERATE_RET tkl_vi_detect_init(TKL_VI_CHN_E chn, TKL_MEDIA_DETECT_TYPE_E type, TKL_VI_DETECT_CONFIG_T *pconfig)
{
    printf("tkl_vi_detect_init %d %d\n", chn, type);
    return ty_dev_detect_init(chn, type, pconfig);
}

OPERATE_RET tkl_vi_detect_uninit(TKL_VI_CHN_E chn, TKL_MEDIA_DETECT_TYPE_E type)
{
    printf("tkl_vi_detect_uninit %d %d\n", chn, type);
    return ty_dev_detect_uninit(chn, type);
}
